searchData={"content_type":"text/markdown","items":[{"doc":"Definition of macro to make mapping of asset URLs from their file paths.\n\nIf a gear invokes `static_prefix/0` in its `Router` module, antikythera automatically serves static assets\n(such as HTML, CSS, JS, images) under `priv/static/` directory (see `Antikythera.Router` for detail).\nAlthough this is handy it's suboptimal in cloud environments.\nInstead, in cloud environments, static assets can be delivered from CDN service.\nUsing CDN for assets has the following benefits:\n\n- faster page load\n- less resource consumption (CPU, network bandwidth, etc.) in both ErlangVMs and load balancers\n- more fine-grained control on asset versions\n\nTo keep track of modifications in asset files, we use MD5 digest of file contents:\ndownload URLs contain MD5 hashes in their paths.\nThis way we can serve multiple versions of a single asset file.\n\nWhen a gear is deployed, antikythera's auto-deploy script uploads each asset to cloud storage\n(if the file has modification since last deploy) so that they are readily served by CDN.\nEach asset is served in gzip-compressed format if its file extension indicates that gzip compression can be beneficial.\nAlso, assets are configured to accept CORS with e.g. `XMLHttpRequest`.\nWhen downloaded, each asset comes with a relatively long `max-age` in `cache-control` response header\nso that the data can be reused as long as its content (and thus MD5 digest) does not change.\n\nThis module provides `__using__/2` macro that\n\n- enables asset serving via CDN\n- generates functions that absorb differences in asset download URLs","ref":"Antikythera.Asset.html","title":"Antikythera.Asset","type":"module"},{"doc":"Define a module that `use`s `Antikythera.Asset` as follows:\n\n    defmodule YourGear.Asset do\n      use Antikythera.Asset\n    end\n\nThen `YourGear.Asset` has the following functions.\n\n- `url/1` : given a file path relative to `priv/static/` directory, returns a URL to download the file\n- `all/0` : returns all pairs of file path and download URL as a map\n\nIf you want antikythera's auto-deploy script to perform asset preparation\nBEFORE compilations of your gear (including `YourGear.Asset` module),\nset up asset preparation methods as described in `Mix.Tasks.Antikythera.PrepareAssets`.\n\nYou can now freely use the module for e.g.\n\n- HAML templates that refers to asset files\n- WebAPI that returns the URLs of the latest assets in your preferred format (JSON, javascript, etc.)\n\nWith `YourGear.Asset` as above, antikythera's auto-deploy script uploads asset files to cloud storage\nso that they are readily served by CDN.","ref":"Antikythera.Asset.html#module-usage","title":"Usage - Antikythera.Asset","type":"module"},{"doc":"If you are using tools such as [webpack-dev-server](https://github.com/webpack/webpack-dev-server)\nduring development, you may want your browser to download assets from a web server other than locally-running antikythera.\nIn this case, pass `:base_url_during_development` option to `__using__/2` so that `url/1` and `all/0`\nreturn URLs that point to the specified endpoint.","ref":"Antikythera.Asset.html#module-asset-urls-during-development","title":"Asset URLs during development - Antikythera.Asset","type":"module"},{"doc":"An asset file uploaded to cloud storage during deployment of a certain gear version\nbecomes \"obsolete\" when a newly-deployed gear version no longer uses it\n(the file has been either modified, removed or renamed).\n\nAntikythera keeps track of whether each asset file is obsolete or not.\nThen, antikythera periodically removes asset files\nthat have been obsolete for more than 90 days.\nThe retention period (chosen such that each period includes several deployments of each gear)\nshould be long enough for clients to switch from older assets to newer ones.","ref":"Antikythera.Asset.html#module-retention-cleanup-policy-for-asset-files","title":"Retention/cleanup policy for asset files - Antikythera.Asset","type":"module"},{"doc":"","ref":"Antikythera.Asset.html#list_asset_file_paths/0","title":"Antikythera.Asset.list_asset_file_paths/0","type":"function"},{"doc":"","ref":"Antikythera.Asset.html#retention_days/0","title":"Antikythera.Asset.retention_days/0","type":"function"},{"doc":"Antikythera's \"async job\" functionality allows gears to run their code in background.\n\nThis module is the interface for gear implementations to define and register their own async jobs.","ref":"Antikythera.AsyncJob.html","title":"Antikythera.AsyncJob","type":"behaviour"},{"doc":"#","ref":"Antikythera.AsyncJob.html#module-usage","title":"Usage - Antikythera.AsyncJob","type":"behaviour"},{"doc":"Each gear can define multiple async job modules.\nThose modules must `use` this module as in the following example:\n\n    defmodule YourGear.SomeAsyncJob do\n      use Antikythera.AsyncJob\n\n      # run/3 callback is required\n      @impl true\n      def run(_payload, _metadata, _context) do\n        # do something here\n      end\n\n      # abandon/3 callback is optional; you can omit it\n      @impl true\n      def abandon(_payload, _metadata, _context) do\n        # cleanup code when all retries failed\n      end\n\n      # inspect_payload/1 callback is optional; you can omit it\n      @impl true\n      def inspect_payload(payload) do\n        # convert a payload to a short string that can be used to identify a job in logs\n      end\n    end\n\nImplementations of `run/3` can return any value; it is simply ignored.\nIf execution of `run/3` terminated abnormally (exception, timeout, etc.) the job is regarded as failed.\nFailed jobs are automatically retried up to a specified number of attempts (see below).\n\n`abandon/3` optional callback is called when a job is abandoned after all attempts failed.\nYou can put your cleanup logic in this callback when e.g. you use external storage system to store job information.\nNote that `abandon/3` must finish within `10` seconds;\nwhen it takes longer, antikythera stops the execution of `abandon/3` in the middle.\n\n`inspect_payload/1` optional callback is solely for logging purpose.\nBy providing a concrete callback implementation you can additionally include\nsummary of each jobs's payload into logs.\nAs an example, suppose your `payload` contains \"id\" field; then it's natural to define\nthe following `inspect_payload/1`:\n\n    def inspect_payload(%{\"id\" => id}) do\n      id\n    end\n\nYour gear can have multiple modules that `use Antikythera.AsyncJob`.\n\n#","ref":"Antikythera.AsyncJob.html#module-preparing-your-async-job-module","title":"Preparing your async job module - Antikythera.AsyncJob","type":"behaviour"},{"doc":"With the above module defined, you can call `register/3` to invoke a new job:\n\n    YourGear.SomeAsyncJob.register(%{\"arbitrary\" => \"map\"}, context, options)\n\nHere first argument is an arbitrary map that is passed to `run/3` callback implementation\n(note that structs are maps and thus usable as payloads).\n`context` is a value of `t:Antikythera.Context.t/0` and is used to obtain\nthe executor pool which this job is registered to.\nWhen you need to register a job to an executor pool that is not the current one,\nyou can pass a `t:Antikythera.ExecutorPool.Id.t/0` instead of `t:Antikythera.Context.t/0`.\n`options` must be a `t:Keyword.t/0` which can include the following values:\n\n- `id`: An ID of the job.\n  If given it must match the regex pattern `~r/\\A[0-9A-Za-z_-]{1,32}\\z/` and must be\n  unique in the job queue specified by the second argument.\n  If not given antikythera automatically generates one for you.\n- `schedule`: When to run the job as a 2-tuple.\n  If not given it defaults to `{:once, Antikythera.Time.now()}`, i.e.,\n  the job will be executed as soon as an available worker process is found in the specified executor pool.\n  Allowed value format is:\n    - `{:once, Antikythera.Time.t}`\n        - The job is executed at the given time.\n          After the job is either successfully completed or abandoned by failure(s), the job is removed from the job queue.\n          The time must be a future time and within 50 days from now.\n    - `{:cron, Antikythera.Cron.t}`\n        - The job is repeatedly executed at the given cron schedule.\n          After the job is either successfully completed or abandoned by failure(s),\n          the job is requeued to the job queue.\n          Note that next start time is computed from the time of requeuing.\n          For example, if a job is scheduled on every 10 minutes (\"*/10 * * * *\")\n          and its execution takes 15 minutes to complete,\n          then the job will in effect run on every 20 minutes.\n          The schedule will repeat indefinitely; when you have done with the job\n          you can remove it by `cancel/3`.\n- `max_duration`: Maximum execution time (in milliseconds) of the job.\n  A job which has been running for more than `max_duration` is brutally killed and\n  if it has remaining attempts it will be retried.\n  Defaults to `300000` (`5` minutes).\n  If explicitly given it cannot exceed `1800000` (`30` minutes).\n- `attempts`: A positive integer within `1..10`), up to which antikythera tries to run the job.\n  Defaults to `3`.\n- `retry_interval`: A 2-tuple of integer and float to calculate time interval between retries.\n  Defaults to `{5000, 2.0}`.\n  First element is a time interval (in milliseconds) between 1st and 2nd attempts\n  and must be within `0..300000`.\n  Second element is a multiplying factor for exponential backoff\n  and must be within `1.0..5.0`.\n  For example:\n    - When `retry_interval: {10_000, 2.0}` is given,\n        - 2nd attempt (1st retry) after failure of 1st attempt is started in `10`s,\n        - 3rd attempt (2nd retry) after failure of 2nd attempt is started in `20`s,\n        - 4th attempt (3rd retry) after failure of 3rd attempt is started in `40`s,\n        - and so on.\n    - If you want to set constant interval, specify `1.0` to second element.\n- `bypass_job_queue`: Whether or not to run the job immediately, skipping addition to the job queue.\n  Defaults to `false`.\n    - When this option is `true`, `register/3` tries to pick a worker process in the executor pool in the same node\n      and then immediately start the job using the worker process.\n      If all processes in the local executor pool are busy, `register/3` returns `{:error, :no_available_workers}`.\n      In that case you have 2 options:\n        - give up running the job, or\n        - call `register/3` again without `bypass_job_queue` option so that the job\n          will be executed afterward, possibly in another node.\n    - Execution with `bypass_job_queue` option takes advantage of being free from rate limiting (see below)\n      and having no overhead with pushing to the distributed job queue.\n      Please note that if you bypass the job queue you cannot\n        - specify start time of the job (it's started immediately and only once), and\n        - the job is not retried when its execution results in a failure.\n    - Therefore this option cannot be used with `schedule`, `attempts` or `retry_interval`.\n      Note also that jobs running with `bypass_job_queue: true` cannot be inspected\n      by `Antikythera.AsyncJob.list/1` or `Antikythera.AsyncJob.status/2`.\n\n`register/3` returns a tuple of `{:ok, Antikythera.AsyncJob.Id.t}` on success.\nYou can register jobs up to 1000.\nWhen 1000 jobs remain unfinished in the job queue, trying to register a new job results in `{:error, :full}`.\n\nThe example call to `register/3` above will eventually invoke `YourGear.SomeAsyncJob.run/3`\nwith the given payload and options.\n\n#","ref":"Antikythera.AsyncJob.html#module-registering-jobs","title":"Registering jobs - Antikythera.AsyncJob","type":"behaviour"},{"doc":"Antikythera's async job queues are distributed, fault tolerant, persistent data structures.\nThese features come at an extra cost: interacting with a job queue is a relatively expensive operation.\nIn order not to overwhelm a job queue, accesses to a job queue are rate-limited by\nthe [token bucket algorithm](https://en.wikipedia.org/wiki/Token_bucket).\nRate limiting is imposed on a per-node basis; in each node, there exists a bucket per job queue.\n\nEach bucket contains up to 30 tokens and\none token is generated in every 500 milliseconds.\nThe following operations take tokens from the corresponding bucket:\n\n- 3 tokens per `register/3`\n- 3 tokens per `cancel/3`\n- 1 token per `status/2`\n- 1 token per `list/1`\n\nWhen there are not enough tokens in the bucket, `register/3` and `cancel/3` return\n`{:error, {:rate_limit_reached, milliseconds_to_wait}}`.\nOn the other hand `status/2` and `list/1` automatically wait and retry several times\nwhen hitting the rate limit.\n\nDuring testing, the rate limiting feature may disrupt smooth test executions.\nTo bypass rate limiting in your tests you can use `Antikythera.Test.AsyncJobHelper.reset_rate_limit_status/1`.\n\n#","ref":"Antikythera.AsyncJob.html#module-rate-limiting-of-accesses-to-each-job-queue","title":"Rate limiting of accesses to each job queue - Antikythera.AsyncJob","type":"behaviour"},{"doc":"Payloads are registered by calling `register/3` and used in `run/3`, `abandon/3` or `inspect_payload/1`.\nPayloads are compressed as binary when `register/3` is called and kept in memory\nuntil job executions, so large payloads could degrade overall system performance.\nPlease avoid including large data in payloads; instead put ID of the data in payload\nand fetch the whole data in run/3.","ref":"Antikythera.AsyncJob.html#module-payload","title":"Payload - Antikythera.AsyncJob","type":"behaviour"},{"doc":"","ref":"Antikythera.AsyncJob.html#c:abandon/3","title":"Antikythera.AsyncJob.abandon/3","type":"callback"},{"doc":"Cancels an async job registered with the job queue specified by `context_or_epool_id`.\n\nNote that currently-running job executions cannot be cancelled.\nHowever, calling `cancel/2` with currently running job's `job_id` prevents retries of the job when it fails.","ref":"Antikythera.AsyncJob.html#cancel/3","title":"Antikythera.AsyncJob.cancel/3","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.html#c:inspect_payload/1","title":"Antikythera.AsyncJob.inspect_payload/1","type":"callback"},{"doc":"Retrieves list of async jobs registered with the job queue specified by `context_or_epool_id`.\n\nEach element of returned list is a 3-tuple: scheduled execution time, job ID and current status.\nThe returned list is already sorted.","ref":"Antikythera.AsyncJob.html#list/1","title":"Antikythera.AsyncJob.list/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.html#c:run/3","title":"Antikythera.AsyncJob.run/3","type":"callback"},{"doc":"Retrieves detailed information of an async job registered with the job queue specified by `context_or_epool_id`.","ref":"Antikythera.AsyncJob.html#status/2","title":"Antikythera.AsyncJob.status/2","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.html#t:option/0","title":"Antikythera.AsyncJob.option/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.Attempts.html","title":"Antikythera.AsyncJob.Attempts","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.Attempts.html#default/0","title":"Antikythera.AsyncJob.Attempts.default/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Attempts.html#max/0","title":"Antikythera.AsyncJob.Attempts.max/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Attempts.html#min/0","title":"Antikythera.AsyncJob.Attempts.min/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Attempts.html#valid?/1","title":"Antikythera.AsyncJob.Attempts.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Attempts.html#t:t/0","title":"Antikythera.AsyncJob.Attempts.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.Id.html","title":"Antikythera.AsyncJob.Id","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.Id.html#generate/0","title":"Antikythera.AsyncJob.Id.generate/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Id.html#pattern/0","title":"Antikythera.AsyncJob.Id.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Id.html#valid?/1","title":"Antikythera.AsyncJob.Id.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Id.html#t:t/0","title":"Antikythera.AsyncJob.Id.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.MaxDuration.html","title":"Antikythera.AsyncJob.MaxDuration","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.MaxDuration.html#default/0","title":"Antikythera.AsyncJob.MaxDuration.default/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.MaxDuration.html#max/0","title":"Antikythera.AsyncJob.MaxDuration.max/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.MaxDuration.html#min/0","title":"Antikythera.AsyncJob.MaxDuration.min/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.MaxDuration.html#valid?/1","title":"Antikythera.AsyncJob.MaxDuration.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.MaxDuration.html#t:t/0","title":"Antikythera.AsyncJob.MaxDuration.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.Metadata.html","title":"Antikythera.AsyncJob.Metadata","type":"module"},{"doc":"Type-aware getter for attempts.","ref":"Antikythera.AsyncJob.Metadata.html#attempts/1","title":"Antikythera.AsyncJob.Metadata.attempts/1","type":"function"},{"doc":"Type-aware setter for attempts.","ref":"Antikythera.AsyncJob.Metadata.html#attempts/2","title":"Antikythera.AsyncJob.Metadata.attempts/2","type":"function"},{"doc":"Type-aware getter for id.","ref":"Antikythera.AsyncJob.Metadata.html#id/1","title":"Antikythera.AsyncJob.Metadata.id/1","type":"function"},{"doc":"Type-aware setter for id.","ref":"Antikythera.AsyncJob.Metadata.html#id/2","title":"Antikythera.AsyncJob.Metadata.id/2","type":"function"},{"doc":"Type-aware getter for max_duration.","ref":"Antikythera.AsyncJob.Metadata.html#max_duration/1","title":"Antikythera.AsyncJob.Metadata.max_duration/1","type":"function"},{"doc":"Type-aware setter for max_duration.","ref":"Antikythera.AsyncJob.Metadata.html#max_duration/2","title":"Antikythera.AsyncJob.Metadata.max_duration/2","type":"function"},{"doc":"Creates a new instance of Antikythera.AsyncJob.Metadata by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.AsyncJob.Metadata.html#new/1","title":"Antikythera.AsyncJob.Metadata.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.AsyncJob.Metadata.html#new!/1","title":"Antikythera.AsyncJob.Metadata.new!/1","type":"function"},{"doc":"Type-aware getter for remaining_attempts.","ref":"Antikythera.AsyncJob.Metadata.html#remaining_attempts/1","title":"Antikythera.AsyncJob.Metadata.remaining_attempts/1","type":"function"},{"doc":"Type-aware setter for remaining_attempts.","ref":"Antikythera.AsyncJob.Metadata.html#remaining_attempts/2","title":"Antikythera.AsyncJob.Metadata.remaining_attempts/2","type":"function"},{"doc":"Type-aware getter for retry_interval.","ref":"Antikythera.AsyncJob.Metadata.html#retry_interval/1","title":"Antikythera.AsyncJob.Metadata.retry_interval/1","type":"function"},{"doc":"Type-aware setter for retry_interval.","ref":"Antikythera.AsyncJob.Metadata.html#retry_interval/2","title":"Antikythera.AsyncJob.Metadata.retry_interval/2","type":"function"},{"doc":"Type-aware getter for run_at.","ref":"Antikythera.AsyncJob.Metadata.html#run_at/1","title":"Antikythera.AsyncJob.Metadata.run_at/1","type":"function"},{"doc":"Type-aware setter for run_at.","ref":"Antikythera.AsyncJob.Metadata.html#run_at/2","title":"Antikythera.AsyncJob.Metadata.run_at/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.AsyncJob.Metadata with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.AsyncJob.Metadata.html#update/2","title":"Antikythera.AsyncJob.Metadata.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.AsyncJob.Metadata.html#update!/2","title":"Antikythera.AsyncJob.Metadata.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.AsyncJob.Metadata.html#valid?/1","title":"Antikythera.AsyncJob.Metadata.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Metadata.html#t:t/0","title":"Antikythera.AsyncJob.Metadata.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.html","title":"Antikythera.AsyncJob.RetryInterval","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.html#default/0","title":"Antikythera.AsyncJob.RetryInterval.default/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.html#interval/2","title":"Antikythera.AsyncJob.RetryInterval.interval/2","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.html#valid?/1","title":"Antikythera.AsyncJob.RetryInterval.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.html#t:t/0","title":"Antikythera.AsyncJob.RetryInterval.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Base.html","title":"Antikythera.AsyncJob.RetryInterval.Base","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Base.html#default/0","title":"Antikythera.AsyncJob.RetryInterval.Base.default/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Base.html#max/0","title":"Antikythera.AsyncJob.RetryInterval.Base.max/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Base.html#min/0","title":"Antikythera.AsyncJob.RetryInterval.Base.min/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Base.html#valid?/1","title":"Antikythera.AsyncJob.RetryInterval.Base.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Base.html#t:t/0","title":"Antikythera.AsyncJob.RetryInterval.Base.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Factor.html","title":"Antikythera.AsyncJob.RetryInterval.Factor","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Factor.html#default/0","title":"Antikythera.AsyncJob.RetryInterval.Factor.default/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Factor.html#max/0","title":"Antikythera.AsyncJob.RetryInterval.Factor.max/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Factor.html#min/0","title":"Antikythera.AsyncJob.RetryInterval.Factor.min/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Factor.html#valid?/1","title":"Antikythera.AsyncJob.RetryInterval.Factor.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.RetryInterval.Factor.html#t:t/0","title":"Antikythera.AsyncJob.RetryInterval.Factor.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.Schedule.html","title":"Antikythera.AsyncJob.Schedule","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.Schedule.html#valid?/1","title":"Antikythera.AsyncJob.Schedule.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Schedule.html#t:t/0","title":"Antikythera.AsyncJob.Schedule.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.StateLabel.html","title":"Antikythera.AsyncJob.StateLabel","type":"module"},{"doc":"","ref":"Antikythera.AsyncJob.StateLabel.html#new/1","title":"Antikythera.AsyncJob.StateLabel.new/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.StateLabel.html#new!/1","title":"Antikythera.AsyncJob.StateLabel.new!/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.StateLabel.html#valid?/1","title":"Antikythera.AsyncJob.StateLabel.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.StateLabel.html#values/0","title":"Antikythera.AsyncJob.StateLabel.values/0","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.StateLabel.html#t:t/0","title":"Antikythera.AsyncJob.StateLabel.t/0","type":"type"},{"doc":"","ref":"Antikythera.AsyncJob.Status.html","title":"Antikythera.AsyncJob.Status","type":"module"},{"doc":"Type-aware getter for attempts.","ref":"Antikythera.AsyncJob.Status.html#attempts/1","title":"Antikythera.AsyncJob.Status.attempts/1","type":"function"},{"doc":"Type-aware setter for attempts.","ref":"Antikythera.AsyncJob.Status.html#attempts/2","title":"Antikythera.AsyncJob.Status.attempts/2","type":"function"},{"doc":"Type-aware getter for gear_name.","ref":"Antikythera.AsyncJob.Status.html#gear_name/1","title":"Antikythera.AsyncJob.Status.gear_name/1","type":"function"},{"doc":"Type-aware setter for gear_name.","ref":"Antikythera.AsyncJob.Status.html#gear_name/2","title":"Antikythera.AsyncJob.Status.gear_name/2","type":"function"},{"doc":"Type-aware getter for id.","ref":"Antikythera.AsyncJob.Status.html#id/1","title":"Antikythera.AsyncJob.Status.id/1","type":"function"},{"doc":"Type-aware setter for id.","ref":"Antikythera.AsyncJob.Status.html#id/2","title":"Antikythera.AsyncJob.Status.id/2","type":"function"},{"doc":"Type-aware getter for max_duration.","ref":"Antikythera.AsyncJob.Status.html#max_duration/1","title":"Antikythera.AsyncJob.Status.max_duration/1","type":"function"},{"doc":"Type-aware setter for max_duration.","ref":"Antikythera.AsyncJob.Status.html#max_duration/2","title":"Antikythera.AsyncJob.Status.max_duration/2","type":"function"},{"doc":"Type-aware getter for module.","ref":"Antikythera.AsyncJob.Status.html#module/1","title":"Antikythera.AsyncJob.Status.module/1","type":"function"},{"doc":"Type-aware setter for module.","ref":"Antikythera.AsyncJob.Status.html#module/2","title":"Antikythera.AsyncJob.Status.module/2","type":"function"},{"doc":"Creates a new instance of Antikythera.AsyncJob.Status by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.AsyncJob.Status.html#new/1","title":"Antikythera.AsyncJob.Status.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.AsyncJob.Status.html#new!/1","title":"Antikythera.AsyncJob.Status.new!/1","type":"function"},{"doc":"Type-aware getter for payload.","ref":"Antikythera.AsyncJob.Status.html#payload/1","title":"Antikythera.AsyncJob.Status.payload/1","type":"function"},{"doc":"Type-aware setter for payload.","ref":"Antikythera.AsyncJob.Status.html#payload/2","title":"Antikythera.AsyncJob.Status.payload/2","type":"function"},{"doc":"Type-aware getter for remaining_attempts.","ref":"Antikythera.AsyncJob.Status.html#remaining_attempts/1","title":"Antikythera.AsyncJob.Status.remaining_attempts/1","type":"function"},{"doc":"Type-aware setter for remaining_attempts.","ref":"Antikythera.AsyncJob.Status.html#remaining_attempts/2","title":"Antikythera.AsyncJob.Status.remaining_attempts/2","type":"function"},{"doc":"Type-aware getter for retry_interval.","ref":"Antikythera.AsyncJob.Status.html#retry_interval/1","title":"Antikythera.AsyncJob.Status.retry_interval/1","type":"function"},{"doc":"Type-aware setter for retry_interval.","ref":"Antikythera.AsyncJob.Status.html#retry_interval/2","title":"Antikythera.AsyncJob.Status.retry_interval/2","type":"function"},{"doc":"Type-aware getter for schedule.","ref":"Antikythera.AsyncJob.Status.html#schedule/1","title":"Antikythera.AsyncJob.Status.schedule/1","type":"function"},{"doc":"Type-aware setter for schedule.","ref":"Antikythera.AsyncJob.Status.html#schedule/2","title":"Antikythera.AsyncJob.Status.schedule/2","type":"function"},{"doc":"Type-aware getter for start_time.","ref":"Antikythera.AsyncJob.Status.html#start_time/1","title":"Antikythera.AsyncJob.Status.start_time/1","type":"function"},{"doc":"Type-aware setter for start_time.","ref":"Antikythera.AsyncJob.Status.html#start_time/2","title":"Antikythera.AsyncJob.Status.start_time/2","type":"function"},{"doc":"Type-aware getter for state.","ref":"Antikythera.AsyncJob.Status.html#state/1","title":"Antikythera.AsyncJob.Status.state/1","type":"function"},{"doc":"Type-aware setter for state.","ref":"Antikythera.AsyncJob.Status.html#state/2","title":"Antikythera.AsyncJob.Status.state/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.AsyncJob.Status with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.AsyncJob.Status.html#update/2","title":"Antikythera.AsyncJob.Status.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.AsyncJob.Status.html#update!/2","title":"Antikythera.AsyncJob.Status.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.AsyncJob.Status.html#valid?/1","title":"Antikythera.AsyncJob.Status.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.AsyncJob.Status.html#t:t/0","title":"Antikythera.AsyncJob.Status.t/0","type":"type"},{"doc":"This module provides functions to generate [a signed URL for CloudFront](https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/private-content-signed-urls.html)","ref":"Antikythera.Aws.CloudfrontSignedUrl.html","title":"Antikythera.Aws.CloudfrontSignedUrl","type":"module"},{"doc":"Generates a signed URL to access a file via CloudFront.","ref":"Antikythera.Aws.CloudfrontSignedUrl.html#generate_signed_url/5","title":"Antikythera.Aws.CloudfrontSignedUrl.generate_signed_url/5","type":"function"},{"doc":"- `resource_url` (string): CloudFront URL used for accessing the file, including query string parameters, if any.\n- `lifetime_in_seconds` (positive integer): Expiration time is determined as the sum of the current time (in seconds) and this value.\n- `key_pair_id` (string): ID for an active CloudFront key pair used for generating the signature.\n- `private_key` (string): RSA private key for the key pair specified by `key_pair_id`.\n- `url_encoded?` (boolean): Whether `resource_url` is encoded or not (optional, default is `false`).","ref":"Antikythera.Aws.CloudfrontSignedUrl.html#generate_signed_url/5-parameters","title":"Parameters - Antikythera.Aws.CloudfrontSignedUrl.generate_signed_url/5","type":"function"},{"doc":"A generated signed URL (string).","ref":"Antikythera.Aws.CloudfrontSignedUrl.html#generate_signed_url/5-return-value","title":"Return value - Antikythera.Aws.CloudfrontSignedUrl.generate_signed_url/5","type":"function"},{"doc":"Generates a signed URL to access a file via CloudFront using a custom policy.","ref":"Antikythera.Aws.CloudfrontSignedUrl.html#generate_signed_url_using_custom_policy/6","title":"Antikythera.Aws.CloudfrontSignedUrl.generate_signed_url_using_custom_policy/6","type":"function"},{"doc":"- `resource_url` (string): CloudFront URL used for accessing the file, including query string parameters, if any.\n- `lifetime_in_seconds` (positive integer): Expiration time is determined as the sum of the current time (in seconds) and this value.\n- `key_pair_id` (string): ID for an active CloudFront key pair used for generating the signature.\n- `private_key` (string): RSA private key for the key pair specified by `key_pair_id`.\n- `url_encoded?` (boolean): Whether `resource_url` is encoded or not (optional, default is `false`).\n- `optional_policy` (Keyword): Optional policy conditions to be added to a custom policy (default is `[]`). Currently, supports only the following keywords:\n  - `:date_greater_than`(integer >= 0): Seconds in `AWS:EpochTime`. Specified to `DateGreaterThan`\n  - `:ip_address` (list of strings): Specified to `IpAddress`.","ref":"Antikythera.Aws.CloudfrontSignedUrl.html#generate_signed_url_using_custom_policy/6-parameters","title":"Parameters - Antikythera.Aws.CloudfrontSignedUrl.generate_signed_url_using_custom_policy/6","type":"function"},{"doc":"A generated signed URL (string).","ref":"Antikythera.Aws.CloudfrontSignedUrl.html#generate_signed_url_using_custom_policy/6-return-value","title":"Return value - Antikythera.Aws.CloudfrontSignedUrl.generate_signed_url_using_custom_policy/6","type":"function"},{"doc":"Utilities involving `Code` module, used for metaprogramming.","ref":"Antikythera.CodeUtil.html","title":"Antikythera.CodeUtil","type":"module"},{"doc":"Fetches function document from already existing module.\n\nUsed when annotating a function with the same doc attribute as that of another function.","ref":"Antikythera.CodeUtil.html#doc_by_mfa!/3","title":"Antikythera.CodeUtil.doc_by_mfa!/3","type":"function"},{"doc":"Definition of `Antikythera.Conn` struct, which represents a client-server connection.\n\nThis module also defines many functions to work with `Antikythera.Conn`.","ref":"Antikythera.Conn.html","title":"Antikythera.Conn","type":"module"},{"doc":"","ref":"Antikythera.Conn.html#assign/3","title":"Antikythera.Conn.assign/3","type":"function"},{"doc":"Type-aware getter for assigns.","ref":"Antikythera.Conn.html#assigns/1","title":"Antikythera.Conn.assigns/1","type":"function"},{"doc":"Type-aware setter for assigns.","ref":"Antikythera.Conn.html#assigns/2","title":"Antikythera.Conn.assigns/2","type":"function"},{"doc":"Type-aware getter for before_send.","ref":"Antikythera.Conn.html#before_send/1","title":"Antikythera.Conn.before_send/1","type":"function"},{"doc":"Type-aware setter for before_send.","ref":"Antikythera.Conn.html#before_send/2","title":"Antikythera.Conn.before_send/2","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#clear_session/1","title":"Antikythera.Conn.clear_session/1","type":"function"},{"doc":"Type-aware getter for context.","ref":"Antikythera.Conn.html#context/1","title":"Antikythera.Conn.context/1","type":"function"},{"doc":"Type-aware setter for context.","ref":"Antikythera.Conn.html#context/2","title":"Antikythera.Conn.context/2","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#delete_session/2","title":"Antikythera.Conn.delete_session/2","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#destroy_session/1","title":"Antikythera.Conn.destroy_session/1","type":"function"},{"doc":"Gets a flash message stored in the given `t:Antikythera.Conn.t/0`.","ref":"Antikythera.Conn.html#get_flash/2","title":"Antikythera.Conn.get_flash/2","type":"function"},{"doc":"Returns a request cookie specified by `name`.","ref":"Antikythera.Conn.html#get_req_cookie/2","title":"Antikythera.Conn.get_req_cookie/2","type":"function"},{"doc":"Returns all request cookies.","ref":"Antikythera.Conn.html#get_req_cookies/1","title":"Antikythera.Conn.get_req_cookies/1","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#get_req_header/2","title":"Antikythera.Conn.get_req_header/2","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#get_req_query/2","title":"Antikythera.Conn.get_req_query/2","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#get_session/2","title":"Antikythera.Conn.get_session/2","type":"function"},{"doc":"Returns a JSON response.","ref":"Antikythera.Conn.html#json/3","title":"Antikythera.Conn.json/3","type":"function"},{"doc":"Creates a new instance of Antikythera.Conn by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Conn.html#new/1","title":"Antikythera.Conn.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Conn.html#new!/1","title":"Antikythera.Conn.new!/1","type":"function"},{"doc":"Put `cache-control` response header for responses that must not be cached.\n\nThe actual header value to be set is: `\"private, no-cache, no-store, max-age=0\"`.","ref":"Antikythera.Conn.html#no_cache/1","title":"Antikythera.Conn.no_cache/1","type":"function"},{"doc":"Stores the flash message into the current `t:Antikythera.Conn.t/0`.","ref":"Antikythera.Conn.html#put_flash/3","title":"Antikythera.Conn.put_flash/3","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#put_resp_body/2","title":"Antikythera.Conn.put_resp_body/2","type":"function"},{"doc":"Adds a `set-cookie` response header to the given `Antikythera.Conn.t`.\n\n`path` directive of `set-cookie` header is automatically filled with `\"/\"` if not explicitly given.\nAlso `secure` directive is filled by default in the cloud environments (assuming that it's serving with HTTPS).\n\nNote that response cookies are stored separately from the other response headers,\nas cookies require special treatment according to the HTTP specs.","ref":"Antikythera.Conn.html#put_resp_cookie/4","title":"Antikythera.Conn.put_resp_cookie/4","type":"function"},{"doc":"Tells the client to delete an existing cookie specified by `name`.\n\nThis is a wrapper around `put_resp_cookie/4` that sets an immediately expiring cookie (whose value is an empty string).","ref":"Antikythera.Conn.html#put_resp_cookie_to_revoke/2","title":"Antikythera.Conn.put_resp_cookie_to_revoke/2","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#put_resp_header/3","title":"Antikythera.Conn.put_resp_header/3","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#put_resp_headers/2","title":"Antikythera.Conn.put_resp_headers/2","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#put_session/3","title":"Antikythera.Conn.put_session/3","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#put_status/2","title":"Antikythera.Conn.put_status/2","type":"function"},{"doc":"Returns an HTTP response that make the client redirect to the specified `url`.","ref":"Antikythera.Conn.html#redirect/3","title":"Antikythera.Conn.redirect/3","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#register_before_send/2","title":"Antikythera.Conn.register_before_send/2","type":"function"},{"doc":"Renders a HAML template file and returns the dynamic content as an HTML response.","ref":"Antikythera.Conn.html#render/5","title":"Antikythera.Conn.render/5","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#renew_session/1","title":"Antikythera.Conn.renew_session/1","type":"function"},{"doc":"Type-aware getter for request.","ref":"Antikythera.Conn.html#request/1","title":"Antikythera.Conn.request/1","type":"function"},{"doc":"Type-aware setter for request.","ref":"Antikythera.Conn.html#request/2","title":"Antikythera.Conn.request/2","type":"function"},{"doc":"Type-aware getter for resp_body.","ref":"Antikythera.Conn.html#resp_body/1","title":"Antikythera.Conn.resp_body/1","type":"function"},{"doc":"Type-aware setter for resp_body.","ref":"Antikythera.Conn.html#resp_body/2","title":"Antikythera.Conn.resp_body/2","type":"function"},{"doc":"Type-aware getter for resp_cookies.","ref":"Antikythera.Conn.html#resp_cookies/1","title":"Antikythera.Conn.resp_cookies/1","type":"function"},{"doc":"Type-aware setter for resp_cookies.","ref":"Antikythera.Conn.html#resp_cookies/2","title":"Antikythera.Conn.resp_cookies/2","type":"function"},{"doc":"Type-aware getter for resp_headers.","ref":"Antikythera.Conn.html#resp_headers/1","title":"Antikythera.Conn.resp_headers/1","type":"function"},{"doc":"Type-aware setter for resp_headers.","ref":"Antikythera.Conn.html#resp_headers/2","title":"Antikythera.Conn.resp_headers/2","type":"function"},{"doc":"Sends a file which resides in `priv/` directory as a response.\n\n`path` must be a file path relative to the `priv/` directory.\ncontent-type header is inferred from the file's extension.\n\nDon't use this function for sending large files; you should use CDN for large files (see `Antikythera.Asset`).\nAlso, if all you need to do is just to return a file (i.e. you don't need any authentication),\nyou should not use this function; just placing the file under `priv/static/` directory should suffice.","ref":"Antikythera.Conn.html#send_priv_file/3","title":"Antikythera.Conn.send_priv_file/3","type":"function"},{"doc":"Type-aware getter for status.","ref":"Antikythera.Conn.html#status/1","title":"Antikythera.Conn.status/1","type":"function"},{"doc":"Type-aware setter for status.","ref":"Antikythera.Conn.html#status/2","title":"Antikythera.Conn.status/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.Conn with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Conn.html#update/2","title":"Antikythera.Conn.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Conn.html#update!/2","title":"Antikythera.Conn.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Conn.html#valid?/1","title":"Antikythera.Conn.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Conn.html#t:t/0","title":"Antikythera.Conn.t/0","type":"type"},{"doc":"","ref":"Antikythera.Conn.Assigns.html","title":"Antikythera.Conn.Assigns","type":"module"},{"doc":"","ref":"Antikythera.Conn.Assigns.html#default/0","title":"Antikythera.Conn.Assigns.default/0","type":"function"},{"doc":"","ref":"Antikythera.Conn.Assigns.html#valid?/1","title":"Antikythera.Conn.Assigns.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Conn.Assigns.html#t:t/0","title":"Antikythera.Conn.Assigns.t/0","type":"type"},{"doc":"","ref":"Antikythera.Conn.BeforeSend.html","title":"Antikythera.Conn.BeforeSend","type":"module"},{"doc":"","ref":"Antikythera.Conn.BeforeSend.html#default/0","title":"Antikythera.Conn.BeforeSend.default/0","type":"function"},{"doc":"","ref":"Antikythera.Conn.BeforeSend.html#valid?/1","title":"Antikythera.Conn.BeforeSend.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Conn.BeforeSend.html#t:t/0","title":"Antikythera.Conn.BeforeSend.t/0","type":"type"},{"doc":"Definition of `Antikythera.Context` struct which represents a context for an execution of gear code.\n\nNote that `gear_entry_point` is basically of type `{module, atom}`, it is `nil` during\n(1) `executor_pool_web_request/1` and (2) `no_route/1` error handler.","ref":"Antikythera.Context.html","title":"Antikythera.Context","type":"module"},{"doc":"Type-aware getter for context_id.","ref":"Antikythera.Context.html#context_id/1","title":"Antikythera.Context.context_id/1","type":"function"},{"doc":"Type-aware setter for context_id.","ref":"Antikythera.Context.html#context_id/2","title":"Antikythera.Context.context_id/2","type":"function"},{"doc":"Type-aware getter for executor_pool_id.","ref":"Antikythera.Context.html#executor_pool_id/1","title":"Antikythera.Context.executor_pool_id/1","type":"function"},{"doc":"Type-aware setter for executor_pool_id.","ref":"Antikythera.Context.html#executor_pool_id/2","title":"Antikythera.Context.executor_pool_id/2","type":"function"},{"doc":"Type-aware getter for gear_entry_point.","ref":"Antikythera.Context.html#gear_entry_point/1","title":"Antikythera.Context.gear_entry_point/1","type":"function"},{"doc":"Type-aware setter for gear_entry_point.","ref":"Antikythera.Context.html#gear_entry_point/2","title":"Antikythera.Context.gear_entry_point/2","type":"function"},{"doc":"Type-aware getter for gear_name.","ref":"Antikythera.Context.html#gear_name/1","title":"Antikythera.Context.gear_name/1","type":"function"},{"doc":"Type-aware setter for gear_name.","ref":"Antikythera.Context.html#gear_name/2","title":"Antikythera.Context.gear_name/2","type":"function"},{"doc":"Creates a new instance of Antikythera.Context by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Context.html#new/1","title":"Antikythera.Context.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Context.html#new!/1","title":"Antikythera.Context.new!/1","type":"function"},{"doc":"Type-aware getter for start_time.","ref":"Antikythera.Context.html#start_time/1","title":"Antikythera.Context.start_time/1","type":"function"},{"doc":"Type-aware setter for start_time.","ref":"Antikythera.Context.html#start_time/2","title":"Antikythera.Context.start_time/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.Context with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Context.html#update/2","title":"Antikythera.Context.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Context.html#update!/2","title":"Antikythera.Context.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Context.html#valid?/1","title":"Antikythera.Context.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Context.html#t:t/0","title":"Antikythera.Context.t/0","type":"type"},{"doc":"","ref":"Antikythera.Context.GearEntryPoint.html","title":"Antikythera.Context.GearEntryPoint","type":"module"},{"doc":"","ref":"Antikythera.Context.GearEntryPoint.html#valid?/1","title":"Antikythera.Context.GearEntryPoint.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Context.GearEntryPoint.html#t:t/0","title":"Antikythera.Context.GearEntryPoint.t/0","type":"type"},{"doc":"","ref":"Antikythera.ContextId.html","title":"Antikythera.ContextId","type":"module"},{"doc":"","ref":"Antikythera.ContextId.html#pattern/0","title":"Antikythera.ContextId.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.ContextId.html#system_context/0","title":"Antikythera.ContextId.system_context/0","type":"function"},{"doc":"","ref":"Antikythera.ContextId.html#valid?/1","title":"Antikythera.ContextId.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.ContextId.html#t:t/0","title":"Antikythera.ContextId.t/0","type":"type"},{"doc":"Helper module to bulk-import many controller-related modules.\n\nIt's recommended to `use Antikythera.Controller` (either directly or indirectly) in your controller modules.","ref":"Antikythera.Controller.html","title":"Antikythera.Controller","type":"module"},{"doc":"Macro definition for plug DSL.\n\n\"Plug\" is a specification of composable modules for web apps.\nFor example, you can define your authentication logic as a plug and apply it to all your controller actions.","ref":"Antikythera.Controller.Plug.html","title":"Antikythera.Controller.Plug","type":"module"},{"doc":"To use plug you must first add `use Antikythera.Controller` in your controller module.\nThen you can invoke `plug/3` macro as follows.\n\n    plug Antikythera.Plug.BasicAuthentication, :check_with_config, []\n\nThe arguments are\n\n1. module\n2. function name\n3. options to be passed to plug function\n4. (optional) options for enabling plug\n\nIn this case `Antikythera.Plug.BasicAuthentication.check_with_config/2` is invoked just before execution of actions\ndefined in this controller module.\nIf you want to skip running plug for some actions in a controller module, you can use `:except` or `:only` option.\n\n    plug YourGear.SomePlug1, :do_something, [], [except: [:action_x, :action_y]]\n    plug YourGear.SomePlug2, :do_something, [], [only: [:action_x]]","ref":"Antikythera.Controller.Plug.html#module-usage","title":"Usage - Antikythera.Controller.Plug","type":"module"},{"doc":"","ref":"Antikythera.Controller.Plug.html#plug/4","title":"Antikythera.Controller.Plug.plug/4","type":"macro"},{"doc":"A type module to represent a wildcard subdomain in a form which is acceptable by cowboy.","ref":"Antikythera.CowboyWildcardSubdomain.html","title":"Antikythera.CowboyWildcardSubdomain","type":"module"},{"doc":"","ref":"Antikythera.CowboyWildcardSubdomain.html#pattern/0","title":"Antikythera.CowboyWildcardSubdomain.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.CowboyWildcardSubdomain.html#valid?/1","title":"Antikythera.CowboyWildcardSubdomain.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.CowboyWildcardSubdomain.html#t:t/0","title":"Antikythera.CowboyWildcardSubdomain.t/0","type":"type"},{"doc":"Calculate time schedules based on cron format strings.\n\n`parse/1` recognizes the [POSIX specifications of crontab format](http://www.unix.com/man-page/posix/1posix/crontab)\nwith the extension of \"step values\" (explained below).\nThe parsed object can be used to compute next matching time in `next/2`.\n\nNote that all times are in UTC, as is the case with `Antikythera.Time`.","ref":"Antikythera.Cron.html","title":"Antikythera.Cron","type":"module"},{"doc":"- The cron schedule is specified by 5 fields separated by whitespaces.\n- Allowed values for each field are:\n    - minutes      : 0-59\n    - hours        : 0-23\n    - day of month : 1-31\n    - month        : 1-12\n    - day of week  : 0-6 (0=Sunday)\n- Multiple elements can be used within a field by separating each by `,`.\n- An element shall be either a number or two numbers separated by a `-` (meaning an inclusive range).\n- A field may contain `*` which stands for \"first-last\".\n- Step values as in \"/ \" can be used in conjunction with ranges.\n  For example,\n    - \"0-18/4\" is identical to \"0,4,8,12,16\", and\n    - \"*/10\" in minutes field is identical to \"0,10,20,30,40,50\".\n- If both 'day of month' and 'day of week' are not \"*\", then the dates are the ones matching **either** of the fields.\n  For example, \"30 4 1,15 * 5\" indicates both of the followings:\n    - 4:30 on the 1st and 15th of each month\n    - 4:30 on every Friday\n- Schedules that actually don't represent valid date are not allowed.\n  For example, \"0 0 31 4 *\" is rejected as 31st of April does not exist.","ref":"Antikythera.Cron.html#module-schedule-format","title":"Schedule format - Antikythera.Cron","type":"module"},{"doc":"Type-aware getter for day_of_month.","ref":"Antikythera.Cron.html#day_of_month/1","title":"Antikythera.Cron.day_of_month/1","type":"function"},{"doc":"Type-aware setter for day_of_month.","ref":"Antikythera.Cron.html#day_of_month/2","title":"Antikythera.Cron.day_of_month/2","type":"function"},{"doc":"Type-aware getter for day_of_week.","ref":"Antikythera.Cron.html#day_of_week/1","title":"Antikythera.Cron.day_of_week/1","type":"function"},{"doc":"Type-aware setter for day_of_week.","ref":"Antikythera.Cron.html#day_of_week/2","title":"Antikythera.Cron.day_of_week/2","type":"function"},{"doc":"Type-aware getter for hour.","ref":"Antikythera.Cron.html#hour/1","title":"Antikythera.Cron.hour/1","type":"function"},{"doc":"Type-aware setter for hour.","ref":"Antikythera.Cron.html#hour/2","title":"Antikythera.Cron.hour/2","type":"function"},{"doc":"Type-aware getter for minute.","ref":"Antikythera.Cron.html#minute/1","title":"Antikythera.Cron.minute/1","type":"function"},{"doc":"Type-aware setter for minute.","ref":"Antikythera.Cron.html#minute/2","title":"Antikythera.Cron.minute/2","type":"function"},{"doc":"Type-aware getter for month.","ref":"Antikythera.Cron.html#month/1","title":"Antikythera.Cron.month/1","type":"function"},{"doc":"Type-aware setter for month.","ref":"Antikythera.Cron.html#month/2","title":"Antikythera.Cron.month/2","type":"function"},{"doc":"Creates a new instance of Antikythera.Cron by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Cron.html#new/1","title":"Antikythera.Cron.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Cron.html#new!/1","title":"Antikythera.Cron.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.html#next/2","title":"Antikythera.Cron.next/2","type":"function"},{"doc":"","ref":"Antikythera.Cron.html#next_in_epoch_milliseconds/2","title":"Antikythera.Cron.next_in_epoch_milliseconds/2","type":"function"},{"doc":"","ref":"Antikythera.Cron.html#parse/1","title":"Antikythera.Cron.parse/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.html#parse!/1","title":"Antikythera.Cron.parse!/1","type":"function"},{"doc":"Type-aware getter for source.","ref":"Antikythera.Cron.html#source/1","title":"Antikythera.Cron.source/1","type":"function"},{"doc":"Type-aware setter for source.","ref":"Antikythera.Cron.html#source/2","title":"Antikythera.Cron.source/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.Cron with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Cron.html#update/2","title":"Antikythera.Cron.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Cron.html#update!/2","title":"Antikythera.Cron.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Cron.html#valid?/1","title":"Antikythera.Cron.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.html#t:t/0","title":"Antikythera.Cron.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.html","title":"Antikythera.Cron.DayOfMonth","type":"module"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.html#max/0","title":"Antikythera.Cron.DayOfMonth.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.html#min/0","title":"Antikythera.Cron.DayOfMonth.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.html#valid?/1","title":"Antikythera.Cron.DayOfMonth.valid?/1","type":"function"},{"doc":"Wildcard `:*` or sorted list of values.","ref":"Antikythera.Cron.DayOfMonth.html#t:t/0","title":"Antikythera.Cron.DayOfMonth.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.Int.html","title":"Antikythera.Cron.DayOfMonth.Int","type":"module"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.Int.html#max/0","title":"Antikythera.Cron.DayOfMonth.Int.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.Int.html#min/0","title":"Antikythera.Cron.DayOfMonth.Int.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.Int.html#valid?/1","title":"Antikythera.Cron.DayOfMonth.Int.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfMonth.Int.html#t:t/0","title":"Antikythera.Cron.DayOfMonth.Int.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.html","title":"Antikythera.Cron.DayOfWeek","type":"module"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.html#max/0","title":"Antikythera.Cron.DayOfWeek.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.html#min/0","title":"Antikythera.Cron.DayOfWeek.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.html#valid?/1","title":"Antikythera.Cron.DayOfWeek.valid?/1","type":"function"},{"doc":"Wildcard `:*` or sorted list of values.","ref":"Antikythera.Cron.DayOfWeek.html#t:t/0","title":"Antikythera.Cron.DayOfWeek.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.Int.html","title":"Antikythera.Cron.DayOfWeek.Int","type":"module"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.Int.html#max/0","title":"Antikythera.Cron.DayOfWeek.Int.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.Int.html#min/0","title":"Antikythera.Cron.DayOfWeek.Int.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.Int.html#valid?/1","title":"Antikythera.Cron.DayOfWeek.Int.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.DayOfWeek.Int.html#t:t/0","title":"Antikythera.Cron.DayOfWeek.Int.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.Hour.html","title":"Antikythera.Cron.Hour","type":"module"},{"doc":"","ref":"Antikythera.Cron.Hour.html#max/0","title":"Antikythera.Cron.Hour.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Hour.html#min/0","title":"Antikythera.Cron.Hour.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Hour.html#valid?/1","title":"Antikythera.Cron.Hour.valid?/1","type":"function"},{"doc":"Wildcard `:*` or sorted list of values.","ref":"Antikythera.Cron.Hour.html#t:t/0","title":"Antikythera.Cron.Hour.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.Hour.Int.html","title":"Antikythera.Cron.Hour.Int","type":"module"},{"doc":"","ref":"Antikythera.Cron.Hour.Int.html#max/0","title":"Antikythera.Cron.Hour.Int.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Hour.Int.html#min/0","title":"Antikythera.Cron.Hour.Int.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Hour.Int.html#valid?/1","title":"Antikythera.Cron.Hour.Int.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.Hour.Int.html#t:t/0","title":"Antikythera.Cron.Hour.Int.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.Minute.html","title":"Antikythera.Cron.Minute","type":"module"},{"doc":"","ref":"Antikythera.Cron.Minute.html#max/0","title":"Antikythera.Cron.Minute.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Minute.html#min/0","title":"Antikythera.Cron.Minute.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Minute.html#valid?/1","title":"Antikythera.Cron.Minute.valid?/1","type":"function"},{"doc":"Wildcard `:*` or sorted list of values.","ref":"Antikythera.Cron.Minute.html#t:t/0","title":"Antikythera.Cron.Minute.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.Minute.Int.html","title":"Antikythera.Cron.Minute.Int","type":"module"},{"doc":"","ref":"Antikythera.Cron.Minute.Int.html#max/0","title":"Antikythera.Cron.Minute.Int.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Minute.Int.html#min/0","title":"Antikythera.Cron.Minute.Int.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Minute.Int.html#valid?/1","title":"Antikythera.Cron.Minute.Int.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.Minute.Int.html#t:t/0","title":"Antikythera.Cron.Minute.Int.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.Month.html","title":"Antikythera.Cron.Month","type":"module"},{"doc":"","ref":"Antikythera.Cron.Month.html#max/0","title":"Antikythera.Cron.Month.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Month.html#min/0","title":"Antikythera.Cron.Month.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Month.html#valid?/1","title":"Antikythera.Cron.Month.valid?/1","type":"function"},{"doc":"Wildcard `:*` or sorted list of values.","ref":"Antikythera.Cron.Month.html#t:t/0","title":"Antikythera.Cron.Month.t/0","type":"type"},{"doc":"","ref":"Antikythera.Cron.Month.Int.html","title":"Antikythera.Cron.Month.Int","type":"module"},{"doc":"","ref":"Antikythera.Cron.Month.Int.html#max/0","title":"Antikythera.Cron.Month.Int.max/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Month.Int.html#min/0","title":"Antikythera.Cron.Month.Int.min/0","type":"function"},{"doc":"","ref":"Antikythera.Cron.Month.Int.html#valid?/1","title":"Antikythera.Cron.Month.Int.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Cron.Month.Int.html#t:t/0","title":"Antikythera.Cron.Month.Int.t/0","type":"type"},{"doc":"","ref":"Antikythera.Crypto.html","title":"Antikythera.Crypto","type":"module"},{"doc":"Checks equality of the given two binaries in constant-time to avoid [timing attacks](http://codahale.com/a-lesson-in-timing-attacks/).","ref":"Antikythera.Crypto.html#secure_compare/2","title":"Antikythera.Crypto.secure_compare/2","type":"function"},{"doc":"Easy to use data encryption/decryption utilities.\n\nBoth Counter (CTR) mode and Galois/Counter mode (GCM) are supported.\nWhen only secrecy of data is required, use CTR mode.\nIf you need not only secrecy but also data integrity, use GCM.","ref":"Antikythera.Crypto.Aes.html","title":"Antikythera.Crypto.Aes","type":"module"},{"doc":"The functions defined in this module accept arbitrary binary as password.\nTo make an AES key (which is 128bit length) from a given password, the functions by default use MD5 hash algorithm.\nIf you need to increase computational cost of key derivation and make attacks such as dictionary attacks more difficult,\nyou may pass your own key derivation function.\nTo implement your key derivation function you can use `:crypto` module.","ref":"Antikythera.Crypto.Aes.html#module-deriving-an-aes-key-from-given-password","title":"Deriving an AES key from given password - Antikythera.Crypto.Aes","type":"module"},{"doc":"When encrypting given data, the encrypt function generates a random initialization vector and prepends it to the encrypted data.\nThe decrypt function extracts the initialization vector and use it to decrypt the rest.","ref":"Antikythera.Crypto.Aes.html#module-transparent-handling-of-initialization-vector","title":"Transparent handling of initialization vector - Antikythera.Crypto.Aes","type":"module"},{"doc":"For GCM you may pass AAD (arbitrary binary) as an additional argument.\nAAD is used only for generating/validating authentication tag; it doesn't affect resulting cipher text.\n\nAAD can be used to provide contextual information for the authentication of cipher text.\nFor example, you could pass \"login user ID\" as AAD when encrypting/decrypting each user's data,\nThis way, even when a malicious user who somehow copied another user's encrypted data and secret key into his own account,\nyou could prevent him from decrypting the data because of the difference in AAD.\n\nIf you don't have any suitable data for AAD you can pass an empty string (which is the default value).","ref":"Antikythera.Crypto.Aes.html#module-associated-authenticated-data-aad-for-gcm","title":"Associated Authenticated Data (AAD) for GCM - Antikythera.Crypto.Aes","type":"module"},{"doc":"","ref":"Antikythera.Crypto.Aes.html#ctr128_decrypt/3","title":"Antikythera.Crypto.Aes.ctr128_decrypt/3","type":"function"},{"doc":"","ref":"Antikythera.Crypto.Aes.html#ctr128_encrypt/3","title":"Antikythera.Crypto.Aes.ctr128_encrypt/3","type":"function"},{"doc":"","ref":"Antikythera.Crypto.Aes.html#gcm128_decrypt/4","title":"Antikythera.Crypto.Aes.gcm128_decrypt/4","type":"function"},{"doc":"","ref":"Antikythera.Crypto.Aes.html#gcm128_encrypt/4","title":"Antikythera.Crypto.Aes.gcm128_encrypt/4","type":"function"},{"doc":"","ref":"Antikythera.Crypto.Aes.html#t:key128/0","title":"Antikythera.Crypto.Aes.key128/0","type":"type"},{"doc":"Domain name format, originally defined in [RFC1034](https://tools.ietf.org/html/rfc1034#section-3.5).\n\n- It accepts 1 to 63 letters label in top-level domains as per original syntax definition.\n- It accepts all-numeric top-level domains as opposed to the restriction in [RFC3696](https://tools.ietf.org/html/rfc3696#section-2).\n    - If it is the sole part of domain name, (e.g. http://2130706433)\n      client applications will most likely parse them as 32-bit integer representation of IPv4 address.","ref":"Antikythera.Domain.html","title":"Antikythera.Domain","type":"module"},{"doc":"","ref":"Antikythera.Domain.html#pattern/0","title":"Antikythera.Domain.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.Domain.html#pattern_body/0","title":"Antikythera.Domain.pattern_body/0","type":"function"},{"doc":"","ref":"Antikythera.Domain.html#valid?/1","title":"Antikythera.Domain.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Domain.html#t:t/0","title":"Antikythera.Domain.t/0","type":"type"},{"doc":"","ref":"Antikythera.DomainList.html","title":"Antikythera.DomainList","type":"module"},{"doc":"","ref":"Antikythera.DomainList.html#max_length/0","title":"Antikythera.DomainList.max_length/0","type":"function"},{"doc":"","ref":"Antikythera.DomainList.html#valid?/1","title":"Antikythera.DomainList.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.DomainList.html#t:t/0","title":"Antikythera.DomainList.t/0","type":"type"},{"doc":"Email address format, defined in [RFC5321](https://tools.ietf.org/html/rfc5321),\nand [RFC5322](https://tools.ietf.org/html/rfc5322).\nThere are some differences from original RFCs for simplicity:\n\n- Leading, trailing and/or consecutive '.'s in local parts are allowed.\n- Double-quoted local parts are not allowed.\n- '[]'-enclosed IP address literals in domain parts are not allowed.\n- Total length of domain parts are not limited to 255.","ref":"Antikythera.Email.html","title":"Antikythera.Email","type":"module"},{"doc":"","ref":"Antikythera.Email.html#pattern/0","title":"Antikythera.Email.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.Email.html#valid?/1","title":"Antikythera.Email.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Email.html#t:t/0","title":"Antikythera.Email.t/0","type":"type"},{"doc":"","ref":"Antikythera.EncodedPath.html","title":"Antikythera.EncodedPath","type":"module"},{"doc":"","ref":"Antikythera.EncodedPath.html#pattern/0","title":"Antikythera.EncodedPath.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.EncodedPath.html#valid?/1","title":"Antikythera.EncodedPath.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.EncodedPath.html#t:t/0","title":"Antikythera.EncodedPath.t/0","type":"type"},{"doc":"Utility functions to work with enumerables.","ref":"Antikythera.EnumUtil.html","title":"Antikythera.EnumUtil","type":"module"},{"doc":"A variant of `Enum.find/2` that raises an exception when no matching element is found.","ref":"Antikythera.EnumUtil.html#find!/2","title":"Antikythera.EnumUtil.find!/2","type":"function"},{"doc":"A variant of `Enum.find_value/2` that raises an exception when no matching element is found.","ref":"Antikythera.EnumUtil.html#find_value!/2","title":"Antikythera.EnumUtil.find_value!/2","type":"function"},{"doc":"Updates items of an enumerable with the given function, depending on context.\nContext can be `any`.\n\nThe function `fun` takes item and context as arguments, and must achieve 2 purposes:\n\n1. Update an item according to current context\n2. Produces new context for next item\n\nthen, return both as tuple `{new_item, new_context}`.","ref":"Antikythera.EnumUtil.html#map_with_context/3","title":"Antikythera.EnumUtil.map_with_context/3","type":"function"},{"doc":"","ref":"Antikythera.EnumUtil.html#t:context/0","title":"Antikythera.EnumUtil.context/0","type":"type"},{"doc":"","ref":"Antikythera.EnumUtil.html#t:element/0","title":"Antikythera.EnumUtil.element/0","type":"type"},{"doc":"Module to provide helpers to access environment variables defined by antikythera.","ref":"Antikythera.Env.html","title":"Antikythera.Env","type":"module"},{"doc":"Antikythera instances and gears may run in the following two modes:\n\n- As a mix project, when invoked by running `iex` or `mix` command-line tool.\n- As an OTP release, which is generated by e.g. `antikythera_core.generate_release` task.\n\nIn general there are multiple deployments per antikythera instance.\nTo distingish the target deployment from code, antikythera defines the following environment variables:\n\n- At compile time (for metaprogramming):\n  `ANTIKYTHERA_COMPILE_ENV` must be appropriately set; the value can be retrieved by `compile_env/0`.\n- At runtime:\n  `ANTIKYTHERA_RUNTIME_ENV` must be appropriately set; the value can be retrieved by `runtime_env/0`.\n\nIt is the responsibility of antikythera instance administrators to correctly set these environment variables\nwhen compiling/running antikythera and gears.\nPossible values returned by `compile_env/0` and `runtime_env/0` are:\n\n- names of deployments given by `:deployments` application config\n- `:local` (running an OTP release at local machine for testing purpose)\n- `:undefined`\n\nAs an example, if you set `:dev` and `:prod` in `:deployments`, then you get:\n\n    |                           | Mix.env() at compile-time              | Mix.env() at runtime    | compile_env(), runtime_env() |\n    |---------------------------+----------------------------------------+-------------------------+------------------------------|\n    | $ iex -S mix              | :dev  (toplevel), :prod (dependencies) | :dev                    | :undefined                   |\n    | $ mix test                | :test (toplevel), :prod (dependencies) | :test                   | :undefined                   |\n    | $ mix antikythera_local.* | :prod                                  | (:mix is not available) | :local                       |\n    | :dev deployment           | :prod                                  | (:mix is not available) | :dev                         |\n    | :prod deployment          | :prod                                  | (:mix is not available) | :prod                        |\n\nYou can use these values to distinguish the current context from your code.","ref":"Antikythera.Env.html#module-environments-and-deployments","title":"Environments and deployments - Antikythera.Env","type":"module"},{"doc":"","ref":"Antikythera.Env.html#antikythera_instance_name/0","title":"Antikythera.Env.antikythera_instance_name/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#asset_base_url/1","title":"Antikythera.Env.asset_base_url/1","type":"function"},{"doc":"Return the base URL which based on the `Host` HTTP header of the request.\n\nThis function is useful if you use a custom domain.\nWhether the scheme of the URL is `https` or `http` depends on whether `Antikythera.Env.Mapping.cloud?/1` returns true or false.","ref":"Antikythera.Env.html#base_url/1","title":"Antikythera.Env.base_url/1","type":"function"},{"doc":"","ref":"Antikythera.Env.html#compile_env/0","title":"Antikythera.Env.compile_env/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#compiling_for_cloud?/0","title":"Antikythera.Env.compiling_for_cloud?/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#compiling_for_mix_task?/0","title":"Antikythera.Env.compiling_for_mix_task?/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#compiling_for_release?/0","title":"Antikythera.Env.compiling_for_release?/0","type":"function"},{"doc":"Return the base URL of the gear.\n\nIf the gear name is `my_gear` and the Antikythera is deployed at `antikythera.example.com`,\nthis function returns `https://my-gear.antikythera.example.com`.","ref":"Antikythera.Env.html#default_base_url/2","title":"Antikythera.Env.default_base_url/2","type":"function"},{"doc":"Default timeout (in milli-seconds) for gear actions.\n\nThis can be configurable by specifying `\"GEAR_ACTION_TIMEOUT\"` environment variable when compiling antikythera.\nDefaults to `10000`.","ref":"Antikythera.Env.html#gear_action_timeout/0","title":"Antikythera.Env.gear_action_timeout/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#new/1","title":"Antikythera.Env.new/1","type":"function"},{"doc":"","ref":"Antikythera.Env.html#new!/1","title":"Antikythera.Env.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Env.html#no_listen?/0","title":"Antikythera.Env.no_listen?/0","type":"function"},{"doc":"TCP port to listen to for incoming web requests.\n\nThe port can be specified by `\"PORT\"` runtime environment variable.\nDefaults to `8081` during `mix test`, and `8080` otherwise\n(thus one can run both `iex -S mix` and `mix test` at the same time).","ref":"Antikythera.Env.html#port_to_listen/0","title":"Antikythera.Env.port_to_listen/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#running_in_cloud?/0","title":"Antikythera.Env.running_in_cloud?/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#running_on_mix_task?/0","title":"Antikythera.Env.running_on_mix_task?/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#running_with_release?/0","title":"Antikythera.Env.running_with_release?/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#runtime_env/0","title":"Antikythera.Env.runtime_env/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#valid?/1","title":"Antikythera.Env.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Env.html#values/0","title":"Antikythera.Env.values/0","type":"function"},{"doc":"","ref":"Antikythera.Env.html#t:t/0","title":"Antikythera.Env.t/0","type":"type"},{"doc":"","ref":"Antikythera.Env.Mapping.html","title":"Antikythera.Env.Mapping","type":"module"},{"doc":"","ref":"Antikythera.Env.Mapping.html#cloud?/1","title":"Antikythera.Env.Mapping.cloud?/1","type":"function"},{"doc":"","ref":"Antikythera.Env.Mapping.html#env_var_to_atom/1","title":"Antikythera.Env.Mapping.env_var_to_atom/1","type":"function"},{"doc":"Type module for error that describes what went wrong during a process execution.\n\nPossible values for `t:t/0` are:\n\n- `{:error, Exception.t}` : An exception was thrown and was not handled.\n- `:timeout`              : Execution timed out.\n- `:killed`               : Process was brutally killed, typically due to heap limit violation.\n- `{:throw, any}`         : A value was thrown but not caught.\n- `{:exit, any}`          : Process exited before completing the execution.","ref":"Antikythera.ErrorReason.html","title":"Antikythera.ErrorReason","type":"module"},{"doc":"","ref":"Antikythera.ErrorReason.html#format/2","title":"Antikythera.ErrorReason.format/2","type":"function"},{"doc":"","ref":"Antikythera.ErrorReason.html#valid?/1","title":"Antikythera.ErrorReason.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.ErrorReason.html#t:gear_action_error_reason/0","title":"Antikythera.ErrorReason.gear_action_error_reason/0","type":"type"},{"doc":"","ref":"Antikythera.ErrorReason.html#t:stack_item/0","title":"Antikythera.ErrorReason.stack_item/0","type":"type"},{"doc":"","ref":"Antikythera.ErrorReason.html#t:stacktrace/0","title":"Antikythera.ErrorReason.stacktrace/0","type":"type"},{"doc":"","ref":"Antikythera.ErrorReason.html#t:t/0","title":"Antikythera.ErrorReason.t/0","type":"type"},{"doc":"","ref":"Antikythera.ExecutorPool.BadIdReason.html","title":"Antikythera.ExecutorPool.BadIdReason","type":"module"},{"doc":"","ref":"Antikythera.ExecutorPool.BadIdReason.html#valid?/1","title":"Antikythera.ExecutorPool.BadIdReason.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.ExecutorPool.BadIdReason.html#t:t/0","title":"Antikythera.ExecutorPool.BadIdReason.t/0","type":"type"},{"doc":"","ref":"Antikythera.ExecutorPool.Id.html","title":"Antikythera.ExecutorPool.Id","type":"module"},{"doc":"","ref":"Antikythera.ExecutorPool.Id.html#nopool/0","title":"Antikythera.ExecutorPool.Id.nopool/0","type":"function"},{"doc":"","ref":"Antikythera.ExecutorPool.Id.html#to_string/1","title":"Antikythera.ExecutorPool.Id.to_string/1","type":"function"},{"doc":"","ref":"Antikythera.ExecutorPool.Id.html#valid?/1","title":"Antikythera.ExecutorPool.Id.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.ExecutorPool.Id.html#t:nopool_t/0","title":"Antikythera.ExecutorPool.Id.nopool_t/0","type":"type"},{"doc":"","ref":"Antikythera.ExecutorPool.Id.html#t:t/0","title":"Antikythera.ExecutorPool.Id.t/0","type":"type"},{"doc":"This module converts structures to JSON at low load by avoiding protocol calls. Internally Jason is used.","ref":"Antikythera.FastJasonEncoder.html","title":"Antikythera.FastJasonEncoder","type":"module"},{"doc":"","ref":"Antikythera.FastJasonEncoder.html#encode/1","title":"Antikythera.FastJasonEncoder.encode/1","type":"function"},{"doc":"","ref":"Antikythera.FastJasonEncoder.html#encode/2","title":"Antikythera.FastJasonEncoder.encode/2","type":"function"},{"doc":"","ref":"Antikythera.FastJasonEncoder.html#encode!/1","title":"Antikythera.FastJasonEncoder.encode!/1","type":"function"},{"doc":"","ref":"Antikythera.FastJasonEncoder.Wrapper.html","title":"Antikythera.FastJasonEncoder.Wrapper","type":"module"},{"doc":"","ref":"Antikythera.G2gRequest.html","title":"Antikythera.G2gRequest","type":"module"},{"doc":"Type-aware getter for body.","ref":"Antikythera.G2gRequest.html#body/1","title":"Antikythera.G2gRequest.body/1","type":"function"},{"doc":"Type-aware setter for body.","ref":"Antikythera.G2gRequest.html#body/2","title":"Antikythera.G2gRequest.body/2","type":"function"},{"doc":"Type-aware getter for cookies.","ref":"Antikythera.G2gRequest.html#cookies/1","title":"Antikythera.G2gRequest.cookies/1","type":"function"},{"doc":"Type-aware setter for cookies.","ref":"Antikythera.G2gRequest.html#cookies/2","title":"Antikythera.G2gRequest.cookies/2","type":"function"},{"doc":"","ref":"Antikythera.G2gRequest.html#from_web_request/1","title":"Antikythera.G2gRequest.from_web_request/1","type":"function"},{"doc":"Type-aware getter for headers.","ref":"Antikythera.G2gRequest.html#headers/1","title":"Antikythera.G2gRequest.headers/1","type":"function"},{"doc":"Type-aware setter for headers.","ref":"Antikythera.G2gRequest.html#headers/2","title":"Antikythera.G2gRequest.headers/2","type":"function"},{"doc":"Type-aware getter for method.","ref":"Antikythera.G2gRequest.html#method/1","title":"Antikythera.G2gRequest.method/1","type":"function"},{"doc":"Type-aware setter for method.","ref":"Antikythera.G2gRequest.html#method/2","title":"Antikythera.G2gRequest.method/2","type":"function"},{"doc":"Creates a new instance of Antikythera.G2gRequest by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.G2gRequest.html#new/1","title":"Antikythera.G2gRequest.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.G2gRequest.html#new!/1","title":"Antikythera.G2gRequest.new!/1","type":"function"},{"doc":"Type-aware getter for path.","ref":"Antikythera.G2gRequest.html#path/1","title":"Antikythera.G2gRequest.path/1","type":"function"},{"doc":"Type-aware setter for path.","ref":"Antikythera.G2gRequest.html#path/2","title":"Antikythera.G2gRequest.path/2","type":"function"},{"doc":"Type-aware getter for query_params.","ref":"Antikythera.G2gRequest.html#query_params/1","title":"Antikythera.G2gRequest.query_params/1","type":"function"},{"doc":"Type-aware setter for query_params.","ref":"Antikythera.G2gRequest.html#query_params/2","title":"Antikythera.G2gRequest.query_params/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.G2gRequest with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.G2gRequest.html#update/2","title":"Antikythera.G2gRequest.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.G2gRequest.html#update!/2","title":"Antikythera.G2gRequest.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.G2gRequest.html#valid?/1","title":"Antikythera.G2gRequest.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.G2gRequest.html#t:t/0","title":"Antikythera.G2gRequest.t/0","type":"type"},{"doc":"","ref":"Antikythera.G2gResponse.html","title":"Antikythera.G2gResponse","type":"module"},{"doc":"Type-aware getter for body.","ref":"Antikythera.G2gResponse.html#body/1","title":"Antikythera.G2gResponse.body/1","type":"function"},{"doc":"Type-aware setter for body.","ref":"Antikythera.G2gResponse.html#body/2","title":"Antikythera.G2gResponse.body/2","type":"function"},{"doc":"Type-aware getter for cookies.","ref":"Antikythera.G2gResponse.html#cookies/1","title":"Antikythera.G2gResponse.cookies/1","type":"function"},{"doc":"Type-aware setter for cookies.","ref":"Antikythera.G2gResponse.html#cookies/2","title":"Antikythera.G2gResponse.cookies/2","type":"function"},{"doc":"Creates a new version of `Antikythera.G2gResponse` struct by body decompression and decoding.\n\nThis function decompresses body of g2g response according to `content-encoding` and then\ndecodes the uncompressed body according to `content-type`.\nIf decompressed, `content-encoding` header is removed from the returned `Antikythera.G2gResponse`.\n\nThis function is used internally in `G2g.send/{1,2}` (i.e. body decoding is done automatically).\nBasically gear implementations do not need this function.\nThe typical use case of this function (in conjunction with `G2g.send_without_decoding/{1,2}`)\nis to avoid unnecessary encoding/decoding when a gear action\n\n- returns the g2g response as it is, and\n- reads body of the g2g response.","ref":"Antikythera.G2gResponse.html#decode_body/1","title":"Antikythera.G2gResponse.decode_body/1","type":"function"},{"doc":"Type-aware getter for headers.","ref":"Antikythera.G2gResponse.html#headers/1","title":"Antikythera.G2gResponse.headers/1","type":"function"},{"doc":"Type-aware setter for headers.","ref":"Antikythera.G2gResponse.html#headers/2","title":"Antikythera.G2gResponse.headers/2","type":"function"},{"doc":"Creates a new instance of Antikythera.G2gResponse by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.G2gResponse.html#new/1","title":"Antikythera.G2gResponse.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.G2gResponse.html#new!/1","title":"Antikythera.G2gResponse.new!/1","type":"function"},{"doc":"Type-aware getter for status.","ref":"Antikythera.G2gResponse.html#status/1","title":"Antikythera.G2gResponse.status/1","type":"function"},{"doc":"Type-aware setter for status.","ref":"Antikythera.G2gResponse.html#status/2","title":"Antikythera.G2gResponse.status/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.G2gResponse with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.G2gResponse.html#update/2","title":"Antikythera.G2gResponse.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.G2gResponse.html#update!/2","title":"Antikythera.G2gResponse.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.G2gResponse.html#valid?/1","title":"Antikythera.G2gResponse.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.G2gResponse.html#t:t/0","title":"Antikythera.G2gResponse.t/0","type":"type"},{"doc":"Type of timeout for gear actions in milliseconds.\nA value must be a positive integer less than or equal to `180000`.\nThe maximum value can be configured by `:gear_action_max_timeout` config.\nThe default value is determined by `Antikythera.Env.gear_action_timeout/0`,\nor the maximum value if it exceeds the maximum value.","ref":"Antikythera.GearActionTimeout.html","title":"Antikythera.GearActionTimeout","type":"module"},{"doc":"","ref":"Antikythera.GearActionTimeout.html#default/0","title":"Antikythera.GearActionTimeout.default/0","type":"function"},{"doc":"","ref":"Antikythera.GearActionTimeout.html#max/0","title":"Antikythera.GearActionTimeout.max/0","type":"function"},{"doc":"","ref":"Antikythera.GearActionTimeout.html#min/0","title":"Antikythera.GearActionTimeout.min/0","type":"function"},{"doc":"","ref":"Antikythera.GearActionTimeout.html#valid?/1","title":"Antikythera.GearActionTimeout.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.GearActionTimeout.html#t:t/0","title":"Antikythera.GearActionTimeout.t/0","type":"type"},{"doc":"Template for gear application module.\n\nInvoking `use Antikythera.GearApplication` generates bunch of code, including (but not limited to)\n\n- `Application` callbacks\n- `Logger` module for the gear\n- accessors to gear config\n\nAll gear implementations must have exactly one module that `use`s this module.\n\nFor antikythera maintainers:\nNote that, when modifying the macros defined in `GearApplication` and `GearApplication.*`,\nwe need to re-compile all gears in order to deploy the changes.","ref":"Antikythera.GearApplication.html","title":"Antikythera.GearApplication","type":"behaviour"},{"doc":"","ref":"Antikythera.GearApplication.html#c:children/0","title":"Antikythera.GearApplication.children/0","type":"callback"},{"doc":"","ref":"Antikythera.GearApplication.html#c:executor_pool_for_web_request/1","title":"Antikythera.GearApplication.executor_pool_for_web_request/1","type":"callback"},{"doc":"","ref":"Antikythera.GearApplication.html#t:child_spec/0","title":"Antikythera.GearApplication.child_spec/0","type":"type"},{"doc":"Helper module to define interface module for alerting.","ref":"Antikythera.GearApplication.AlertManager.html","title":"Antikythera.GearApplication.AlertManager","type":"module"},{"doc":"Helper module to define getter functions of gear configs.","ref":"Antikythera.GearApplication.ConfigGetter.html","title":"Antikythera.GearApplication.ConfigGetter","type":"module"},{"doc":"","ref":"Antikythera.GearApplication.ConfigGetter.html#cleanup_configs_in_process_dictionary/0","title":"Antikythera.GearApplication.ConfigGetter.cleanup_configs_in_process_dictionary/0","type":"function"},{"doc":"Helper module for gear's custom error handler.\n\nTo customize HTTP responses returned on errors, gear must implement an error handler module which\n\n- is named `YourGear.Controller.Error`, and\n- defines the following functions:\n    - Mandatory error handlers\n        - `error(Antikythera.Conn.t, Antikythera.ErrorReason.gear_action_error_reason) :: Antikythera.Conn.t`\n        - `no_route(Antikythera.Conn.t) :: Antikythera.Conn.t`\n        - `bad_request(Antikythera.Conn.t) :: Antikythera.Conn.t`\n    - Optional error handlers\n        - `bad_executor_pool_id(Antikythera.Conn.t, Antikythera.ExecutorPool.BadIdReason.t) :: Antikythera.Conn.t`\n        - `ws_too_many_connections(Antikythera.Conn.t) :: Antikythera.Conn.t` (when your gear uses websocket)\n\nThis module generates `YourGear.error_handler_module/0` function, which is called by antikythera when handling errors.","ref":"Antikythera.GearApplication.ErrorHandler.html","title":"Antikythera.GearApplication.ErrorHandler","type":"module"},{"doc":"Helper module to define each gear's `G2g` module that contains interface for gear-to-gear communication.","ref":"Antikythera.GearApplication.G2g.html","title":"Antikythera.GearApplication.G2g","type":"module"},{"doc":"Helper module to define each gear's `Logger` module.","ref":"Antikythera.GearApplication.Logger.html","title":"Antikythera.GearApplication.Logger","type":"module"},{"doc":"Helper module to define interface module to submit custom metrics data.","ref":"Antikythera.GearApplication.MetricsUploader.html","title":"Antikythera.GearApplication.MetricsUploader","type":"module"},{"doc":"Type module to represent gear names as atoms.","ref":"Antikythera.GearName.html","title":"Antikythera.GearName","type":"module"},{"doc":"","ref":"Antikythera.GearName.html#valid?/1","title":"Antikythera.GearName.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.GearName.html#t:t/0","title":"Antikythera.GearName.t/0","type":"type"},{"doc":"","ref":"Antikythera.GearNameStr.html","title":"Antikythera.GearNameStr","type":"module"},{"doc":"","ref":"Antikythera.GearNameStr.html#pattern/0","title":"Antikythera.GearNameStr.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.GearNameStr.html#valid?/1","title":"Antikythera.GearNameStr.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.GearNameStr.html#t:t/0","title":"Antikythera.GearNameStr.t/0","type":"type"},{"doc":"A thin wrapper around `Gettext.__using__/1` to forward error logs to each gear's logger.\n\nUsage of this module is exactly the same as `Gettext`; just `use Antikythera.Gettext` instead of `use Gettext`.\nOptions given to `use Antikythera.Gettext` are directly passed to `use Gettext`.\nThis module overrides the default implementation of `c:Gettext.Backend.handle_missing_bindings/2`\nto suit the use cases for antikythera gears.","ref":"Antikythera.Gettext.html","title":"Antikythera.Gettext","type":"module"},{"doc":"","ref":"Antikythera.Http.html","title":"Antikythera.Http","type":"module"},{"doc":"","ref":"Antikythera.Http.Body.html","title":"Antikythera.Http.Body","type":"module"},{"doc":"","ref":"Antikythera.Http.Body.html#default/0","title":"Antikythera.Http.Body.default/0","type":"function"},{"doc":"","ref":"Antikythera.Http.Body.html#valid?/1","title":"Antikythera.Http.Body.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Body.html#t:t/0","title":"Antikythera.Http.Body.t/0","type":"type"},{"doc":"HTTP headers as a map.\n\nIf multiple headers in a single request/response have the same header name,\ntheir values are concatenated with commas.\nIn case of `cookie` header values are concatenated using semicolons instead of commas.","ref":"Antikythera.Http.Headers.html","title":"Antikythera.Http.Headers","type":"module"},{"doc":"","ref":"Antikythera.Http.Headers.html#default/0","title":"Antikythera.Http.Headers.default/0","type":"function"},{"doc":"","ref":"Antikythera.Http.Headers.html#valid?/1","title":"Antikythera.Http.Headers.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Headers.html#t:t/0","title":"Antikythera.Http.Headers.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.Method.html","title":"Antikythera.Http.Method","type":"module"},{"doc":"","ref":"Antikythera.Http.Method.html#all/0","title":"Antikythera.Http.Method.all/0","type":"function"},{"doc":"","ref":"Antikythera.Http.Method.html#from_string/1","title":"Antikythera.Http.Method.from_string/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Method.html#new/1","title":"Antikythera.Http.Method.new/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Method.html#new!/1","title":"Antikythera.Http.Method.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Method.html#to_string/1","title":"Antikythera.Http.Method.to_string/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Method.html#valid?/1","title":"Antikythera.Http.Method.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Method.html#values/0","title":"Antikythera.Http.Method.values/0","type":"function"},{"doc":"","ref":"Antikythera.Http.Method.html#t:t/0","title":"Antikythera.Http.Method.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.QueryParams.html","title":"Antikythera.Http.QueryParams","type":"module"},{"doc":"","ref":"Antikythera.Http.QueryParams.html#default/0","title":"Antikythera.Http.QueryParams.default/0","type":"function"},{"doc":"","ref":"Antikythera.Http.QueryParams.html#valid?/1","title":"Antikythera.Http.QueryParams.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.QueryParams.html#t:t/0","title":"Antikythera.Http.QueryParams.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.RawBody.html","title":"Antikythera.Http.RawBody","type":"module"},{"doc":"","ref":"Antikythera.Http.RawBody.html#default/0","title":"Antikythera.Http.RawBody.default/0","type":"function"},{"doc":"","ref":"Antikythera.Http.RawBody.html#valid?/1","title":"Antikythera.Http.RawBody.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.RawBody.html#t:t/0","title":"Antikythera.Http.RawBody.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.ReqCookiesMap.html","title":"Antikythera.Http.ReqCookiesMap","type":"module"},{"doc":"","ref":"Antikythera.Http.ReqCookiesMap.html#default/0","title":"Antikythera.Http.ReqCookiesMap.default/0","type":"function"},{"doc":"","ref":"Antikythera.Http.ReqCookiesMap.html#valid?/1","title":"Antikythera.Http.ReqCookiesMap.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.ReqCookiesMap.html#t:t/0","title":"Antikythera.Http.ReqCookiesMap.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.SetCookie.html","title":"Antikythera.Http.SetCookie","type":"module"},{"doc":"Type-aware getter for domain.","ref":"Antikythera.Http.SetCookie.html#domain/1","title":"Antikythera.Http.SetCookie.domain/1","type":"function"},{"doc":"Type-aware setter for domain.","ref":"Antikythera.Http.SetCookie.html#domain/2","title":"Antikythera.Http.SetCookie.domain/2","type":"function"},{"doc":"Type-aware getter for http_only.","ref":"Antikythera.Http.SetCookie.html#http_only/1","title":"Antikythera.Http.SetCookie.http_only/1","type":"function"},{"doc":"Type-aware setter for http_only.","ref":"Antikythera.Http.SetCookie.html#http_only/2","title":"Antikythera.Http.SetCookie.http_only/2","type":"function"},{"doc":"Type-aware getter for max_age.","ref":"Antikythera.Http.SetCookie.html#max_age/1","title":"Antikythera.Http.SetCookie.max_age/1","type":"function"},{"doc":"Type-aware setter for max_age.","ref":"Antikythera.Http.SetCookie.html#max_age/2","title":"Antikythera.Http.SetCookie.max_age/2","type":"function"},{"doc":"Creates a new instance of Antikythera.Http.SetCookie by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Http.SetCookie.html#new/1","title":"Antikythera.Http.SetCookie.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Http.SetCookie.html#new!/1","title":"Antikythera.Http.SetCookie.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Http.SetCookie.html#parse!/1","title":"Antikythera.Http.SetCookie.parse!/1","type":"function"},{"doc":"Type-aware getter for path.","ref":"Antikythera.Http.SetCookie.html#path/1","title":"Antikythera.Http.SetCookie.path/1","type":"function"},{"doc":"Type-aware setter for path.","ref":"Antikythera.Http.SetCookie.html#path/2","title":"Antikythera.Http.SetCookie.path/2","type":"function"},{"doc":"Type-aware getter for secure.","ref":"Antikythera.Http.SetCookie.html#secure/1","title":"Antikythera.Http.SetCookie.secure/1","type":"function"},{"doc":"Type-aware setter for secure.","ref":"Antikythera.Http.SetCookie.html#secure/2","title":"Antikythera.Http.SetCookie.secure/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.Http.SetCookie with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Http.SetCookie.html#update/2","title":"Antikythera.Http.SetCookie.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Http.SetCookie.html#update!/2","title":"Antikythera.Http.SetCookie.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Http.SetCookie.html#valid?/1","title":"Antikythera.Http.SetCookie.valid?/1","type":"function"},{"doc":"Type-aware getter for value.","ref":"Antikythera.Http.SetCookie.html#value/1","title":"Antikythera.Http.SetCookie.value/1","type":"function"},{"doc":"Type-aware setter for value.","ref":"Antikythera.Http.SetCookie.html#value/2","title":"Antikythera.Http.SetCookie.value/2","type":"function"},{"doc":"","ref":"Antikythera.Http.SetCookie.html#t:options_t/0","title":"Antikythera.Http.SetCookie.options_t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.SetCookie.html#t:t/0","title":"Antikythera.Http.SetCookie.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.SetCookiesMap.html","title":"Antikythera.Http.SetCookiesMap","type":"module"},{"doc":"","ref":"Antikythera.Http.SetCookiesMap.html#default/0","title":"Antikythera.Http.SetCookiesMap.default/0","type":"function"},{"doc":"","ref":"Antikythera.Http.SetCookiesMap.html#new/1","title":"Antikythera.Http.SetCookiesMap.new/1","type":"function"},{"doc":"","ref":"Antikythera.Http.SetCookiesMap.html#new!/1","title":"Antikythera.Http.SetCookiesMap.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Http.SetCookiesMap.html#valid?/1","title":"Antikythera.Http.SetCookiesMap.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.SetCookiesMap.html#t:t/0","title":"Antikythera.Http.SetCookiesMap.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.Status.html","title":"Antikythera.Http.Status","type":"module"},{"doc":"","ref":"Antikythera.Http.Status.html#code/1","title":"Antikythera.Http.Status.code/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.html#valid?/1","title":"Antikythera.Http.Status.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.html#t:t/0","title":"Antikythera.Http.Status.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.Status.Atom.html","title":"Antikythera.Http.Status.Atom","type":"module"},{"doc":"","ref":"Antikythera.Http.Status.Atom.html#new/1","title":"Antikythera.Http.Status.Atom.new/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.Atom.html#new!/1","title":"Antikythera.Http.Status.Atom.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.Atom.html#valid?/1","title":"Antikythera.Http.Status.Atom.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.Atom.html#values/0","title":"Antikythera.Http.Status.Atom.values/0","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.Atom.html#t:t/0","title":"Antikythera.Http.Status.Atom.t/0","type":"type"},{"doc":"","ref":"Antikythera.Http.Status.Int.html","title":"Antikythera.Http.Status.Int","type":"module"},{"doc":"","ref":"Antikythera.Http.Status.Int.html#max/0","title":"Antikythera.Http.Status.Int.max/0","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.Int.html#min/0","title":"Antikythera.Http.Status.Int.min/0","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.Int.html#valid?/1","title":"Antikythera.Http.Status.Int.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Http.Status.Int.html#t:t/0","title":"Antikythera.Http.Status.Int.t/0","type":"type"},{"doc":"HTTP client library.\n\nThis is a wrapper around [`hackney`](https://github.com/benoitc/hackney), an HTTP client library.\n`Httpc` supports the following features:\n\n- gzip-compression is tranparently handled\n- headers are represented by maps instead of lists\n- header names are always lower case\n- TCP connections are automatically re-established when closed by server due to keepalive timeout","ref":"Antikythera.Httpc.html","title":"Antikythera.Httpc","type":"module"},{"doc":"The `body` argument in `post/4`, `put/4`, `patch/4`, `request/5` takes either:\n\n- `binary` - Sends raw data\n- `iolist` - Sends raw data list (for more details see [documentation for built-in types](https://hexdocs.pm/elixir/typespecs.html#built-in-types))\n- `{:form, [{key, value}]}` - Sends key-value data as x-www-form-urlencoded\n- `{:json, map}` - Converts map into JSON and sends as application/json\n- `{:file, path}` - Sends given file contents","ref":"Antikythera.Httpc.html#module-body-format","title":"Body format - Antikythera.Httpc","type":"module"},{"doc":"- `:timeout` - Timeout to establish a connection, in milliseconds. Default is `8000`.\n- `:recv_timeout` - Timeout used when receiving a response. Default is `5000`.\n- `:params` - An enumerable of 2-tuples that will be URL-encoded and appended to the URL as query string parameters.\n- `:cookie` - An enumerable of name-value pairs of cookies. `Httpc` automatically URL-encodes the given names/values for you.\n- `:basic_auth` - A pair of `{username, password}` tuple to be used for HTTP basic authentication.\n- `:proxy` - A proxy to be used for the request; it can be a regular URL or a `{host, port}` tuple.\n- `:proxy_auth` - Proxy authentication `{username, password}` tuple.\n- `:ssl` - SSL options supported by the `ssl` erlang module.\n- `:skip_ssl_verification` - Whether to verify server's SSL certificate or not. Defaults to `false`.\n  Specify `skip_ssl_verification: true` when accessing insecure server with HTTPS.\n- `:max_body` - Maximum content-length of the response body (compressed size if it's compressed).\n  Defaults to `10485760` (10MB)\n  and must not exceed 150MB.\n  Responses having body larger than the specified size will be rejected with `{:error, :response_too_large}`.\n- `:follow_redirect` - A boolean that causes redirects to be followed. Defaults to `false`.\n- `:max_redirect` - An integer denoting the maximum number of redirects to follow if `follow_redirect: true` is given.\n- `:skip_body_decompression` - By default gzip-compressed body is automatically decompressed (i.e. defaults to `false`).\n  Pass `skip_body_decompression: true` if compressed body is what you need.","ref":"Antikythera.Httpc.html#module-options","title":"Options - Antikythera.Httpc","type":"module"},{"doc":"","ref":"Antikythera.Httpc.html#default_max_body/0","title":"Antikythera.Httpc.default_max_body/0","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#delete/3","title":"Antikythera.Httpc.delete/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#delete!/3","title":"Antikythera.Httpc.delete!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#get/3","title":"Antikythera.Httpc.get/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#get!/3","title":"Antikythera.Httpc.get!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#head/3","title":"Antikythera.Httpc.head/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#head!/3","title":"Antikythera.Httpc.head!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#maximum_max_body/0","title":"Antikythera.Httpc.maximum_max_body/0","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#options/3","title":"Antikythera.Httpc.options/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#options!/3","title":"Antikythera.Httpc.options!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#patch/4","title":"Antikythera.Httpc.patch/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#patch!/4","title":"Antikythera.Httpc.patch!/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#post/4","title":"Antikythera.Httpc.post/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#post!/4","title":"Antikythera.Httpc.post!/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#put/4","title":"Antikythera.Httpc.put/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#put!/4","title":"Antikythera.Httpc.put!/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#request/5","title":"Antikythera.Httpc.request/5","type":"function"},{"doc":"","ref":"Antikythera.Httpc.html#request!/5","title":"Antikythera.Httpc.request!/5","type":"function"},{"doc":"Just wrapping `Httpc` without any modification.\nCan be mocked with `:meck.expect(Httpc.Mockable, :request, ...)` without interfering other Httpc action.","ref":"Antikythera.Httpc.Mockable.html","title":"Antikythera.Httpc.Mockable","type":"module"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#delete/3","title":"Antikythera.Httpc.Mockable.delete/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#delete!/3","title":"Antikythera.Httpc.Mockable.delete!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#get/3","title":"Antikythera.Httpc.Mockable.get/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#get!/3","title":"Antikythera.Httpc.Mockable.get!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#head/3","title":"Antikythera.Httpc.Mockable.head/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#head!/3","title":"Antikythera.Httpc.Mockable.head!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#options/3","title":"Antikythera.Httpc.Mockable.options/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#options!/3","title":"Antikythera.Httpc.Mockable.options!/3","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#patch/4","title":"Antikythera.Httpc.Mockable.patch/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#patch!/4","title":"Antikythera.Httpc.Mockable.patch!/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#post/4","title":"Antikythera.Httpc.Mockable.post/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#post!/4","title":"Antikythera.Httpc.Mockable.post!/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#put/4","title":"Antikythera.Httpc.Mockable.put/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#put!/4","title":"Antikythera.Httpc.Mockable.put!/4","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#request/5","title":"Antikythera.Httpc.Mockable.request/5","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Mockable.html#request!/5","title":"Antikythera.Httpc.Mockable.request!/5","type":"function"},{"doc":"Type for `Antikythera.Httpc`'s request body.","ref":"Antikythera.Httpc.ReqBody.html","title":"Antikythera.Httpc.ReqBody","type":"module"},{"doc":"","ref":"Antikythera.Httpc.ReqBody.html#convert_body_and_headers_by_body_type/2","title":"Antikythera.Httpc.ReqBody.convert_body_and_headers_by_body_type/2","type":"function"},{"doc":"","ref":"Antikythera.Httpc.ReqBody.html#valid?/1","title":"Antikythera.Httpc.ReqBody.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Httpc.ReqBody.html#t:json_obj/0","title":"Antikythera.Httpc.ReqBody.json_obj/0","type":"type"},{"doc":"","ref":"Antikythera.Httpc.ReqBody.html#t:t/0","title":"Antikythera.Httpc.ReqBody.t/0","type":"type"},{"doc":"A struct to represent an HTTP response.\n\nResponse headers are converted to a `Antikythera.Http.Headers.t` and all header names are lower-cased.\n`set-cookie` response headers are handled separately and stored in `cookies` field as a `Antikythera.Http.SetCookiesMap.t`.","ref":"Antikythera.Httpc.Response.html","title":"Antikythera.Httpc.Response","type":"module"},{"doc":"Type-aware getter for body.","ref":"Antikythera.Httpc.Response.html#body/1","title":"Antikythera.Httpc.Response.body/1","type":"function"},{"doc":"Type-aware setter for body.","ref":"Antikythera.Httpc.Response.html#body/2","title":"Antikythera.Httpc.Response.body/2","type":"function"},{"doc":"Type-aware getter for cookies.","ref":"Antikythera.Httpc.Response.html#cookies/1","title":"Antikythera.Httpc.Response.cookies/1","type":"function"},{"doc":"Type-aware setter for cookies.","ref":"Antikythera.Httpc.Response.html#cookies/2","title":"Antikythera.Httpc.Response.cookies/2","type":"function"},{"doc":"Type-aware getter for headers.","ref":"Antikythera.Httpc.Response.html#headers/1","title":"Antikythera.Httpc.Response.headers/1","type":"function"},{"doc":"Type-aware setter for headers.","ref":"Antikythera.Httpc.Response.html#headers/2","title":"Antikythera.Httpc.Response.headers/2","type":"function"},{"doc":"Creates a new instance of Antikythera.Httpc.Response by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Httpc.Response.html#new/1","title":"Antikythera.Httpc.Response.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Httpc.Response.html#new!/1","title":"Antikythera.Httpc.Response.new!/1","type":"function"},{"doc":"Type-aware getter for status.","ref":"Antikythera.Httpc.Response.html#status/1","title":"Antikythera.Httpc.Response.status/1","type":"function"},{"doc":"Type-aware setter for status.","ref":"Antikythera.Httpc.Response.html#status/2","title":"Antikythera.Httpc.Response.status/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.Httpc.Response with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Httpc.Response.html#update/2","title":"Antikythera.Httpc.Response.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Httpc.Response.html#update!/2","title":"Antikythera.Httpc.Response.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Httpc.Response.html#valid?/1","title":"Antikythera.Httpc.Response.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Httpc.Response.html#t:t/0","title":"Antikythera.Httpc.Response.t/0","type":"type"},{"doc":"An IMF-fixdate format of date/time. Used in HTTP headers.\nOnly 'GMT' is allowed as timezone string.","ref":"Antikythera.ImfFixdate.html","title":"Antikythera.ImfFixdate","type":"module"},{"doc":"","ref":"Antikythera.ImfFixdate.html#pattern/0","title":"Antikythera.ImfFixdate.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.ImfFixdate.html#valid?/1","title":"Antikythera.ImfFixdate.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.ImfFixdate.html#t:t/0","title":"Antikythera.ImfFixdate.t/0","type":"type"},{"doc":"Functions to parse/compare IP addresses.","ref":"Antikythera.IpAddress.html","title":"Antikythera.IpAddress","type":"module"},{"doc":"","ref":"Antikythera.IpAddress.V4.html","title":"Antikythera.IpAddress.V4","type":"module"},{"doc":"","ref":"Antikythera.IpAddress.V4.html#parse/1","title":"Antikythera.IpAddress.V4.parse/1","type":"function"},{"doc":"","ref":"Antikythera.IpAddress.V4.html#parse!/1","title":"Antikythera.IpAddress.V4.parse!/1","type":"function"},{"doc":"","ref":"Antikythera.IpAddress.V4.html#parse_range!/1","title":"Antikythera.IpAddress.V4.parse_range!/1","type":"function"},{"doc":"","ref":"Antikythera.IpAddress.V4.html#range_include?/2","title":"Antikythera.IpAddress.V4.range_include?/2","type":"function"},{"doc":"","ref":"Antikythera.IpAddress.V4.html#t:range/0","title":"Antikythera.IpAddress.V4.range/0","type":"type"},{"doc":"","ref":"Antikythera.IpAddress.V4.html#t:u32/0","title":"Antikythera.IpAddress.V4.u32/0","type":"type"},{"doc":"","ref":"Antikythera.IpAddress.V6.html","title":"Antikythera.IpAddress.V6","type":"module"},{"doc":"A strict subset of ISO8601 format of timestamp.","ref":"Antikythera.IsoTimestamp.html","title":"Antikythera.IsoTimestamp","type":"module"},{"doc":"","ref":"Antikythera.IsoTimestamp.html#valid?/1","title":"Antikythera.IsoTimestamp.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.IsoTimestamp.html#t:t/0","title":"Antikythera.IsoTimestamp.t/0","type":"type"},{"doc":"ISO8601 basic format.","ref":"Antikythera.IsoTimestamp.Basic.html","title":"Antikythera.IsoTimestamp.Basic","type":"module"},{"doc":"","ref":"Antikythera.IsoTimestamp.Basic.html#valid?/1","title":"Antikythera.IsoTimestamp.Basic.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.IsoTimestamp.Basic.html#t:t/0","title":"Antikythera.IsoTimestamp.Basic.t/0","type":"type"},{"doc":"Utilities to manipulate Elixir AST.","ref":"Antikythera.MacroUtil.html","title":"Antikythera.MacroUtil","type":"module"},{"doc":"","ref":"Antikythera.MacroUtil.html#prewalk_accumulate/3","title":"Antikythera.MacroUtil.prewalk_accumulate/3","type":"function"},{"doc":"Utility functions to work with maps.","ref":"Antikythera.MapUtil.html","title":"Antikythera.MapUtil","type":"module"},{"doc":"","ref":"Antikythera.MapUtil.html#difference/2","title":"Antikythera.MapUtil.difference/2","type":"function"},{"doc":"","ref":"Antikythera.MapUtil.html#map_values/2","title":"Antikythera.MapUtil.map_values/2","type":"function"},{"doc":"Update the `key` in `map` with the given function, only when the `key` exists.\nUnlike `Map.update!/3`, it returns `:error` if the `key` does not exist.","ref":"Antikythera.MapUtil.html#update_existing/3","title":"Antikythera.MapUtil.update_existing/3","type":"function"},{"doc":"Easy-to-use in-memory cache for each executor pool.\n\nAntikythera.Memcache behaves as a key-value storage.\nCached key-value pairs are internally stored in ETS.\nIt accepts arbitrary (but not too large) terms as both keys and values.","ref":"Antikythera.Memcache.html","title":"Antikythera.Memcache","type":"module"},{"doc":"iex> Antikythera.Memcache.write(\"foo\", \"bar\", epool_id, 3_600)\n    :ok\n    iex> Antikythera.Memcache.read(\"foo\", epool_id)\n    {:ok, \"bar\"}","ref":"Antikythera.Memcache.html#module-usage","title":"Usage - Antikythera.Memcache","type":"module"},{"doc":"The number of records and the size of keys and values are limited.\n\n- The maximum number of records for each executor pool\n  is 100.\n    - If exceeds the limit, a record nearest to expiration is evicted so that a new record can be inserted.\n- The maximum size of keys and values is defined in `Antikythera.Memcache.Key` and `Antikythera.Memcache.Value`.\n    - To know how the size of keys and values is calculated, see `Antikythera.TermUtil`.\n    - If exceeds the limit, `write/5` returns an error `:too_large_key` or `:too_large_value`.","ref":"Antikythera.Memcache.html#module-limits","title":"Limits - Antikythera.Memcache","type":"module"},{"doc":"There are 2 cases where records in Antikythera.Memcache are evicted:\n\n1. Records are expired (see [Mechanism of Expiration](#module-mechanism-of-expiration) below for more details)\n2. Reach the maximum number of records for each executor pool (as mentioned in [Limits](#module-limits))\n\nPlease note that records in Antikythera.Memcache could be evicted anytime.","ref":"Antikythera.Memcache.html#module-lifetime-of-records","title":"Lifetime of records - Antikythera.Memcache","type":"module"},{"doc":"The lifetime of records must be set as `lifetime_in_sec` in `write/5`.\nThis lifetime does not guarantee that records remain in the entire specified lifetime.\n\nTo avoid the [thundering herd](https://en.wikipedia.org/wiki/Thundering_herd_problem), whether records are expired is decided probabilistically.\nThe probability of expiration is shown in the following.\n\n![Mechanism of Expiration](assets/MemcacheExpiration.png)\n\nIf the thundering herd becomes a big problem, adjust `prob_lifetime_ratio` in `write/5`.","ref":"Antikythera.Memcache.html#module-mechanism-of-expiration","title":"Mechanism of Expiration - Antikythera.Memcache","type":"module"},{"doc":"Read the value associated with the `key` from Antikythera.Memcache.\n\nPlease note that records in Antikythera.Memcache could be evicted anytime so the error handling must be needed.","ref":"Antikythera.Memcache.html#read/2","title":"Antikythera.Memcache.read/2","type":"function"},{"doc":"Try to read a value associated with the `key` from Antikythera.Memcache and if that fails,\nwrite a value returned by `fun` to Antikythera.Memcache.\n\n`fun` is evaluated only if a value is not found,\nand the new value returned by `fun` is stored in Antikythera.Memcache.\nIf a value is found in Antikythera.Memcache or writing the new value to Antikythera.Memcache succeeds,\nthe value is returned as `{:ok, value}`, but if writing the new value fails, an error is returned in the same manner as `write/5`.\n\nParameters `lifetime_in_sec` and `prob_lifetime_ratio` are used to call `write/5` and the details are described above.","ref":"Antikythera.Memcache.html#read_or_else_write/5","title":"Antikythera.Memcache.read_or_else_write/5","type":"function"},{"doc":"Write a key-value pair to Antikythera.Memcache.\n\nSee above descriptions for more details.","ref":"Antikythera.Memcache.html#write/5","title":"Antikythera.Memcache.write/5","type":"function"},{"doc":"A type module of keys for `Antikythera.Memcache`.\n\nThe maximum size of keys is 128 bytes.\nTo know how the size is calculated, see `Antikythera.TermUtil`.","ref":"Antikythera.Memcache.Key.html","title":"Antikythera.Memcache.Key","type":"module"},{"doc":"","ref":"Antikythera.Memcache.Key.html#max_size/0","title":"Antikythera.Memcache.Key.max_size/0","type":"function"},{"doc":"","ref":"Antikythera.Memcache.Key.html#valid?/1","title":"Antikythera.Memcache.Key.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Memcache.Key.html#t:t/0","title":"Antikythera.Memcache.Key.t/0","type":"type"},{"doc":"","ref":"Antikythera.Memcache.NormalizedFloat.html","title":"Antikythera.Memcache.NormalizedFloat","type":"module"},{"doc":"","ref":"Antikythera.Memcache.NormalizedFloat.html#max/0","title":"Antikythera.Memcache.NormalizedFloat.max/0","type":"function"},{"doc":"","ref":"Antikythera.Memcache.NormalizedFloat.html#min/0","title":"Antikythera.Memcache.NormalizedFloat.min/0","type":"function"},{"doc":"","ref":"Antikythera.Memcache.NormalizedFloat.html#valid?/1","title":"Antikythera.Memcache.NormalizedFloat.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Memcache.NormalizedFloat.html#t:t/0","title":"Antikythera.Memcache.NormalizedFloat.t/0","type":"type"},{"doc":"A type module of values for `Antikythera.Memcache`.\n\nThe maximum size of values is 65536 bytes.\nTo know how the size is calculated, see `Antikythera.TermUtil`.","ref":"Antikythera.Memcache.Value.html","title":"Antikythera.Memcache.Value","type":"module"},{"doc":"","ref":"Antikythera.Memcache.Value.html#max_size/0","title":"Antikythera.Memcache.Value.max_size/0","type":"function"},{"doc":"","ref":"Antikythera.Memcache.Value.html#valid?/1","title":"Antikythera.Memcache.Value.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Memcache.Value.html#t:t/0","title":"Antikythera.Memcache.Value.t/0","type":"type"},{"doc":"","ref":"Antikythera.Metrics.Data.html","title":"Antikythera.Metrics.Data","type":"module"},{"doc":"","ref":"Antikythera.Metrics.Data.html#valid?/1","title":"Antikythera.Metrics.Data.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Metrics.Data.html#t:t/0","title":"Antikythera.Metrics.Data.t/0","type":"type"},{"doc":"","ref":"Antikythera.Metrics.DataList.html","title":"Antikythera.Metrics.DataList","type":"module"},{"doc":"","ref":"Antikythera.Metrics.DataList.html#min_length/0","title":"Antikythera.Metrics.DataList.min_length/0","type":"function"},{"doc":"","ref":"Antikythera.Metrics.DataList.html#valid?/1","title":"Antikythera.Metrics.DataList.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Metrics.DataList.html#t:t/0","title":"Antikythera.Metrics.DataList.t/0","type":"type"},{"doc":"","ref":"Antikythera.MilliSecondsInGregorian.html","title":"Antikythera.MilliSecondsInGregorian","type":"module"},{"doc":"","ref":"Antikythera.MilliSecondsInGregorian.html#min/0","title":"Antikythera.MilliSecondsInGregorian.min/0","type":"function"},{"doc":"","ref":"Antikythera.MilliSecondsInGregorian.html#time_epoch_offset_milliseconds/0","title":"Antikythera.MilliSecondsInGregorian.time_epoch_offset_milliseconds/0","type":"function"},{"doc":"","ref":"Antikythera.MilliSecondsInGregorian.html#valid?/1","title":"Antikythera.MilliSecondsInGregorian.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.MilliSecondsInGregorian.html#t:t/0","title":"Antikythera.MilliSecondsInGregorian.t/0","type":"type"},{"doc":"","ref":"Antikythera.MilliSecondsSinceEpoch.html","title":"Antikythera.MilliSecondsSinceEpoch","type":"module"},{"doc":"","ref":"Antikythera.MilliSecondsSinceEpoch.html#min/0","title":"Antikythera.MilliSecondsSinceEpoch.min/0","type":"function"},{"doc":"","ref":"Antikythera.MilliSecondsSinceEpoch.html#valid?/1","title":"Antikythera.MilliSecondsSinceEpoch.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.MilliSecondsSinceEpoch.html#t:t/0","title":"Antikythera.MilliSecondsSinceEpoch.t/0","type":"type"},{"doc":"Helper functions for making mix tasks in gears.\n\n**Functions in this module can only be used in mix tasks.**","ref":"Antikythera.Mix.Task.html","title":"Antikythera.Mix.Task","type":"module"},{"doc":"Starts the current antikythera instance and its dependency applications without web server functionality.\n\nIf you need web server functionality in your mix task,\nuse `Application.ensure_all_started(Antikythera.Env.antikythera_instance_name())`.","ref":"Antikythera.Mix.Task.html#prepare_antikythera_instance/0","title":"Antikythera.Mix.Task.prepare_antikythera_instance/0","type":"function"},{"doc":"Set the specified `node_id` to the `Antikythera.ContextId` in GearLog.\n\nIf you want to use GearLog in mix task, you must set `node_id` before you call GearLog functions.\n\nThe `Antikythera.ContextId` in GearLog will become `{timestamp}_{node_id}_{PID}`.\nThe `timestamp` and `PID` are automatically got from system.","ref":"Antikythera.Mix.Task.html#set_node_id_to_gear_log_context/1","title":"Antikythera.Mix.Task.set_node_id_to_gear_log_context/1","type":"function"},{"doc":"Utility functions to work with nested maps.","ref":"Antikythera.NestedMap.html","title":"Antikythera.NestedMap","type":"module"},{"doc":"","ref":"Antikythera.NestedMap.html#deep_merge/2","title":"Antikythera.NestedMap.deep_merge/2","type":"function"},{"doc":"","ref":"Antikythera.NestedMap.html#force_update/3","title":"Antikythera.NestedMap.force_update/3","type":"function"},{"doc":"Updates a key in a nested map. Recursively traversing the map according to the given `keys`\nto the last member. If the last member exists, update it with the given function.\nUnlike `Kernel.update_in/3`, returns `:error` if any of the `keys` cannot be found.","ref":"Antikythera.NestedMap.html#update_existing_in/3","title":"Antikythera.NestedMap.update_existing_in/3","type":"function"},{"doc":"","ref":"Antikythera.NodeId.html","title":"Antikythera.NodeId","type":"module"},{"doc":"","ref":"Antikythera.NodeId.html#pattern/0","title":"Antikythera.NodeId.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.NodeId.html#pattern_string/0","title":"Antikythera.NodeId.pattern_string/0","type":"function"},{"doc":"","ref":"Antikythera.NodeId.html#valid?/1","title":"Antikythera.NodeId.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.NodeId.html#t:t/0","title":"Antikythera.NodeId.t/0","type":"type"},{"doc":"","ref":"Antikythera.PathInfo.html","title":"Antikythera.PathInfo","type":"module"},{"doc":"","ref":"Antikythera.PathInfo.html#valid?/1","title":"Antikythera.PathInfo.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.PathInfo.html#t:t/0","title":"Antikythera.PathInfo.t/0","type":"type"},{"doc":"A type module to represent URI-encoded segment of URL path.\n\nSee [RFC3986](https://tools.ietf.org/html/rfc3986#section-3.3) for the specifications of URI path.\nNote that this module accepts empty strings.","ref":"Antikythera.PathSegment.html","title":"Antikythera.PathSegment","type":"module"},{"doc":"","ref":"Antikythera.PathSegment.html#charclass/0","title":"Antikythera.PathSegment.charclass/0","type":"function"},{"doc":"","ref":"Antikythera.PathSegment.html#pattern/0","title":"Antikythera.PathSegment.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.PathSegment.html#valid?/1","title":"Antikythera.PathSegment.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.PathSegment.html#t:t/0","title":"Antikythera.PathSegment.t/0","type":"type"},{"doc":"Plug to restrict access to controller action by basic authentication.","ref":"Antikythera.Plug.BasicAuthentication.html","title":"Antikythera.Plug.BasicAuthentication","type":"module"},{"doc":"#","ref":"Antikythera.Plug.BasicAuthentication.html#module-usage","title":"Usage - Antikythera.Plug.BasicAuthentication","type":"module"},{"doc":"The following line installs a plug that checks username/password in incoming requests against\n`\"BASIC_AUTHENTICATION_ID\"` and `\"BASIC_AUTHENTICATION_PW\"` in your gear's gear config.\n\n    plug Antikythera.Plug.BasicAuthentication, :check_with_config, []\n\n#","ref":"Antikythera.Plug.BasicAuthentication.html#module-static-username-password-using-gear-config","title":"Static username/password: using gear config - Antikythera.Plug.BasicAuthentication","type":"module"},{"doc":"In the following case you can check the username/password pair with arbitrary logic by the specified function.\n\n    plug Antikythera.Plug.BasicAuthentication, :check_with_fun, [mod: YourGear.AuthModule, fun: :function_name]\n\nThe given function (`YourGear.AuthModule.function_name/3` in this case) must have the following type signature:\n\n- receives (1) a `Antikythera.Conn.t`, (2) username, and (3) password\n- returns (1) `{:ok, Antikythera.Conn.t}` if successfully authenticated, (2) `:error` otherwise","ref":"Antikythera.Plug.BasicAuthentication.html#module-dynamic-username-password-using-module-function-pair","title":"Dynamic username/password: using module-function pair. - Antikythera.Plug.BasicAuthentication","type":"module"},{"doc":"","ref":"Antikythera.Plug.BasicAuthentication.html#check_with_config/2","title":"Antikythera.Plug.BasicAuthentication.check_with_config/2","type":"function"},{"doc":"","ref":"Antikythera.Plug.BasicAuthentication.html#check_with_fun/2","title":"Antikythera.Plug.BasicAuthentication.check_with_fun/2","type":"function"},{"doc":"Plug to decompress the request body when the request has `Content-Encoding` header.\n\nCurrently only `gzip` is supported.\n\nAntikythera does not transparently handle compressed request body, as [cowboy](https://github.com/ninenines/cowboy/issues/946).\nCompression can pack very large data into a small body, so automatically decompressing it may lead to running out of memory.\n\nTherefore, this plug can be used only if the request client is trusted (e.g. after authentication).","ref":"Antikythera.Plug.ContentDecoding.html","title":"Antikythera.Plug.ContentDecoding","type":"module"},{"doc":"Put the following line in your controller module:\n\n    plug Antikythera.Plug.ContentDecoding, :decode, []","ref":"Antikythera.Plug.ContentDecoding.html#module-usage","title":"Usage - Antikythera.Plug.ContentDecoding","type":"module"},{"doc":"","ref":"Antikythera.Plug.ContentDecoding.html#decode/2","title":"Antikythera.Plug.ContentDecoding.decode/2","type":"function"},{"doc":"Plug to automatically load flash messages from session.","ref":"Antikythera.Plug.Flash.html","title":"Antikythera.Plug.Flash","type":"module"},{"doc":"Add the following line in a controller module (note that the order of the plugs is important):\n\n    plug Antikythera.Plug.Session, :load, [key: \"12345678\"]\n    plug Antikythera.Plug.Flash  , :load, []\n\nThen you can access flash messages by\n\n- `Antikythera.Conn.get_flash/2`\n- `Antikythera.Conn.put_flash/3`","ref":"Antikythera.Plug.Flash.html#module-usage","title":"Usage - Antikythera.Plug.Flash","type":"module"},{"doc":"","ref":"Antikythera.Plug.Flash.html#load/2","title":"Antikythera.Plug.Flash.load/2","type":"function"},{"doc":"Plug to restrict access to controller action only from within specified IP ranges.","ref":"Antikythera.Plug.IpFiltering.html","title":"Antikythera.Plug.IpFiltering","type":"module"},{"doc":"#","ref":"Antikythera.Plug.IpFiltering.html#module-usage","title":"Usage - Antikythera.Plug.IpFiltering","type":"module"},{"doc":"The following lines reject requests from IP not within the 2 ranges, `\"123.45.67.0/24\", \"135.79.135.0/24\"`.\n\n    ranges = Enum.map([\"123.45.67.0/24\", \"135.79.135.0/24\"], &Antikythera.IpAddress.V4.parse!/1)\n    plug Antikythera.Plug.IpFiltering, :check_by_static_ranges, [ranges: ranges]\n\nNote that this plug accepts only parsed result and not string, in order to avoid parsing the given strings on every request.\n\n#","ref":"Antikythera.Plug.IpFiltering.html#module-static-ip-ranges-specified-as-a-plug-argument","title":"Static IP ranges specified as a plug argument - Antikythera.Plug.IpFiltering","type":"module"},{"doc":"The following line uses `\"ALLOWED_IP_RANGES\"` field in the gear config as the list of allowed IP ranges.\n\n    plug Antikythera.Plug.IpFiltering, :check_by_gear_config, []\n\nThe field name can be customized by giving `:field_name` option as follows:\n\n    plug Antikythera.Plug.IpFiltering, :check_by_gear_config, [field_name: \"ALLOWED_IP_RANGES_2\"]","ref":"Antikythera.Plug.IpFiltering.html#module-dynamic-ip-ranges-specified-by-gear-config","title":"Dynamic IP ranges specified by gear config - Antikythera.Plug.IpFiltering","type":"module"},{"doc":"Both plug functions explained above reject not only web requests from outside of the specified IP ranges but also gear-to-gear requests.\nIf you want to restrict web requests and at the same time allow gear-to-gear requests, pass `:allow_g2g` option.\n\n    plug Antikythera.Plug.IpFiltering, :check_by_gear_config, [allow_g2g: true]","ref":"Antikythera.Plug.IpFiltering.html#module-gear-to-gear-requests","title":"gear-to-gear requests - Antikythera.Plug.IpFiltering","type":"module"},{"doc":"","ref":"Antikythera.Plug.IpFiltering.html#check_by_gear_config/2","title":"Antikythera.Plug.IpFiltering.check_by_gear_config/2","type":"function"},{"doc":"","ref":"Antikythera.Plug.IpFiltering.html#check_by_static_ranges/2","title":"Antikythera.Plug.IpFiltering.check_by_static_ranges/2","type":"function"},{"doc":"","ref":"Antikythera.Plug.IpFiltering.html#run_check_on_cloud/3","title":"Antikythera.Plug.IpFiltering.run_check_on_cloud/3","type":"function"},{"doc":"","ref":"Antikythera.Plug.IpFiltering.html#t:arg_gear_config/0","title":"Antikythera.Plug.IpFiltering.arg_gear_config/0","type":"type"},{"doc":"","ref":"Antikythera.Plug.IpFiltering.html#t:arg_static/0","title":"Antikythera.Plug.IpFiltering.arg_static/0","type":"type"},{"doc":"Plug to set a fixed value of `cache-control` response header in order to disable client side caching.\n\nThe value used is: `\"private, no-cache, no-store, max-age=0\"`.","ref":"Antikythera.Plug.NoCache.html","title":"Antikythera.Plug.NoCache","type":"module"},{"doc":"Put the following line in your controller module:\n\n    plug Antikythera.Plug.NoCache, :put_resp_header, []\n\nNote that this plug runs before your controller's actions; in your actions you can overwrite the `cache-control` header set by this plug.","ref":"Antikythera.Plug.NoCache.html#module-usage","title":"Usage - Antikythera.Plug.NoCache","type":"module"},{"doc":"","ref":"Antikythera.Plug.NoCache.html#header_value/0","title":"Antikythera.Plug.NoCache.header_value/0","type":"function"},{"doc":"","ref":"Antikythera.Plug.NoCache.html#put_resp_header/2","title":"Antikythera.Plug.NoCache.put_resp_header/2","type":"function"},{"doc":"Plug to automatically load/store session information using a specific session store.\nUses cookie store by default.","ref":"Antikythera.Plug.Session.html","title":"Antikythera.Plug.Session","type":"module"},{"doc":"Adding the following line in a controller module enables this plug:\n\n    plug Antikythera.Plug.Session, :load, [key: \"12345678\"]\n\nThen,\n- session values are loaded from cookie before controller action is executed, and\n- session values are stored into cookie after controller action is executed.","ref":"Antikythera.Plug.Session.html#module-usage","title":"Usage - Antikythera.Plug.Session","type":"module"},{"doc":"","ref":"Antikythera.Plug.Session.html#load/2","title":"Antikythera.Plug.Session.load/2","type":"function"},{"doc":"","ref":"Antikythera.Plug.Session.html#t:load_option_t/0","title":"Antikythera.Plug.Session.load_option_t/0","type":"type"},{"doc":"","ref":"Antikythera.Registry.html","title":"Antikythera.Registry","type":"module"},{"doc":"","ref":"Antikythera.Registry.html#t:name/0","title":"Antikythera.Registry.name/0","type":"type"},{"doc":"A global (cluster-wide) process registry for implementing publisher-subscriber communication pattern.\n\nIn this registry you can register multiple processes with the same name.\nThen you can broadcast a message to the group of processes having the same name.\nGroup names can be arbitrary string.\nEach process can join multiple groups at the same time.\nWhen the registered process dies its pid will be automatically removed from all the groups that the process has joined.","ref":"Antikythera.Registry.Group.html","title":"Antikythera.Registry.Group","type":"module"},{"doc":"","ref":"Antikythera.Registry.Group.html#join/2","title":"Antikythera.Registry.Group.join/2","type":"function"},{"doc":"","ref":"Antikythera.Registry.Group.html#leave/2","title":"Antikythera.Registry.Group.leave/2","type":"function"},{"doc":"","ref":"Antikythera.Registry.Group.html#publish/3","title":"Antikythera.Registry.Group.publish/3","type":"function"},{"doc":"A global (cluster-wide) process registry to implement 1-to-1 process communications.\n\nProcess names can be arbitrary string as long as it's unique within an executor pool.\nEach process is not allowed to have more than one name.\nWhen the registered process dies its name will be automatically removed from the registry.\n\nNote that the uniqueness of names is not strictly checked;\nin case of race conditions of simultaneous registrations on different nodes,\nmultiple processes successfully register the same name.\nThe conflict will be resolved by choosing a single process and killing the others.\nTo avoid troubles with this kind of naming conflicts,\nit's recommended to use intrinsically unique IDs such as login IDs of clients.","ref":"Antikythera.Registry.Unique.html","title":"Antikythera.Registry.Unique","type":"module"},{"doc":"","ref":"Antikythera.Registry.Unique.html#register/2","title":"Antikythera.Registry.Unique.register/2","type":"function"},{"doc":"","ref":"Antikythera.Registry.Unique.html#send_message/3","title":"Antikythera.Registry.Unique.send_message/3","type":"function"},{"doc":"Definition of `Antikythera.Request` struct.","ref":"Antikythera.Request.html","title":"Antikythera.Request","type":"module"},{"doc":"Type-aware getter for body.","ref":"Antikythera.Request.html#body/1","title":"Antikythera.Request.body/1","type":"function"},{"doc":"Type-aware setter for body.","ref":"Antikythera.Request.html#body/2","title":"Antikythera.Request.body/2","type":"function"},{"doc":"Type-aware getter for cookies.","ref":"Antikythera.Request.html#cookies/1","title":"Antikythera.Request.cookies/1","type":"function"},{"doc":"Type-aware setter for cookies.","ref":"Antikythera.Request.html#cookies/2","title":"Antikythera.Request.cookies/2","type":"function"},{"doc":"Type-aware getter for headers.","ref":"Antikythera.Request.html#headers/1","title":"Antikythera.Request.headers/1","type":"function"},{"doc":"Type-aware setter for headers.","ref":"Antikythera.Request.html#headers/2","title":"Antikythera.Request.headers/2","type":"function"},{"doc":"Type-aware getter for method.","ref":"Antikythera.Request.html#method/1","title":"Antikythera.Request.method/1","type":"function"},{"doc":"Type-aware setter for method.","ref":"Antikythera.Request.html#method/2","title":"Antikythera.Request.method/2","type":"function"},{"doc":"Creates a new instance of Antikythera.Request by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Request.html#new/1","title":"Antikythera.Request.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Request.html#new!/1","title":"Antikythera.Request.new!/1","type":"function"},{"doc":"Type-aware getter for path_info.","ref":"Antikythera.Request.html#path_info/1","title":"Antikythera.Request.path_info/1","type":"function"},{"doc":"Type-aware setter for path_info.","ref":"Antikythera.Request.html#path_info/2","title":"Antikythera.Request.path_info/2","type":"function"},{"doc":"Type-aware getter for path_matches.","ref":"Antikythera.Request.html#path_matches/1","title":"Antikythera.Request.path_matches/1","type":"function"},{"doc":"Type-aware setter for path_matches.","ref":"Antikythera.Request.html#path_matches/2","title":"Antikythera.Request.path_matches/2","type":"function"},{"doc":"Type-aware getter for query_params.","ref":"Antikythera.Request.html#query_params/1","title":"Antikythera.Request.query_params/1","type":"function"},{"doc":"Type-aware setter for query_params.","ref":"Antikythera.Request.html#query_params/2","title":"Antikythera.Request.query_params/2","type":"function"},{"doc":"Type-aware getter for raw_body.","ref":"Antikythera.Request.html#raw_body/1","title":"Antikythera.Request.raw_body/1","type":"function"},{"doc":"Type-aware setter for raw_body.","ref":"Antikythera.Request.html#raw_body/2","title":"Antikythera.Request.raw_body/2","type":"function"},{"doc":"Type-aware getter for sender.","ref":"Antikythera.Request.html#sender/1","title":"Antikythera.Request.sender/1","type":"function"},{"doc":"Type-aware setter for sender.","ref":"Antikythera.Request.html#sender/2","title":"Antikythera.Request.sender/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.Request with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Request.html#update/2","title":"Antikythera.Request.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Request.html#update!/2","title":"Antikythera.Request.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Request.html#valid?/1","title":"Antikythera.Request.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Request.html#t:t/0","title":"Antikythera.Request.t/0","type":"type"},{"doc":"","ref":"Antikythera.Request.PathMatches.html","title":"Antikythera.Request.PathMatches","type":"module"},{"doc":"","ref":"Antikythera.Request.PathMatches.html#valid?/1","title":"Antikythera.Request.PathMatches.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Request.PathMatches.html#t:t/0","title":"Antikythera.Request.PathMatches.t/0","type":"type"},{"doc":"","ref":"Antikythera.Request.Sender.html","title":"Antikythera.Request.Sender","type":"module"},{"doc":"","ref":"Antikythera.Request.Sender.html#valid?/1","title":"Antikythera.Request.Sender.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Request.Sender.html#t:sender_ip/0","title":"Antikythera.Request.Sender.sender_ip/0","type":"type"},{"doc":"","ref":"Antikythera.Request.Sender.html#t:t/0","title":"Antikythera.Request.Sender.t/0","type":"type"},{"doc":"Defines the antikythera routing DSL.","ref":"Antikythera.Router.html","title":"Antikythera.Router","type":"module"},{"doc":"This module defines macros to be used in each gear's Router module.\nThe names of the macros are the same as the HTTP verbs: `get`, `post`, etc.\n\nThe macros take the following 4 arguments (although you can omit the last and just pass 3 of them):\n\n- URL path pattern which consists of '/'-separated segments. The 1st character must be '/'.\n  To match against incoming request path to a pattern you can use placeholders. See examples below for the usage.\n- Controller module.\n  Antikythera expects that the module name given here does not contain `GearName.Controller.` as a prefix;\n  it's automatically prepended by antikythera.\n- Name of the controller action as an atom.\n- Keyword list of options.\n  Currently available options are `:from` and `:as`. See below for further explanations.","ref":"Antikythera.Router.html#module-routing-macros","title":"Routing macros - Antikythera.Router","type":"module"},{"doc":"If you define the following router module,\n\n    defmodule MyGear.Router do\n      use Antikythera.Router\n\n      static_prefix \"/static\"\n\n      websocket \"/ws\"\n\n      get  \"/foo\"       , Hello, :exact_match\n      post \"/foo/:a/:b\" , Hello, :placeholders\n      put  \"/foo/bar/*w\", Hello, :wildcard\n    end\n\nThen the following requests are routed as:\n\n- `GET  \"/foo\"`                 => `MyGear.Controller.Hello.exact_match/1`  is invoked with `path_matches`: `%{}`\n- `POST \"/foo/bar/baz\"`         => `MyGear.Controller.Hello.placeholders/1` is invoked with `path_matches`: `%{a: \"bar\", b: \"baz\"}`\n- `PUT  \"/foo/bar/abc/def/ghi\"` => `MyGear.Controller.Hello.wildcard/1`     is invoked with `path_matches`: `%{w: \"abc/def/ghi\"}`\n\nNote that\n\n- Each controller action is expected to receive a `Antikythera.Conn` struct and returns a `Antikythera.Conn` struct.\n- `Antikythera.Conn` struct has a field `request` which is a `Antikythera.Request` struct.\n- Matched segments are URL-decoded and stored in `path_matches` field in `Antikythera.Request`.\n  If the result of URL-decoding is nonprintable binary, the request is rejected.","ref":"Antikythera.Router.html#module-example","title":"Example - Antikythera.Router","type":"module"},{"doc":"To enable websocket interaction with clients, you must first define `MyGear.Websocket` module.\nSee `Antikythera.Websocket` for more details about websocket handler module.\nThen invoke `websocket/1` macro in your router.\n\n    websocket \"/ws_path_pattern\"\n\nThe path pattern may have placeholders in the same way as normal routes.\nGET request with appropriate headers to this path will initialize a websocket connection using the HTTP 1.1 upgrade mechanism.\n\nIf your gear does not interact with clients via websocket, simply don't invoke `websocket/1` macro in your router.","ref":"Antikythera.Router.html#module-websocket-endpoint","title":"Websocket endpoint - Antikythera.Router","type":"module"},{"doc":"You can serve your static assets by placing them under `/priv/static` directory in your gear project.\nThe endpoint to be used can be specified by `static_prefix/1` macro.\nFor example, if you add\n\n    static_prefix \"/assets\"\n\nto your router, you can download `/priv/static/html/index.html` file by sending GET request to the path `/assets/html/index.html`.\n\nIf you don't need to serve static assets, just don't call `static_prefix/1` macro in your router.\n\nCurrently, static assets served in this way are NOT automatically gzip compressed,\neven if `acceept-encoding: gzip` request header is set.\nIt is recommended to use CDN to deliver large static assets in production.\n\nSee also `Antikythera.Asset` for usage of CDN in delivery of static assets.","ref":"Antikythera.Router.html#module-static-file-serving","title":"Static file serving - Antikythera.Router","type":"module"},{"doc":"Antikythera treats both web requests and g2g requests in basically the same way.\nThis means that if you define a route in your gear one can send request to the route using both HTTP and g2g communication.\nIf you want to define a route that can be accessible only via g2g communication, specify `from: :gear` option.\n\n    get  \"/foo\", Hello, :action1, from: :gear\n    post \"/bar\", Hello, :action2, from: :gear\n\nSimilarly passing `from: :web` makes the route accessible only from web request.\n\nWhen dealing with multiple routes, `only_from_web/1` and `only_from_gear/1` macros can be used.\nFor example, the following routes definition is the same as above one.\n\n    only_from_gear do\n      get  \"/foo\", Hello, :action1\n      post \"/bar\", Hello, :action2\n    end","ref":"Antikythera.Router.html#module-web-requests-and-gear-to-gear-g2g-requests","title":"Web requests and gear-to-gear (g2g) requests - Antikythera.Router","type":"module"},{"doc":"To generate URL path of a route (e.g. a link in HTML), you will want to refer to the route's path.\nFor this purpose you can specify `:as` option.\nFor example, you have the following router module\n\n    defmodule MyGear.Router do\n      use Antikythera.Router\n\n      get \"/foo/:a/:b/*c\", Hello, :placeholders, as: :myroute\n    end\n\nBy writing this the router automatically defines a function `myroute_path/4`,\nwhich receives segments that fill placeholders and an optional map for query parameters.\n\n    MyGear.Router.myroute_path(\"segment_a\", \"segment_b\", [\"wildcard\", \"part\"])\n    => \"/foo/segment_a/segment_b/wildcard/part\n    MyGear.Router.myroute_path(\"segment_a\", \"segment_b\", [\"wildcard\", \"part\"], %{\"query\" => \"param\"})\n    => \"/foo/segment_a/segment_b/wildcard/part?query=param\n\nReverse routing helper functions automatically URI-encode all given arguments.\n\nIf websocket endpoint is enabled, you can get its path with `MyGear.Router.websocket_path/0`.\nAlso if static file serving is enabled, path prefix for static files can be obtained by `MyGear.Router.static_prefix/0`.","ref":"Antikythera.Router.html#module-reverse-routing","title":"Reverse routing - Antikythera.Router","type":"module"},{"doc":"You can specify timeout for each API by `:timeout` option.\nThe timeout is specified in milliseconds.\nFor example, the following API times out after 60 seconds.\n\n    get \"/foo\", Hello, :long_action, timeout: 60_000\n\nThe maximum timeout is determined by `:gear_action_max_timeout` configuration in `config/config.exs`.\nThe default value is 10 seconds, which can be configured by `GEAR_ACTION_TIMEOUT` environment variable.","ref":"Antikythera.Router.html#module-per-api-timeout","title":"Per-API timeout - Antikythera.Router","type":"module"},{"doc":"","ref":"Antikythera.Router.html#connect/4","title":"Antikythera.Router.connect/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#delete/4","title":"Antikythera.Router.delete/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#fully_qualified_controller_module/3","title":"Antikythera.Router.fully_qualified_controller_module/3","type":"function"},{"doc":"","ref":"Antikythera.Router.html#get/4","title":"Antikythera.Router.get/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#head/4","title":"Antikythera.Router.head/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#only_from_gear/1","title":"Antikythera.Router.only_from_gear/1","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#only_from_web/1","title":"Antikythera.Router.only_from_web/1","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#options/4","title":"Antikythera.Router.options/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#patch/4","title":"Antikythera.Router.patch/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#post/4","title":"Antikythera.Router.post/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#put/4","title":"Antikythera.Router.put/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#static_prefix/1","title":"Antikythera.Router.static_prefix/1","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#trace/4","title":"Antikythera.Router.trace/4","type":"macro"},{"doc":"","ref":"Antikythera.Router.html#websocket/2","title":"Antikythera.Router.websocket/2","type":"macro"},{"doc":"Internal functions to implement request routing.","ref":"Antikythera.Router.Impl.html","title":"Antikythera.Router.Impl","type":"module"},{"doc":"","ref":"Antikythera.Router.Impl.html#check_path_pattern/1","title":"Antikythera.Router.Impl.check_path_pattern/1","type":"function"},{"doc":"","ref":"Antikythera.Router.Impl.html#generate_route_function_clauses/3","title":"Antikythera.Router.Impl.generate_route_function_clauses/3","type":"function"},{"doc":"","ref":"Antikythera.Router.Impl.html#route_clause_body/5","title":"Antikythera.Router.Impl.route_clause_body/5","type":"function"},{"doc":"Internal helper functions for macros in `Antikythera.Router`, to generate/implement reverse routing functions.","ref":"Antikythera.Router.Reverse.html","title":"Antikythera.Router.Reverse","type":"module"},{"doc":"","ref":"Antikythera.Router.Reverse.html#define_path_helper/2","title":"Antikythera.Router.Reverse.define_path_helper/2","type":"function"},{"doc":"","ref":"Antikythera.Router.Reverse.html#make_path/3","title":"Antikythera.Router.Reverse.make_path/3","type":"function"},{"doc":"","ref":"Antikythera.SecondsSinceEpoch.html","title":"Antikythera.SecondsSinceEpoch","type":"module"},{"doc":"","ref":"Antikythera.SecondsSinceEpoch.html#min/0","title":"Antikythera.SecondsSinceEpoch.min/0","type":"function"},{"doc":"","ref":"Antikythera.SecondsSinceEpoch.html#valid?/1","title":"Antikythera.SecondsSinceEpoch.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.SecondsSinceEpoch.html#t:t/0","title":"Antikythera.SecondsSinceEpoch.t/0","type":"type"},{"doc":"Definition of data structure to work with session.\n\nGear implementations usually don't use the functions defined in this module;\ninstead use\n\n- `Antikythera.Plug.Session`\n- interfaces in `Antikythera.Conn` (e.g. `Antikythera.Conn.get_session/2`).","ref":"Antikythera.Session.html","title":"Antikythera.Session","type":"module"},{"doc":"","ref":"Antikythera.Session.html#clear/1","title":"Antikythera.Session.clear/1","type":"function"},{"doc":"Type-aware getter for data.","ref":"Antikythera.Session.html#data/1","title":"Antikythera.Session.data/1","type":"function"},{"doc":"Type-aware setter for data.","ref":"Antikythera.Session.html#data/2","title":"Antikythera.Session.data/2","type":"function"},{"doc":"","ref":"Antikythera.Session.html#delete/2","title":"Antikythera.Session.delete/2","type":"function"},{"doc":"","ref":"Antikythera.Session.html#destroy/1","title":"Antikythera.Session.destroy/1","type":"function"},{"doc":"","ref":"Antikythera.Session.html#get/2","title":"Antikythera.Session.get/2","type":"function"},{"doc":"Type-aware getter for id.","ref":"Antikythera.Session.html#id/1","title":"Antikythera.Session.id/1","type":"function"},{"doc":"Type-aware setter for id.","ref":"Antikythera.Session.html#id/2","title":"Antikythera.Session.id/2","type":"function"},{"doc":"Creates a new instance of Antikythera.Session by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Session.html#new/1","title":"Antikythera.Session.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Session.html#new!/1","title":"Antikythera.Session.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Session.html#put/3","title":"Antikythera.Session.put/3","type":"function"},{"doc":"","ref":"Antikythera.Session.html#renew/1","title":"Antikythera.Session.renew/1","type":"function"},{"doc":"Type-aware getter for state.","ref":"Antikythera.Session.html#state/1","title":"Antikythera.Session.state/1","type":"function"},{"doc":"Type-aware setter for state.","ref":"Antikythera.Session.html#state/2","title":"Antikythera.Session.state/2","type":"function"},{"doc":"Updates an existing instance of Antikythera.Session with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Session.html#update/2","title":"Antikythera.Session.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Session.html#update!/2","title":"Antikythera.Session.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Session.html#valid?/1","title":"Antikythera.Session.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Session.html#t:t/0","title":"Antikythera.Session.t/0","type":"type"},{"doc":"Implementation of `Antikythera.Session.Store` that stores session info in cookie.","ref":"Antikythera.Session.Cookie.html","title":"Antikythera.Session.Cookie","type":"module"},{"doc":"","ref":"Antikythera.Session.State.html","title":"Antikythera.Session.State","type":"module"},{"doc":"","ref":"Antikythera.Session.State.html#new/1","title":"Antikythera.Session.State.new/1","type":"function"},{"doc":"","ref":"Antikythera.Session.State.html#new!/1","title":"Antikythera.Session.State.new!/1","type":"function"},{"doc":"","ref":"Antikythera.Session.State.html#valid?/1","title":"Antikythera.Session.State.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Session.State.html#values/0","title":"Antikythera.Session.State.values/0","type":"function"},{"doc":"","ref":"Antikythera.Session.State.html#t:t/0","title":"Antikythera.Session.State.t/0","type":"type"},{"doc":"Behaviour of session store.","ref":"Antikythera.Session.Store.html","title":"Antikythera.Session.Store","type":"behaviour"},{"doc":"","ref":"Antikythera.Session.Store.html#c:delete/1","title":"Antikythera.Session.Store.delete/1","type":"callback"},{"doc":"","ref":"Antikythera.Session.Store.html#c:load/1","title":"Antikythera.Session.Store.load/1","type":"callback"},{"doc":"","ref":"Antikythera.Session.Store.html#c:save/2","title":"Antikythera.Session.Store.save/2","type":"callback"},{"doc":"","ref":"Antikythera.Session.Store.html#t:session_id/0","title":"Antikythera.Session.Store.session_id/0","type":"type"},{"doc":"","ref":"Antikythera.Session.Store.html#t:session_kv/0","title":"Antikythera.Session.Store.session_kv/0","type":"type"},{"doc":"","ref":"Antikythera.StringFormat.html","title":"Antikythera.StringFormat","type":"module"},{"doc":"","ref":"Antikythera.StringFormat.html#pad2/1","title":"Antikythera.StringFormat.pad2/1","type":"function"},{"doc":"","ref":"Antikythera.StringFormat.html#pad3/1","title":"Antikythera.StringFormat.pad3/1","type":"function"},{"doc":"Definition of macro to precompile HAML templates.\n\nEach gear's template module (web/template.ex) must use this module as follows:\n\n    defmodule YourGear.Template do\n      use Antikythera.TemplatePrecompiler\n    end\n\nHAML files whose paths match `web/template/**/*.html.haml` are loaded and converted into function clauses at compile time.\nTo render HAML files in controller actions, use `Antikythera.Conn.render/5`.\n\nAs all macro-generated function clauses of `content_for/2` reside in `YourGear.Template`,\nyou can, for example, put `alias` before `use Antikythera.TemplatePrecompiler` so that it takes effect in all HAML templates.","ref":"Antikythera.TemplatePrecompiler.html","title":"Antikythera.TemplatePrecompiler","type":"module"},{"doc":"","ref":"Antikythera.TemplateSanitizer.html","title":"Antikythera.TemplateSanitizer","type":"module"},{"doc":"Converts special characters in the given string to character entity references.","ref":"Antikythera.TemplateSanitizer.html#html_escape/1","title":"Antikythera.TemplateSanitizer.html_escape/1","type":"function"},{"doc":"Marks data as HTML safe to avoid double-escaping.","ref":"Antikythera.TemplateSanitizer.html#raw/1","title":"Antikythera.TemplateSanitizer.raw/1","type":"function"},{"doc":"","ref":"Antikythera.TemplateSanitizer.html#t:safe/0","title":"Antikythera.TemplateSanitizer.safe/0","type":"type"},{"doc":"","ref":"Antikythera.TenantId.html","title":"Antikythera.TenantId","type":"module"},{"doc":"","ref":"Antikythera.TenantId.html#notenant/0","title":"Antikythera.TenantId.notenant/0","type":"function"},{"doc":"","ref":"Antikythera.TenantId.html#pattern/0","title":"Antikythera.TenantId.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.TenantId.html#valid?/1","title":"Antikythera.TenantId.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.TenantId.html#t:t/0","title":"Antikythera.TenantId.t/0","type":"type"},{"doc":"Utils for calculating the actual size of terms.\n\nThese utils traverse terms and accumulate the size of terms including the actual size of binary.","ref":"Antikythera.TermUtil.html","title":"Antikythera.TermUtil","type":"module"},{"doc":"Returns the actual size of `term` in bytes.","ref":"Antikythera.TermUtil.html#size/1","title":"Antikythera.TermUtil.size/1","type":"function"},{"doc":"Returns whether actual size of `term` exceeds `limit` bytes.\n\nThis is more efficient than deciding by using `size/1` because this function returns immediately after exceeding `limit`, not traverses the entire term.","ref":"Antikythera.TermUtil.html#size_smaller_or_equal?/2","title":"Antikythera.TermUtil.size_smaller_or_equal?/2","type":"function"},{"doc":"Helpers for tests that target async jobs.","ref":"Antikythera.Test.AsyncJobHelper.html","title":"Antikythera.Test.AsyncJobHelper","type":"module"},{"doc":"Resets the token bucket that rate-limits accesses to the specified async job queue.\n\nThis function is useful when you hit the rate limit in your tests.\nYou can accelerate your test execution by resetting the token bucket.\nSee `Antikythera.AsyncJob` for more details about rate limiting.","ref":"Antikythera.Test.AsyncJobHelper.html#reset_rate_limit_status/1","title":"Antikythera.Test.AsyncJobHelper.reset_rate_limit_status/1","type":"function"},{"doc":"Helpers to be used in each gear's `test/test_helper.exs`.\n\nAt the top of `test/test_helper.exs`, all gears must place the following line:\n\n    Antikythera.Test.Config.init()\n\nThis line will start [`ExUnit`](http://elixir-lang.org/docs/stable/ex_unit/) and\nset configurations for whitebox/blackbox test mode.","ref":"Antikythera.Test.Config.html","title":"Antikythera.Test.Config","type":"module"},{"doc":"","ref":"Antikythera.Test.Config.html#base_url/0","title":"Antikythera.Test.Config.base_url/0","type":"function"},{"doc":"","ref":"Antikythera.Test.Config.html#blackbox_test?/0","title":"Antikythera.Test.Config.blackbox_test?/0","type":"function"},{"doc":"","ref":"Antikythera.Test.Config.html#blackbox_test_secret/0","title":"Antikythera.Test.Config.blackbox_test_secret/0","type":"function"},{"doc":"","ref":"Antikythera.Test.Config.html#init/0","title":"Antikythera.Test.Config.init/0","type":"function"},{"doc":"","ref":"Antikythera.Test.Config.html#test_mode/0","title":"Antikythera.Test.Config.test_mode/0","type":"function"},{"doc":"","ref":"Antikythera.Test.Config.html#whitebox_test_secret/0","title":"Antikythera.Test.Config.whitebox_test_secret/0","type":"function"},{"doc":"Helper functions to construct `Antikythera.Conn` object, to be used within tests.\n\nTypical controller tests are executed as follows:\n\n1. Make a `Antikythera.Conn` object.\n2. Pass the object to the target controller action.\n3. Inspect the returned `Antikythera.Conn` object.\n\nYou can use `make_conn/1` for the step-1 of the above procedure.\nIn controller tests this module is auto-imported by `use Antikythera.Test.ControllerTestCase`.","ref":"Antikythera.Test.ConnHelper.html","title":"Antikythera.Test.ConnHelper","type":"module"},{"doc":"","ref":"Antikythera.Test.ConnHelper.html#make_conn/1","title":"Antikythera.Test.ConnHelper.make_conn/1","type":"function"},{"doc":"Helper functions to set gear configs from within gears' test code.","ref":"Antikythera.Test.GearConfigHelper.html","title":"Antikythera.Test.GearConfigHelper","type":"module"},{"doc":"","ref":"Antikythera.Test.GearConfigHelper.html#set_config/2","title":"Antikythera.Test.GearConfigHelper.set_config/2","type":"function"},{"doc":"Helpers to work with gear logs within tests.","ref":"Antikythera.Test.GearLogHelper.html","title":"Antikythera.Test.GearLogHelper","type":"module"},{"doc":"Sets a context ID (which is included in gear logs) to process dictionary so that logs can be emitted during test executions.\n\nIf no argument is given, generates a new context ID and sets it.","ref":"Antikythera.Test.GearLogHelper.html#set_context_id/1","title":"Antikythera.Test.GearLogHelper.set_context_id/1","type":"function"},{"doc":"Helper functions to be used within tests for `GenServer`s.","ref":"Antikythera.Test.GenServerHelper.html","title":"Antikythera.Test.GenServerHelper","type":"module"},{"doc":"","ref":"Antikythera.Test.GenServerHelper.html#receive_cast_message/0","title":"Antikythera.Test.GenServerHelper.receive_cast_message/0","type":"function"},{"doc":"","ref":"Antikythera.Test.GenServerHelper.html#send_message_and_wait/2","title":"Antikythera.Test.GenServerHelper.send_message_and_wait/2","type":"function"},{"doc":"","ref":"Antikythera.Test.HttpClient.html","title":"Antikythera.Test.HttpClient","type":"module"},{"doc":"Helper functions to work with processes in test code.","ref":"Antikythera.Test.ProcessHelper.html","title":"Antikythera.Test.ProcessHelper","type":"module"},{"doc":"","ref":"Antikythera.Test.ProcessHelper.html#flush/0","title":"Antikythera.Test.ProcessHelper.flush/0","type":"function"},{"doc":"","ref":"Antikythera.Test.ProcessHelper.html#monitor_wait/1","title":"Antikythera.Test.ProcessHelper.monitor_wait/1","type":"function"},{"doc":"Websocket client for gear tests.\n\nPrepare your client module with:\n\n    defmodule YourGear.Socket do\n      use Antikythera.Test.WebsocketClient\n    end\n\nThen, call `YourGear.Socket.spawn_link(\"/ws\")` to open Websocket connection at \"ws(s):// /ws\" endpoint.\nIt will return pid of Websocket connection handling process if successful.\n\nBy default, it connects to the server specified by `Antikythera.Test.Config.base_url/0`,\nand redirects received frame to the caller (test running) process, in `{frame, handler_pid}` form.\n\nThese default behavior can be overridable with two overridable functions:\n\n- `base_url/0`\n- `handle_received_frame/2`","ref":"Antikythera.Test.WebsocketClient.html","title":"Antikythera.Test.WebsocketClient","type":"module"},{"doc":"Data structure to represent date and time in milli-seconds resolution.\n\nNote that all values of `Antikythera.Time.t` are in UTC.\n\n`Poison.Encoder` protocol is implemented for `Antikythera.Time.t`,\nso that values of this type can be directly converted to `Antikythera.IsoTimestamp.t` on `Poison.encode/1`.\n\n    iex> Poison.encode(%{time: {Antikythera.Time, {2017, 1, 1}, {0, 0, 0}, 0}})\n    {:ok, \"{\\\"time\\\":\\\"2017-01-01T00:00:00.000+00:00\\\"}\"}\n\nSee also `new/1`.","ref":"Antikythera.Time.html","title":"Antikythera.Time","type":"module"},{"doc":"","ref":"Antikythera.Time.html#diff_milliseconds/2","title":"Antikythera.Time.diff_milliseconds/2","type":"function"},{"doc":"","ref":"Antikythera.Time.html#from_epoch_milliseconds/1","title":"Antikythera.Time.from_epoch_milliseconds/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#from_gregorian_milliseconds/1","title":"Antikythera.Time.from_gregorian_milliseconds/1","type":"function"},{"doc":"Parses HTTP-date formats into Antikythera.Time.t.\n\nSupports IMF-fixdate format, RFC 850 format and ANSI C's `asctime()` format for compatibility.\n\nNote: An HTTP-date value must represents time in UTC(GMT). Thus timezone string in the end must always be 'GMT'.\nAny other timezone string (such as 'JST') will actually be ignored and parsed as GMT.\n\nhttps://tools.ietf.org/html/rfc7231#section-7.1.1.1","ref":"Antikythera.Time.html#from_http_date/1","title":"Antikythera.Time.from_http_date/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#from_http_date!/1","title":"Antikythera.Time.from_http_date!/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#from_iso_basic/1","title":"Antikythera.Time.from_iso_basic/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#from_iso_basic!/1","title":"Antikythera.Time.from_iso_basic!/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#from_iso_timestamp/1","title":"Antikythera.Time.from_iso_timestamp/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#from_iso_timestamp!/1","title":"Antikythera.Time.from_iso_timestamp!/1","type":"function"},{"doc":"Convert timestamps into `Antikythera.Time.t` or wrap valid `Antikythera.Time.t`,\nleveraging `recursive_new?` option of `Croma.Struct`.\n\nOnly `Antikythera.IsoTimestamp.t` can be converted.\n\n    iex> {:ok, time} = Elixir.Antikythera.Time.new(\"2015-01-23T23:50:07Z\")\n    {:ok, {Elixir.Antikythera.Time, {2015, 1, 23}, {23, 50, 7}, 0}}\n    iex> Elixir.Antikythera.Time.new(time)\n    {:ok, {Elixir.Antikythera.Time, {2015, 1, 23}, {23, 50, 7}, 0}}\n    iex> Elixir.Antikythera.Time.new(\"2015-01-23T23:50:07\") |> Croma.Result.error?()\n    true\n    iex> Elixir.Antikythera.Time.new(nil) |> Croma.Result.error?()\n    true","ref":"Antikythera.Time.html#new/1","title":"Antikythera.Time.new/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#now/0","title":"Antikythera.Time.now/0","type":"function"},{"doc":"","ref":"Antikythera.Time.html#shift_days/2","title":"Antikythera.Time.shift_days/2","type":"function"},{"doc":"","ref":"Antikythera.Time.html#shift_hours/2","title":"Antikythera.Time.shift_hours/2","type":"function"},{"doc":"","ref":"Antikythera.Time.html#shift_milliseconds/2","title":"Antikythera.Time.shift_milliseconds/2","type":"function"},{"doc":"","ref":"Antikythera.Time.html#shift_minutes/2","title":"Antikythera.Time.shift_minutes/2","type":"function"},{"doc":"","ref":"Antikythera.Time.html#shift_seconds/2","title":"Antikythera.Time.shift_seconds/2","type":"function"},{"doc":"","ref":"Antikythera.Time.html#to_epoch_milliseconds/1","title":"Antikythera.Time.to_epoch_milliseconds/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#to_gregorian_milliseconds/1","title":"Antikythera.Time.to_gregorian_milliseconds/1","type":"function"},{"doc":"Returns date/time in IMF-fixdate format.\n\nThe format is subset of Internet Message Format (RFC5322, formarly RFC822, RFC1123).\nDefined as 'preferred' format in RFC7231 and modern web servers or clients should send in this format.\n\nhttps://tools.ietf.org/html/rfc7231#section-7.1.1.1","ref":"Antikythera.Time.html#to_http_date/1","title":"Antikythera.Time.to_http_date/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#to_iso_basic/1","title":"Antikythera.Time.to_iso_basic/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#to_iso_timestamp/1","title":"Antikythera.Time.to_iso_timestamp/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#truncate_to_day/1","title":"Antikythera.Time.truncate_to_day/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#truncate_to_hour/1","title":"Antikythera.Time.truncate_to_hour/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#truncate_to_minute/1","title":"Antikythera.Time.truncate_to_minute/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#truncate_to_second/1","title":"Antikythera.Time.truncate_to_second/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#valid?/1","title":"Antikythera.Time.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Time.html#t:t/0","title":"Antikythera.Time.t/0","type":"type"},{"doc":"","ref":"Antikythera.Tmpdir.html","title":"Antikythera.Tmpdir","type":"module"},{"doc":"Creates a temporary directory which can be used as a working space for the passed function `f`.\n\nThis function is basically intended for async jobs which processes large amount of data.\nFor example, an async job that accumulates data into files and upload them to somewhere\ncan utilize this function to obtain a temporary working space.\n\nThe temporary directory is created before `f` is invoked.\nWhen execution of `f` is finished (either successfully or by exception) the directory is automatically removed.\nThe function returns the return value of `f`.\n\nNested calls to this function is not allowed.\nInstead you can freely make subdirectories of the temporary directory.","ref":"Antikythera.Tmpdir.html#make/2","title":"Antikythera.Tmpdir.make/2","type":"function"},{"doc":"Antikythera.Tmpdir.make(context, fn tmpdir ->\n      path = Path.join(tmpdir, \"foo\")\n      File.open(path, [:write], fn file ->\n        IO.write(file, \"some data 1\")\n        IO.write(file, \"some data 2\")\n      end)\n      upload_to_object_storage_service(\"object_key\", path)\n    end)","ref":"Antikythera.Tmpdir.html#make/2-example","title":"Example - Antikythera.Tmpdir.make/2","type":"function"},{"doc":"A thin wrapper around [`Foretoken`](https://github.com/skirino/foretoken) to avoid collisions between bucket names by prefixing executor pool IDs.\n\nFor details, refer to [Foretoken's documentation](https://hexdocs.pm/foretoken/api-reference.html).","ref":"Antikythera.TokenBucket.html","title":"Antikythera.TokenBucket","type":"module"},{"doc":"Takes the specified tokens from the bucket.\n\nInternally the actual bucket name is prefixed with the given `epool_id`.\nNote that return value on error is slightly different from that of `Foretoken.take/5` (for backward compatibility).","ref":"Antikythera.TokenBucket.html#take/5","title":"Antikythera.TokenBucket.take/5","type":"function"},{"doc":"","ref":"Antikythera.UnencodedPath.html","title":"Antikythera.UnencodedPath","type":"module"},{"doc":"","ref":"Antikythera.UnencodedPath.html#pattern/0","title":"Antikythera.UnencodedPath.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.UnencodedPath.html#segment_charclass/0","title":"Antikythera.UnencodedPath.segment_charclass/0","type":"function"},{"doc":"","ref":"Antikythera.UnencodedPath.html#valid?/1","title":"Antikythera.UnencodedPath.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.UnencodedPath.html#t:t/0","title":"Antikythera.UnencodedPath.t/0","type":"type"},{"doc":"URL format, originally defined in [RFC1738](https://tools.ietf.org/html/rfc1738),\nand updated in [RFC3986](https://tools.ietf.org/html/rfc3986) as a subset of URI.\n\n- Only accepts `http` or `https` as scheme.\n- IPv4 addresses in URLs must be 4-part-dotted-decimal formats.\n    - e.g. 192.168.0.1\n    - See [here](https://tools.ietf.org/html/rfc3986#section-3.2.2)\n- IPv6 addresses are not supported currently.","ref":"Antikythera.Url.html","title":"Antikythera.Url","type":"module"},{"doc":"","ref":"Antikythera.Url.html#pattern/0","title":"Antikythera.Url.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.Url.html#valid?/1","title":"Antikythera.Url.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Url.html#t:t/0","title":"Antikythera.Url.t/0","type":"type"},{"doc":"Format of versions of antikythera instances and gears.\n\nThe format rule is a stricter variant of [semantic versioning](https://semver.org/);\npre-release part and build metadata part are filled with information\nfrom the current git commit (committer date and commit hash, respectively).\nThis way we always make a new version for each new git commit,\nwhich enables both antikythera instances and gears to be deployed on a per-commit basis.\nThe actual version strings are created by\n`Antikythera.MixCommon.version_with_last_commit_info/1` defined in `mix_common.exs` file.\n\nNote that the current format prohibits multi-digits numbers as major/minor/patch version;\nthis is just to simplify deployment and not an intrinsic limitation.","ref":"Antikythera.VersionStr.html","title":"Antikythera.VersionStr","type":"module"},{"doc":"","ref":"Antikythera.VersionStr.html#pattern/0","title":"Antikythera.VersionStr.pattern/0","type":"function"},{"doc":"","ref":"Antikythera.VersionStr.html#valid?/1","title":"Antikythera.VersionStr.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.VersionStr.html#t:t/0","title":"Antikythera.VersionStr.t/0","type":"type"},{"doc":"Functions to convert numeric values into human-readable strings.","ref":"Antikythera.View.Unit.html","title":"Antikythera.View.Unit","type":"module"},{"doc":"","ref":"Antikythera.View.Unit.html#bytes/1","title":"Antikythera.View.Unit.bytes/1","type":"function"},{"doc":"Behaviour module for websocket handlers.\n\nNote the naming convention of the websocket-related modules; we use `Websocket`, `WebSocket` is not allowed.\n\nWebsocket module of gears must `use` this module as in the example below.\n`use Antikythera.Websocket` implicitly invokes `use Antikythera.Controller`, for convenience in implementing `connect/1` callback.","ref":"Antikythera.Websocket.html","title":"Antikythera.Websocket","type":"behaviour"},{"doc":"The following example simply echoes back messages from client:\n\n    defmodule MyGear.Websocket do\n      use Antikythera.Websocket\n\n      def init(_conn) do\n        {%{}, []}\n      end\n\n      def handle_client_message(state, _conn, frame) do\n        {state, [frame]}\n      end\n\n      def handle_server_message(state, _conn, _msg) do\n        {state, []}\n      end\n    end","ref":"Antikythera.Websocket.html#module-example","title":"Example - Antikythera.Websocket","type":"behaviour"},{"doc":"Once a websocket connection is established, subsequent bidirectional communication is handled by a dedicated connection process.\nTo send websocket frames to the connected client,\nyou should first be able to send messages to the connection process when a particular event occurs somewhere in the cluster.\nTo this end antikythera provides a process registry mechanism which makes connection processes accessible by \"name\"s.\n\nTo register connection processes, call `Antikythera.Registry.Unique.register/2` and/or `Antikythera.Registry.Group.join/2`\nin your `init/1` implementation.\nThen, to notify events of connection processes, use `Antikythera.Registry.Unique.send_message/3` or\n`Antikythera.Registry.Group.publish/3`.\nFinally to send websocket message from a connection process to client, implement `handle_server_message/3` callback\nso that it returns an appropriate websocket frame using the message.\n\nSee `Antikythera.Registry.Unique` and `Antikythera.Registry.Group` for more detail of the registry.","ref":"Antikythera.Websocket.html#module-name-registration","title":"Name registration - Antikythera.Websocket","type":"behaviour"},{"doc":"Callback function to be used during websocket handshake request.\n\nThis callback is implemented in basically the same way as ordinary controller actions.\nYou can use plugs and controller helper functions.\nThe only difference is that on success this function returns a `Antikythera.Conn.t` without setting an HTTP status code.\n\nThis callback is responsible for authenticating/authorizing the client.\nIf the client is valid and it's OK to start websocket communication, implementation of this callback must return the given `Antikythera.Conn.t`.\nOn the other hand if the client is not allowed to open websocket connection, this function must return an error as a usual HTTP response.\n\n`use Antikythera.Websocket` generates a default implementation of this callback, which just returns the given `Antikythera.Conn.t`.\nNote that you can use plugs without overriding the default.","ref":"Antikythera.Websocket.html#c:connect/1","title":"Antikythera.Websocket.connect/1","type":"callback"},{"doc":"Callback function to be called on receipt of a client message.","ref":"Antikythera.Websocket.html#c:handle_client_message/3","title":"Antikythera.Websocket.handle_client_message/3","type":"callback"},{"doc":"Callback function to be called on receipt of a message from other process in the cluster.","ref":"Antikythera.Websocket.html#c:handle_server_message/3","title":"Antikythera.Websocket.handle_server_message/3","type":"callback"},{"doc":"Callback function to be called right after a connection is established.\n\nThis callback is responsible for:\n\n1. initialize the process state (1st element of return value)\n2. send initial message to client (2nd element of return value)\n3. register the process to make it accessible from other processes in the system (see \"Name registration\" above)","ref":"Antikythera.Websocket.html#c:init/1","title":"Antikythera.Websocket.init/1","type":"callback"},{"doc":"Callback function to clean up resources used by the websocket connection.\n\nFor typical use cases you don't need to implement this callback;\n`Antikythera.Websocket` generates a default implementation (which does nothing) for you.","ref":"Antikythera.Websocket.html#c:terminate/3","title":"Antikythera.Websocket.terminate/3","type":"callback"},{"doc":"Type of return value of `init/1`, `handle_client_message/3` and `handle_server_message/3` callbacks.\nThe 1st element of the return value is used as the new state.\nThe 2nd element of the return value is sent to the client.\n\nTo close the connection, include a `:close` frame in the 2nd element of the return value.\nNote that the remaining frames after the close frame will not be sent.","ref":"Antikythera.Websocket.html#t:callback_result/0","title":"Antikythera.Websocket.callback_result/0","type":"type"},{"doc":"","ref":"Antikythera.Websocket.html#t:state/0","title":"Antikythera.Websocket.state/0","type":"type"},{"doc":"","ref":"Antikythera.Websocket.html#t:terminate_reason/0","title":"Antikythera.Websocket.terminate_reason/0","type":"type"},{"doc":"Simplified data type for websocket frames.","ref":"Antikythera.Websocket.Frame.html","title":"Antikythera.Websocket.Frame","type":"module"},{"doc":"","ref":"Antikythera.Websocket.Frame.html#valid?/1","title":"Antikythera.Websocket.Frame.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Websocket.Frame.html#t:close_code/0","title":"Antikythera.Websocket.Frame.close_code/0","type":"type"},{"doc":"","ref":"Antikythera.Websocket.Frame.html#t:close_payload/0","title":"Antikythera.Websocket.Frame.close_payload/0","type":"type"},{"doc":"","ref":"Antikythera.Websocket.Frame.html#t:t/0","title":"Antikythera.Websocket.Frame.t/0","type":"type"},{"doc":"","ref":"Antikythera.Websocket.FrameList.html","title":"Antikythera.Websocket.FrameList","type":"module"},{"doc":"","ref":"Antikythera.Websocket.FrameList.html#valid?/1","title":"Antikythera.Websocket.FrameList.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Websocket.FrameList.html#t:t/0","title":"Antikythera.Websocket.FrameList.t/0","type":"type"},{"doc":"Convenient XML parser module wrapping [fast_xml](https://github.com/processone/fast_xml).\n\n`decode/2` can parse XML into `Antikythera.Xml.Element.t`, and `encode/2` can serialize `Antikythera.Xml.Element.t` back to XML string.\n\n`Antikythera.Xml.Element.t` is XML element data structure, and it is JSON-convertible struct.\nYou can safely convert them to JSON using `Poison.encode/2` while keeping order of appearance of children,\nand also convert them back to `Antikythera.Xml.Element.t` with `Poison.decode/2` and `Antikythera.Xml.Element.new/1`.\n\nNote that order of attributes will not be preserved, since it is not significant.\nSee [here](https://www.w3.org/TR/xml/#sec-starttags)\n\nNamespace of tags (e.g. \"ns\" in ` `) are kept as is in `:name` of elements.\n\nNamespace definitions (e.g. `xmlns:ns='http://example.com/ns'`) are treated as plain attributes,\nand kept as is in `:attributes` of elements.\n\n## `Access` behaviour\n\n`Antikythera.Xml.Element` implements `Access` behaviour for convenient lookups and updates.\nFollowing access patterns are available:\n\n- `element[:name]`, `element[:attributes]`, `element[:children]`\n    - Fetch values of fields in dynamic lookup style.\n- `element[\"@some_attr\"]`\n    - Fetch value of \"some_attr\" in `:attributes` map.\n- `element[:texts]`\n    - Fetch text (character data) children. It always returns list.\n- `element[\"some_name\"]`\n    - Fetch child elements with `name: \"some_name\"`. It always returns list.\n\nYou can also use these patterns in `Kernel.get_in/2` and its variants.\n\n    iex> xml = \" foo bar baz \"\n    iex> element = Antikythera.Xml.decode!(xml)\n    %Antikythera.Xml.Element{name: \"a\", attributes: %{}, children: [\n      \"foo\",\n      %Antikythera.Xml.Element{name: \"b\", attributes: %{}, children: [\"bar\"]},\n      \"baz\",\n    ]}\n    iex> get_in(element, [:texts])\n    [\"foo\", \"baz\"]\n    iex> get_in(element, [\"b\", Access.at(0), :texts])\n    [\"bar\"]\n    iex> get_and_update_in(element, [:children, Access.at(0)], fn _ -> :pop end)\n    {\"foo\",\n    %Antikythera.Xml.Element{name: \"a\", attributes: %{}, children: [\n      %Antikythera.Xml.Element{name: \"b\", attributes: %{}, children: [\"bar\"]},\n      \"baz\",\n    ]}}\n    iex> update_in(element, [:children, Access.all()], fn\n    ...>   text when is_binary(text) -> %Antikythera.Xml.Element{name: \"b\", attributes: %{}, children: [text]}\n    ...>   e -> e\n    ...> end)\n    %Antikythera.Xml.Element{name: \"a\", attributes: %{}, children: [\n      %Antikythera.Xml.Element{name: \"b\", attributes: %{}, children: [\"foo\"]},\n      %Antikythera.Xml.Element{name: \"b\", attributes: %{}, children: [\"bar\"]},\n      %Antikythera.Xml.Element{name: \"b\", attributes: %{}, children: [\"baz\"]},\n    ]}\n    iex> update_in(element, [\"@id\"], fn _ -> \"001\" end)\n    %Antikythera.Xml.Element{name: \"a\", attributes: %{\"id\" => \"001\"}, children: [\n      \"foo\",\n      %Antikythera.Xml.Element{name: \"b\", attributes: %{}, children: [\"bar\"]},\n      \"baz\",\n    ]}\n\nNotes on updating with `Kernel.get_and_update_in/3` and its variants:\n\n- Struct fields are static and cannot be popped.\n- Custom access keys except \"@some_attr\" cannot be used in updating.\n  Use `:children` instead, in order to update children while preserving order of appearance.","ref":"Antikythera.Xml.html","title":"Antikythera.Xml","type":"module"},{"doc":"Reads an XML string and parses it into `Antikythera.Xml.Element.t`.\n\nComments and header will be discarded.\n\nIt can read XHTML document as long as they are well-formatted,\nthough it does not understand Document Type Definition (DTD, header line with \"<!DOCTYPE html PUBLIC ...\"),\nso you must remove them.\n\nIt tries to read a document with UTF-8 encoding, regardless of \"encoding\" attribute in the header.\n\nOptions:\n\n- `:trim` - Drop whitespace-only texts. Default `false`.\n    - There are no universal way to distinguish significant and insignificant whitespaces,\n      so this option may alter the meaning of original document. Use with caution.\n    - In [W3C recommendation](https://www.w3.org/TR/REC-xml/#sec-white-space),\n      it is stated that whitespace texts (character data) are basically significant and must be preserved.","ref":"Antikythera.Xml.html#decode/2","title":"Antikythera.Xml.decode/2","type":"function"},{"doc":"","ref":"Antikythera.Xml.html#decode!/1","title":"Antikythera.Xml.decode!/1","type":"function"},{"doc":"","ref":"Antikythera.Xml.html#decode!/2","title":"Antikythera.Xml.decode!/2","type":"function"},{"doc":"Serializes `Antikythera.Xml.Element.t` into XML string.\n\nSpecifications:\n\n- Trailing newline will not be generated.\n- All single- and double-quotations in attribute values or entity values are escaped to\n  `&apos;` and `&quot;` respectively.\n- All attribute values are SINGLE-quoted.\n- Does not insert a whitespace before \"/>\" in element without children.\n\nOptions:\n\n- `:pretty` - Pretty print with 2-space indents. Default `false`.\n    - Similar to `:trim` option in `decode/2`, inserted whitespaces may be significant,\n      thus it can alter meaning of original document. Use with caution.\n    - It does not insert whitespaces to elements with [mixed-content](https://www.w3.org/TR/REC-xml/#sec-mixed-content)\n      and their descendants, in order to reduce probability to alter the meaning of original document.\n- `:with_header` - Prepend ` \\n`. Default `false`.","ref":"Antikythera.Xml.html#encode/2","title":"Antikythera.Xml.encode/2","type":"function"},{"doc":"","ref":"Antikythera.Xml.html#t:decode_option/0","title":"Antikythera.Xml.decode_option/0","type":"type"},{"doc":"","ref":"Antikythera.Xml.html#t:encode_option/0","title":"Antikythera.Xml.encode_option/0","type":"type"},{"doc":"","ref":"Antikythera.Xml.Content.html","title":"Antikythera.Xml.Content","type":"module"},{"doc":"","ref":"Antikythera.Xml.Content.html#new/1","title":"Antikythera.Xml.Content.new/1","type":"function"},{"doc":"","ref":"Antikythera.Xml.Content.html#valid?/1","title":"Antikythera.Xml.Content.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Xml.Content.html#t:t/0","title":"Antikythera.Xml.Content.t/0","type":"type"},{"doc":"","ref":"Antikythera.Xml.Element.html","title":"Antikythera.Xml.Element","type":"module"},{"doc":"Type-aware getter for attributes.","ref":"Antikythera.Xml.Element.html#attributes/1","title":"Antikythera.Xml.Element.attributes/1","type":"function"},{"doc":"Type-aware setter for attributes.","ref":"Antikythera.Xml.Element.html#attributes/2","title":"Antikythera.Xml.Element.attributes/2","type":"function"},{"doc":"Type-aware getter for children.","ref":"Antikythera.Xml.Element.html#children/1","title":"Antikythera.Xml.Element.children/1","type":"function"},{"doc":"Type-aware setter for children.","ref":"Antikythera.Xml.Element.html#children/2","title":"Antikythera.Xml.Element.children/2","type":"function"},{"doc":"Type-aware getter for name.","ref":"Antikythera.Xml.Element.html#name/1","title":"Antikythera.Xml.Element.name/1","type":"function"},{"doc":"Type-aware setter for name.","ref":"Antikythera.Xml.Element.html#name/2","title":"Antikythera.Xml.Element.name/2","type":"function"},{"doc":"Creates a new instance of Antikythera.Xml.Element by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"Antikythera.Xml.Element.html#new/1","title":"Antikythera.Xml.Element.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Xml.Element.html#new!/1","title":"Antikythera.Xml.Element.new!/1","type":"function"},{"doc":"Updates an existing instance of Antikythera.Xml.Element with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"Antikythera.Xml.Element.html#update/2","title":"Antikythera.Xml.Element.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"Antikythera.Xml.Element.html#update!/2","title":"Antikythera.Xml.Element.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"Antikythera.Xml.Element.html#valid?/1","title":"Antikythera.Xml.Element.valid?/1","type":"function"},{"doc":"","ref":"Antikythera.Xml.Element.html#t:t/0","title":"Antikythera.Xml.Element.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.html","title":"AntikytheraCore","type":"module"},{"doc":"","ref":"AntikytheraCore.html#add_translator_to_logger/0","title":"AntikytheraCore.add_translator_to_logger/0","type":"function"},{"doc":"Callback implementation of `Application.start/2`.\n\nInterdependencies between initialization steps here are crucial.\nSee also `AntikytheraCore.StartupManager` for initializations after construction\nof the supervision tree.","ref":"AntikytheraCore.html#start/2","title":"AntikytheraCore.start/2","type":"function"},{"doc":"`:gen_event` callback module that reports number of errors to `AntikytheraCore.ErrorCountsAccumulator` process.\n\nThis event handler module is installed in all `AntikytheraCore.Alert.Manager` processes.\nThe received alerts are buffered for 60000 milliseconds and then reported in one batch.","ref":"AntikytheraCore.Alert.ErrorCountReporter.html","title":"AntikytheraCore.Alert.ErrorCountReporter","type":"module"},{"doc":"Behaviour module for alert handlers.\n\nImplementations must be prefixed with `AntikytheraCore.Alert.Handler`, e.g. `AntikytheraCore.Alert.Handler.Email`.\nThis module also implements `:gen_event` behaviour and installed as handlers for `AntikytheraCore.Alert.Manager` processes.\nEach installed handlers will call callbacks of their corresponding implementation modules (references to those modules are held in handler states).\n\nA simple buffering/throttling mechanism is built in.\n\n- Messages received by handlers will be sent out in \"fast-then-delayed\" pattern:\n    - Occasional messages will be flushed from the buffer and sent out in `fast_interval`.\n    - Messages arriving too frequently will be buffered for `delayed_interval` until they are sent out.\n- By default, `fast_interval` is 60 seconds\n  and `delayed_interval` is 1800 seconds.\n    - They can be customized via core/gear configs.\n      Specify fast_interval or delayed_interval for the handler.","ref":"AntikytheraCore.Alert.Handler.html","title":"AntikytheraCore.Alert.Handler","type":"behaviour"},{"doc":"Through `validate_config/1` callback, `handler_config` will be validated\nwhether it includes sufficient information required for the implementation.\nIf the validation passed, a handler, with a reference to the implementation module, will be installed for that OTP application.\nIf the validation failed, the handler will not be installed, and will be uninstalled if it is installed already.\n\nThis means, customization and installation are purely done via core/gear config.","ref":"AntikytheraCore.Alert.Handler.html#module-customization-and-installation-of-handlers","title":"Customization and installation of handlers - AntikytheraCore.Alert.Handler","type":"behaviour"},{"doc":"Send alert(s) for messages in the buffer, using `handler_config`. Summarize messages if needed.\nMust return messages which could not be sent for whatever reason for retry.\nOldest message comes first in `messages`, same goes for returned messages.\n\nNote: If the handler is trying to perform alerts which can take some time to finish (e.g. send email),\nconsider dispatching them to a temporary process.\nIn that case though, results of alerts cannot be received (`[]` should always be returned).","ref":"AntikytheraCore.Alert.Handler.html#c:send_alerts/3","title":"AntikytheraCore.Alert.Handler.send_alerts/3","type":"callback"},{"doc":"Validate `handler_config` whether it includes sufficient configurations for the handler.\nReturn `true` when it is valid.","ref":"AntikytheraCore.Alert.Handler.html#c:validate_config/1","title":"AntikytheraCore.Alert.Handler.validate_config/1","type":"callback"},{"doc":"Alert handler implementation that sends an email.\n\nThis is considered the default alert backend (and also used for testing of `AntikytheraCore.Alert.Manager`).\nEmail delivery method is provided by a callback module of `AntikytheraEal.AlertMailer.Behaviour`.","ref":"AntikytheraCore.Alert.Handler.Email.html","title":"AntikytheraCore.Alert.Handler.Email","type":"module"},{"doc":"- `to` - Required. List of email addresses to be sent. Must not be an empty list.\n- `errors_per_body` - Optional. Integer number of errors printed in body of an alert mail.\n  Details of errors beyond this threshold will be omitted.\n  Defaults to 3.","ref":"AntikytheraCore.Alert.Handler.Email.html#module-handler-config","title":"Handler config - AntikytheraCore.Alert.Handler.Email","type":"module"},{"doc":"Type module for map of configurations for a single alert handler.\n\nFields are treated as opaque at this layer; details are defined by each handler implementation.","ref":"AntikytheraCore.Alert.HandlerConfig.html","title":"AntikytheraCore.Alert.HandlerConfig","type":"module"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerConfig.html#get/2","title":"AntikytheraCore.Alert.HandlerConfig.get/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerConfig.html#valid?/1","title":"AntikytheraCore.Alert.HandlerConfig.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerConfig.html#t:t/0","title":"AntikytheraCore.Alert.HandlerConfig.t/0","type":"type"},{"doc":"Type module for map of all alert configurations for a single OTP application (antikythera or a gear).\n\nKeys must be snake-cased handler names and values must be maps of each alert handler's configurations.\nThese maps are stored in core/gear configs.","ref":"AntikytheraCore.Alert.HandlerConfigsMap.html","title":"AntikytheraCore.Alert.HandlerConfigsMap","type":"module"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerConfigsMap.html#get/1","title":"AntikytheraCore.Alert.HandlerConfigsMap.get/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerConfigsMap.html#valid?/1","title":"AntikytheraCore.Alert.HandlerConfigsMap.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerConfigsMap.html#t:t/0","title":"AntikytheraCore.Alert.HandlerConfigsMap.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerState.html","title":"AntikytheraCore.Alert.HandlerState","type":"module"},{"doc":"Type-aware getter for busy?.","ref":"AntikytheraCore.Alert.HandlerState.html#busy?/1","title":"AntikytheraCore.Alert.HandlerState.busy?/1","type":"function"},{"doc":"Type-aware setter for busy?.","ref":"AntikytheraCore.Alert.HandlerState.html#busy?/2","title":"AntikytheraCore.Alert.HandlerState.busy?/2","type":"function"},{"doc":"Type-aware getter for handler_module.","ref":"AntikytheraCore.Alert.HandlerState.html#handler_module/1","title":"AntikytheraCore.Alert.HandlerState.handler_module/1","type":"function"},{"doc":"Type-aware setter for handler_module.","ref":"AntikytheraCore.Alert.HandlerState.html#handler_module/2","title":"AntikytheraCore.Alert.HandlerState.handler_module/2","type":"function"},{"doc":"Type-aware getter for message_buffer.","ref":"AntikytheraCore.Alert.HandlerState.html#message_buffer/1","title":"AntikytheraCore.Alert.HandlerState.message_buffer/1","type":"function"},{"doc":"Type-aware setter for message_buffer.","ref":"AntikytheraCore.Alert.HandlerState.html#message_buffer/2","title":"AntikytheraCore.Alert.HandlerState.message_buffer/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.Alert.HandlerState by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.Alert.HandlerState.html#new/1","title":"AntikytheraCore.Alert.HandlerState.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Alert.HandlerState.html#new!/1","title":"AntikytheraCore.Alert.HandlerState.new!/1","type":"function"},{"doc":"Type-aware getter for otp_app_name.","ref":"AntikytheraCore.Alert.HandlerState.html#otp_app_name/1","title":"AntikytheraCore.Alert.HandlerState.otp_app_name/1","type":"function"},{"doc":"Type-aware setter for otp_app_name.","ref":"AntikytheraCore.Alert.HandlerState.html#otp_app_name/2","title":"AntikytheraCore.Alert.HandlerState.otp_app_name/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.Alert.HandlerState with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.Alert.HandlerState.html#update/2","title":"AntikytheraCore.Alert.HandlerState.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Alert.HandlerState.html#update!/2","title":"AntikytheraCore.Alert.HandlerState.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.Alert.HandlerState.html#valid?/1","title":"AntikytheraCore.Alert.HandlerState.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.HandlerState.html#t:t/0","title":"AntikytheraCore.Alert.HandlerState.t/0","type":"type"},{"doc":"Backend handler for `Logger`, which notifies `AntikytheraCore.Alert.Manager` of error events.","ref":"AntikytheraCore.Alert.LoggerBackend.html","title":"AntikytheraCore.Alert.LoggerBackend","type":"module"},{"doc":"An event manager hosting multiple `AntikytheraCore.Alert.Handler`s.\n\nManagers are spawned per OTP application (antikythera and each gear).\nEach handler has its own buffer to store messages before sending them as an alert.","ref":"AntikytheraCore.Alert.Manager.html","title":"AntikytheraCore.Alert.Manager","type":"module"},{"doc":"","ref":"AntikytheraCore.Alert.Manager.html#child_spec/1","title":"AntikytheraCore.Alert.Manager.child_spec/1","type":"function"},{"doc":"Notify a `manager` process of a message `body` with optional `time`.","ref":"AntikytheraCore.Alert.Manager.html#notify/3","title":"AntikytheraCore.Alert.Manager.notify/3","type":"function"},{"doc":"Schedule next flushing from handlers. Assuming the `interval` is in seconds.\nNote that handlers are kept as state in gen_event manager process,\nthus calling `self/0` always refers to the manager process which the handler is installed in.","ref":"AntikytheraCore.Alert.Manager.html#schedule_handler_timeout/2","title":"AntikytheraCore.Alert.Manager.schedule_handler_timeout/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.Manager.html#start_link/1","title":"AntikytheraCore.Alert.Manager.start_link/1","type":"function"},{"doc":"Update handler installations according to alert config of an OTP application (antikythera or gear).\nHandlers will be installed/uninstalled depending on contents of the config.\nIf the config has sufficient information for a handler, it will be installed (noop if already installed).\nOtherwise, it will not be installed and will be uninstalled if it is already installed.\n\nAny errors will be logged but this function always returns `:ok`.","ref":"AntikytheraCore.Alert.Manager.html#update_handler_installations/2","title":"AntikytheraCore.Alert.Manager.update_handler_installations/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.Message.html","title":"AntikytheraCore.Alert.Message","type":"module"},{"doc":"","ref":"AntikytheraCore.Alert.Message.html#new/1","title":"AntikytheraCore.Alert.Message.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.Message.html#new!/1","title":"AntikytheraCore.Alert.Message.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.Message.html#valid?/1","title":"AntikytheraCore.Alert.Message.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Alert.Message.html#t:t/0","title":"AntikytheraCore.Alert.Message.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.AsyncJob.html","title":"AntikytheraCore.AsyncJob","type":"module"},{"doc":"Type-aware getter for attempts.","ref":"AntikytheraCore.AsyncJob.html#attempts/1","title":"AntikytheraCore.AsyncJob.attempts/1","type":"function"},{"doc":"Type-aware setter for attempts.","ref":"AntikytheraCore.AsyncJob.html#attempts/2","title":"AntikytheraCore.AsyncJob.attempts/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.html#compute_retry_interval/1","title":"AntikytheraCore.AsyncJob.compute_retry_interval/1","type":"function"},{"doc":"Type-aware getter for gear_name.","ref":"AntikytheraCore.AsyncJob.html#gear_name/1","title":"AntikytheraCore.AsyncJob.gear_name/1","type":"function"},{"doc":"Type-aware setter for gear_name.","ref":"AntikytheraCore.AsyncJob.html#gear_name/2","title":"AntikytheraCore.AsyncJob.gear_name/2","type":"function"},{"doc":"Type-aware getter for max_duration.","ref":"AntikytheraCore.AsyncJob.html#max_duration/1","title":"AntikytheraCore.AsyncJob.max_duration/1","type":"function"},{"doc":"Type-aware setter for max_duration.","ref":"AntikytheraCore.AsyncJob.html#max_duration/2","title":"AntikytheraCore.AsyncJob.max_duration/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.html#max_start_time_from_now/0","title":"AntikytheraCore.AsyncJob.max_start_time_from_now/0","type":"function"},{"doc":"Type-aware getter for module.","ref":"AntikytheraCore.AsyncJob.html#module/1","title":"AntikytheraCore.AsyncJob.module/1","type":"function"},{"doc":"Type-aware setter for module.","ref":"AntikytheraCore.AsyncJob.html#module/2","title":"AntikytheraCore.AsyncJob.module/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.AsyncJob by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.AsyncJob.html#new/1","title":"AntikytheraCore.AsyncJob.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.AsyncJob.html#new!/1","title":"AntikytheraCore.AsyncJob.new!/1","type":"function"},{"doc":"Type-aware getter for payload.","ref":"AntikytheraCore.AsyncJob.html#payload/1","title":"AntikytheraCore.AsyncJob.payload/1","type":"function"},{"doc":"Type-aware setter for payload.","ref":"AntikytheraCore.AsyncJob.html#payload/2","title":"AntikytheraCore.AsyncJob.payload/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.html#register/5","title":"AntikytheraCore.AsyncJob.register/5","type":"function"},{"doc":"Type-aware getter for remaining_attempts.","ref":"AntikytheraCore.AsyncJob.html#remaining_attempts/1","title":"AntikytheraCore.AsyncJob.remaining_attempts/1","type":"function"},{"doc":"Type-aware setter for remaining_attempts.","ref":"AntikytheraCore.AsyncJob.html#remaining_attempts/2","title":"AntikytheraCore.AsyncJob.remaining_attempts/2","type":"function"},{"doc":"Type-aware getter for retry_interval.","ref":"AntikytheraCore.AsyncJob.html#retry_interval/1","title":"AntikytheraCore.AsyncJob.retry_interval/1","type":"function"},{"doc":"Type-aware setter for retry_interval.","ref":"AntikytheraCore.AsyncJob.html#retry_interval/2","title":"AntikytheraCore.AsyncJob.retry_interval/2","type":"function"},{"doc":"Type-aware getter for schedule.","ref":"AntikytheraCore.AsyncJob.html#schedule/1","title":"AntikytheraCore.AsyncJob.schedule/1","type":"function"},{"doc":"Type-aware setter for schedule.","ref":"AntikytheraCore.AsyncJob.html#schedule/2","title":"AntikytheraCore.AsyncJob.schedule/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.AsyncJob with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.AsyncJob.html#update/2","title":"AntikytheraCore.AsyncJob.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.AsyncJob.html#update!/2","title":"AntikytheraCore.AsyncJob.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.AsyncJob.html#valid?/1","title":"AntikytheraCore.AsyncJob.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.html#t:t/0","title":"AntikytheraCore.AsyncJob.t/0","type":"type"},{"doc":"A queue-like data structure, replicated using `RaftedValue`, to store `AntikytheraCore.AsyncJob` structs.","ref":"AntikytheraCore.AsyncJob.Queue.html","title":"AntikytheraCore.AsyncJob.Queue","type":"module"},{"doc":"Type-aware getter for abandoned_jobs.","ref":"AntikytheraCore.AsyncJob.Queue.html#abandoned_jobs/1","title":"AntikytheraCore.AsyncJob.Queue.abandoned_jobs/1","type":"function"},{"doc":"Type-aware setter for abandoned_jobs.","ref":"AntikytheraCore.AsyncJob.Queue.html#abandoned_jobs/2","title":"AntikytheraCore.AsyncJob.Queue.abandoned_jobs/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#add_job/5","title":"AntikytheraCore.AsyncJob.Queue.add_job/5","type":"function"},{"doc":"Type-aware getter for brokers_to_notify.","ref":"AntikytheraCore.AsyncJob.Queue.html#brokers_to_notify/1","title":"AntikytheraCore.AsyncJob.Queue.brokers_to_notify/1","type":"function"},{"doc":"Type-aware setter for brokers_to_notify.","ref":"AntikytheraCore.AsyncJob.Queue.html#brokers_to_notify/2","title":"AntikytheraCore.AsyncJob.Queue.brokers_to_notify/2","type":"function"},{"doc":"Type-aware getter for brokers_waiting.","ref":"AntikytheraCore.AsyncJob.Queue.html#brokers_waiting/1","title":"AntikytheraCore.AsyncJob.Queue.brokers_waiting/1","type":"function"},{"doc":"Type-aware setter for brokers_waiting.","ref":"AntikytheraCore.AsyncJob.Queue.html#brokers_waiting/2","title":"AntikytheraCore.AsyncJob.Queue.brokers_waiting/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#cancel/2","title":"AntikytheraCore.AsyncJob.Queue.cancel/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#fetch_job/1","title":"AntikytheraCore.AsyncJob.Queue.fetch_job/1","type":"function"},{"doc":"Type-aware getter for index_runnable.","ref":"AntikytheraCore.AsyncJob.Queue.html#index_runnable/1","title":"AntikytheraCore.AsyncJob.Queue.index_runnable/1","type":"function"},{"doc":"Type-aware setter for index_runnable.","ref":"AntikytheraCore.AsyncJob.Queue.html#index_runnable/2","title":"AntikytheraCore.AsyncJob.Queue.index_runnable/2","type":"function"},{"doc":"Type-aware getter for index_running.","ref":"AntikytheraCore.AsyncJob.Queue.html#index_running/1","title":"AntikytheraCore.AsyncJob.Queue.index_running/1","type":"function"},{"doc":"Type-aware setter for index_running.","ref":"AntikytheraCore.AsyncJob.Queue.html#index_running/2","title":"AntikytheraCore.AsyncJob.Queue.index_running/2","type":"function"},{"doc":"Type-aware getter for index_waiting.","ref":"AntikytheraCore.AsyncJob.Queue.html#index_waiting/1","title":"AntikytheraCore.AsyncJob.Queue.index_waiting/1","type":"function"},{"doc":"Type-aware setter for index_waiting.","ref":"AntikytheraCore.AsyncJob.Queue.html#index_waiting/2","title":"AntikytheraCore.AsyncJob.Queue.index_waiting/2","type":"function"},{"doc":"Type-aware getter for jobs.","ref":"AntikytheraCore.AsyncJob.Queue.html#jobs/1","title":"AntikytheraCore.AsyncJob.Queue.jobs/1","type":"function"},{"doc":"Type-aware setter for jobs.","ref":"AntikytheraCore.AsyncJob.Queue.html#jobs/2","title":"AntikytheraCore.AsyncJob.Queue.jobs/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#list/1","title":"AntikytheraCore.AsyncJob.Queue.list/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#make_rv_config/1","title":"AntikytheraCore.AsyncJob.Queue.make_rv_config/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#max_jobs/0","title":"AntikytheraCore.AsyncJob.Queue.max_jobs/0","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.AsyncJob.Queue by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.AsyncJob.Queue.html#new/1","title":"AntikytheraCore.AsyncJob.Queue.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.AsyncJob.Queue.html#new!/1","title":"AntikytheraCore.AsyncJob.Queue.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#remove_broker_from_waiting_list/1","title":"AntikytheraCore.AsyncJob.Queue.remove_broker_from_waiting_list/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#remove_locked_job/2","title":"AntikytheraCore.AsyncJob.Queue.remove_locked_job/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#start_jobs_and_get_metrics/1","title":"AntikytheraCore.AsyncJob.Queue.start_jobs_and_get_metrics/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#status/2","title":"AntikytheraCore.AsyncJob.Queue.status/2","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#unlock_job_for_retry/2","title":"AntikytheraCore.AsyncJob.Queue.unlock_job_for_retry/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.AsyncJob.Queue with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.AsyncJob.Queue.html#update/2","title":"AntikytheraCore.AsyncJob.Queue.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.AsyncJob.Queue.html#update!/2","title":"AntikytheraCore.AsyncJob.Queue.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.AsyncJob.Queue.html#valid?/1","title":"AntikytheraCore.AsyncJob.Queue.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.html#t:t/0","title":"AntikytheraCore.AsyncJob.Queue.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.Hook.html","title":"AntikytheraCore.AsyncJob.Queue.Hook","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.IdJobPair.html","title":"AntikytheraCore.AsyncJob.Queue.IdJobPair","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.IdJobPair.html#new/1","title":"AntikytheraCore.AsyncJob.Queue.IdJobPair.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.IdJobPair.html#new!/1","title":"AntikytheraCore.AsyncJob.Queue.IdJobPair.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.IdJobPair.html#valid?/1","title":"AntikytheraCore.AsyncJob.Queue.IdJobPair.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.IdJobPair.html#t:t/0","title":"AntikytheraCore.AsyncJob.Queue.IdJobPair.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobKey.html","title":"AntikytheraCore.AsyncJob.Queue.JobKey","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobKey.html#valid?/1","title":"AntikytheraCore.AsyncJob.Queue.JobKey.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobKey.html#t:t/0","title":"AntikytheraCore.AsyncJob.Queue.JobKey.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.html","title":"AntikytheraCore.AsyncJob.Queue.JobsMap","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.html#new/1","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.html#new!/1","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.html#valid?/1","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.html#t:t/0","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.html","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.html#new/1","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.html#new!/1","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.html#valid?/1","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.html#t:t/0","title":"AntikytheraCore.AsyncJob.Queue.JobsMap.Triplet.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.SecondaryIndex.html","title":"AntikytheraCore.AsyncJob.Queue.SecondaryIndex","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.SecondaryIndex.html#valid?/1","title":"AntikytheraCore.AsyncJob.Queue.SecondaryIndex.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.Queue.SecondaryIndex.html#t:t/0","title":"AntikytheraCore.AsyncJob.Queue.SecondaryIndex.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RaftPerMemberOptionsMaker.html","title":"AntikytheraCore.AsyncJob.RaftPerMemberOptionsMaker","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RaftedValueConfigMaker.html","title":"AntikytheraCore.AsyncJob.RaftedValueConfigMaker","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RaftedValueConfigMaker.html#options/0","title":"AntikytheraCore.AsyncJob.RaftedValueConfigMaker.options/0","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RateLimit.html","title":"AntikytheraCore.AsyncJob.RateLimit","type":"module"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RateLimit.html#check_for_command/1","title":"AntikytheraCore.AsyncJob.RateLimit.check_for_command/1","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RateLimit.html#check_with_retry_for_query/3","title":"AntikytheraCore.AsyncJob.RateLimit.check_with_retry_for_query/3","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RateLimit.html#max_tokens/0","title":"AntikytheraCore.AsyncJob.RateLimit.max_tokens/0","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RateLimit.html#milliseconds_per_token/0","title":"AntikytheraCore.AsyncJob.RateLimit.milliseconds_per_token/0","type":"function"},{"doc":"","ref":"AntikytheraCore.AsyncJob.RateLimit.html#tokens_per_command/0","title":"AntikytheraCore.AsyncJob.RateLimit.tokens_per_command/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Cluster.html","title":"AntikytheraCore.Cluster","type":"module"},{"doc":"","ref":"AntikytheraCore.Cluster.html#connect_to_other_nodes/1","title":"AntikytheraCore.Cluster.connect_to_other_nodes/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Cluster.html#connect_to_other_nodes_on_start/0","title":"AntikytheraCore.Cluster.connect_to_other_nodes_on_start/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Cluster.html#node_to_host/1","title":"AntikytheraCore.Cluster.node_to_host/1","type":"function"},{"doc":"ID of instance that is visible to gear developers.\n\nNote that the ID string here uses short hostname (`hostname -s`) instead of fully-qualified one (`hostname -f`)\nin order to make it short by truncating common parts.","ref":"AntikytheraCore.Cluster.NodeId.html","title":"AntikytheraCore.Cluster.NodeId","type":"module"},{"doc":"","ref":"AntikytheraCore.Cluster.NodeId.html#get/0","title":"AntikytheraCore.Cluster.NodeId.get/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Cluster.NodeId.html#init/0","title":"AntikytheraCore.Cluster.NodeId.init/0","type":"function"},{"doc":"A `GenServer` that periodically fetches current members of the cluster (all existing hostnames and their lifecycle states)\nfrom underlying infrastructure.\n\nIn rare occasions fetching hosts information may take longer than 5 seconds;\nwe spawn dedicated process in order to keep this GenServer responsive to `call`s from other processes.","ref":"AntikytheraCore.ClusterHostsPoller.html","title":"AntikytheraCore.ClusterHostsPoller","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ClusterHostsPoller.html#child_spec/1","title":"AntikytheraCore.ClusterHostsPoller.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ClusterHostsPoller.html#current_hosts/0","title":"AntikytheraCore.ClusterHostsPoller.current_hosts/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ClusterHostsPoller.html#start_link/1","title":"AntikytheraCore.ClusterHostsPoller.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ClusterHostsPoller.Fetcher.html","title":"AntikytheraCore.ClusterHostsPoller.Fetcher","type":"module"},{"doc":"","ref":"AntikytheraCore.ClusterHostsPoller.Fetcher.html#run/0","title":"AntikytheraCore.ClusterHostsPoller.Fetcher.run/0","type":"function"},{"doc":"A `GenServer` that tries to keep connections to other known nodes.\n\nDepends on `AntikytheraCore.ClusterHostsPoller`.","ref":"AntikytheraCore.ClusterNodesConnector.html","title":"AntikytheraCore.ClusterNodesConnector","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ClusterNodesConnector.html#child_spec/1","title":"AntikytheraCore.ClusterNodesConnector.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ClusterNodesConnector.html#start_link/1","title":"AntikytheraCore.ClusterNodesConnector.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Cmd.html","title":"AntikytheraCore.Cmd","type":"module"},{"doc":"","ref":"AntikytheraCore.Cmd.html#hostname/0","title":"AntikytheraCore.Cmd.hostname/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Core.html","title":"AntikytheraCore.Config.Core","type":"module"},{"doc":"","ref":"AntikytheraCore.Config.Core.html#dump_from_env_to_file/0","title":"AntikytheraCore.Config.Core.dump_from_env_to_file/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Core.html#load/1","title":"AntikytheraCore.Config.Core.load/1","type":"function"},{"doc":"Writes a map to antikythera's config file.\n\nCurrently this function is intended to be used within remote_console.","ref":"AntikytheraCore.Config.Core.html#write/1","title":"AntikytheraCore.Config.Core.write/1","type":"function"},{"doc":"Utility functions to handle secret key which is used to encrypt/decrypt config files.","ref":"AntikytheraCore.Config.EncryptionKey.html","title":"AntikytheraCore.Config.EncryptionKey","type":"module"},{"doc":"","ref":"AntikytheraCore.Config.EncryptionKey.html#get/0","title":"AntikytheraCore.Config.EncryptionKey.get/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.EncryptionKey.html#init/0","title":"AntikytheraCore.Config.EncryptionKey.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.EncryptionKey.html#t:t/0","title":"AntikytheraCore.Config.EncryptionKey.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html","title":"AntikytheraCore.Config.Gear","type":"module"},{"doc":"Type-aware getter for alerts.","ref":"AntikytheraCore.Config.Gear.html#alerts/1","title":"AntikytheraCore.Config.Gear.alerts/1","type":"function"},{"doc":"Type-aware setter for alerts.","ref":"AntikytheraCore.Config.Gear.html#alerts/2","title":"AntikytheraCore.Config.Gear.alerts/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#default/0","title":"AntikytheraCore.Config.Gear.default/0","type":"function"},{"doc":"Type-aware getter for domains.","ref":"AntikytheraCore.Config.Gear.html#domains/1","title":"AntikytheraCore.Config.Gear.domains/1","type":"function"},{"doc":"Type-aware setter for domains.","ref":"AntikytheraCore.Config.Gear.html#domains/2","title":"AntikytheraCore.Config.Gear.domains/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#dump_all_from_env_to_file/0","title":"AntikytheraCore.Config.Gear.dump_all_from_env_to_file/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#ensure_loaded/1","title":"AntikytheraCore.Config.Gear.ensure_loaded/1","type":"function"},{"doc":"Type-aware getter for internal_kv.","ref":"AntikytheraCore.Config.Gear.html#internal_kv/1","title":"AntikytheraCore.Config.Gear.internal_kv/1","type":"function"},{"doc":"Type-aware setter for internal_kv.","ref":"AntikytheraCore.Config.Gear.html#internal_kv/2","title":"AntikytheraCore.Config.Gear.internal_kv/2","type":"function"},{"doc":"Type-aware getter for kv.","ref":"AntikytheraCore.Config.Gear.html#kv/1","title":"AntikytheraCore.Config.Gear.kv/1","type":"function"},{"doc":"Type-aware setter for kv.","ref":"AntikytheraCore.Config.Gear.html#kv/2","title":"AntikytheraCore.Config.Gear.kv/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#load_all/1","title":"AntikytheraCore.Config.Gear.load_all/1","type":"function"},{"doc":"Type-aware getter for log_level.","ref":"AntikytheraCore.Config.Gear.html#log_level/1","title":"AntikytheraCore.Config.Gear.log_level/1","type":"function"},{"doc":"Type-aware setter for log_level.","ref":"AntikytheraCore.Config.Gear.html#log_level/2","title":"AntikytheraCore.Config.Gear.log_level/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.Config.Gear by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.Config.Gear.html#new/1","title":"AntikytheraCore.Config.Gear.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Config.Gear.html#new!/1","title":"AntikytheraCore.Config.Gear.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#read/1","title":"AntikytheraCore.Config.Gear.read/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#read_all/0","title":"AntikytheraCore.Config.Gear.read_all/0","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.Config.Gear with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.Config.Gear.html#update/2","title":"AntikytheraCore.Config.Gear.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Config.Gear.html#update!/2","title":"AntikytheraCore.Config.Gear.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.Config.Gear.html#valid?/1","title":"AntikytheraCore.Config.Gear.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#write/2","title":"AntikytheraCore.Config.Gear.write/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.html#t:t/0","title":"AntikytheraCore.Config.Gear.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Config.Gear.CustomDomainList.html","title":"AntikytheraCore.Config.Gear.CustomDomainList","type":"module"},{"doc":"","ref":"AntikytheraCore.Config.Gear.CustomDomainList.html#max_length/0","title":"AntikytheraCore.Config.Gear.CustomDomainList.max_length/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.CustomDomainList.html#new/1","title":"AntikytheraCore.Config.Gear.CustomDomainList.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.CustomDomainList.html#new!/1","title":"AntikytheraCore.Config.Gear.CustomDomainList.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.CustomDomainList.html#valid?/1","title":"AntikytheraCore.Config.Gear.CustomDomainList.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Config.Gear.CustomDomainList.html#t:t/0","title":"AntikytheraCore.Config.Gear.CustomDomainList.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Conn.html","title":"AntikytheraCore.Conn","type":"module"},{"doc":"","ref":"AntikytheraCore.Conn.html#gear_name/1","title":"AntikytheraCore.Conn.gear_name/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Conn.html#make_from_cowboy_req/4","title":"AntikytheraCore.Conn.make_from_cowboy_req/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Conn.html#make_from_g2g_req_and_context/6","title":"AntikytheraCore.Conn.make_from_g2g_req_and_context/6","type":"function"},{"doc":"","ref":"AntikytheraCore.Conn.html#reply_as_cowboy_res/2","title":"AntikytheraCore.Conn.reply_as_cowboy_res/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Conn.html#reply_as_g2g_res/1","title":"AntikytheraCore.Conn.reply_as_g2g_res/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Conn.html#request_info/1","title":"AntikytheraCore.Conn.request_info/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Conn.html#run_before_send/2","title":"AntikytheraCore.Conn.run_before_send/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Conn.html#validate/1","title":"AntikytheraCore.Conn.validate/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Context.html","title":"AntikytheraCore.Context","type":"module"},{"doc":"","ref":"AntikytheraCore.Context.html#make/4","title":"AntikytheraCore.Context.make/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Context.html#make_context_id/1","title":"AntikytheraCore.Context.make_context_id/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Context.html#make_context_id/2","title":"AntikytheraCore.Context.make_context_id/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Cookies.html","title":"AntikytheraCore.Cookies","type":"module"},{"doc":"","ref":"AntikytheraCore.Cookies.html#make_from_cowboy_req/1","title":"AntikytheraCore.Cookies.make_from_cowboy_req/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Cookies.html#merge_cookies_to_cowboy_req/2","title":"AntikytheraCore.Cookies.merge_cookies_to_cowboy_req/2","type":"function"},{"doc":"Periodically polls antikythera's config file and apply changes to the cache (in ETS table).\n\nNote that core config is loaded in `AntikytheraCore.start/2` and thus\nthis `GenServer`'s responsibility is to keep up with the latest changes.","ref":"AntikytheraCore.CoreConfigPoller.html","title":"AntikytheraCore.CoreConfigPoller","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.CoreConfigPoller.html#child_spec/1","title":"AntikytheraCore.CoreConfigPoller.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.CoreConfigPoller.html#start_link/1","title":"AntikytheraCore.CoreConfigPoller.start_link/1","type":"function"},{"doc":"Translator for `Logger`, installed via application config.\n\nMost of translations are delegated to `Logger.Translator`;\nthis translator neglects messages of the following types:\n\n- SASL progress report\n- error log emitted by syn about mnesia down event (when a node in the cluster is terminated)\n- supervisor report on brutal kill of a worker process in PoolSup (when a too-long-running worker is stopped)","ref":"AntikytheraCore.ErlangLogTranslator.html","title":"AntikytheraCore.ErlangLogTranslator","type":"module"},{"doc":"","ref":"AntikytheraCore.ErlangLogTranslator.html#translate/4","title":"AntikytheraCore.ErlangLogTranslator.translate/4","type":"function"},{"doc":"A `GenServer` to hold number of errors reported to each OTP application's alert manager process (implemented by `AntikytheraCore.Alert.Manager`).\n\nError counts are reported by `AntikytheraCore.Alert.ErrorCountReporter` installed in `AntikytheraCore.Alert.Manager`.\nThe received error counts are stored with `otp_app_name` and the timestamp (in minute).\nError counts accumulated in this `GenServer` can be fetched via HTTP: see `AntikytheraCore.Handler.SystemInfoExporter.ErrorCount`.\n\nAt the beginning of each minute,\n- newly-received error counts become visible from `get/1` and `get_total/0`, and\n- error counts having too old timestamps are cleaned up from the process state.","ref":"AntikytheraCore.ErrorCountsAccumulator.html","title":"AntikytheraCore.ErrorCountsAccumulator","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ErrorCountsAccumulator.html#child_spec/1","title":"AntikytheraCore.ErrorCountsAccumulator.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.html#get/1","title":"AntikytheraCore.ErrorCountsAccumulator.get/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.html#get_total/0","title":"AntikytheraCore.ErrorCountsAccumulator.get_total/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.html#start_link/1","title":"AntikytheraCore.ErrorCountsAccumulator.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.html#submit/2","title":"AntikytheraCore.ErrorCountsAccumulator.submit/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.html#t:results/0","title":"AntikytheraCore.ErrorCountsAccumulator.results/0","type":"type"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html","title":"AntikytheraCore.ErrorCountsAccumulator.State","type":"module"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#add/3","title":"AntikytheraCore.ErrorCountsAccumulator.State.add/3","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#advance_to_next_minute/2","title":"AntikytheraCore.ErrorCountsAccumulator.State.advance_to_next_minute/2","type":"function"},{"doc":"Type-aware getter for counts.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#counts/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.counts/1","type":"function"},{"doc":"Type-aware setter for counts.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#counts/2","title":"AntikytheraCore.ErrorCountsAccumulator.State.counts/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#get/2","title":"AntikytheraCore.ErrorCountsAccumulator.State.get/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#get_total/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.get_total/1","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.ErrorCountsAccumulator.State by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#new/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#new!/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.new!/1","type":"function"},{"doc":"Type-aware getter for now_minute.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#now_minute/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.now_minute/1","type":"function"},{"doc":"Type-aware setter for now_minute.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#now_minute/2","title":"AntikytheraCore.ErrorCountsAccumulator.State.now_minute/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.ErrorCountsAccumulator.State with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#update/2","title":"AntikytheraCore.ErrorCountsAccumulator.State.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#update!/2","title":"AntikytheraCore.ErrorCountsAccumulator.State.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#valid?/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#t:results/0","title":"AntikytheraCore.ErrorCountsAccumulator.State.results/0","type":"type"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.html#t:t/0","title":"AntikytheraCore.ErrorCountsAccumulator.State.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.html","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute","type":"module"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.html#new/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.html#new!/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.html#valid?/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.html#t:t/0","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinute.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.html","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp","type":"module"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.html#new/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.html#new!/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.html#valid?/1","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.html#t:t/0","title":"AntikytheraCore.ErrorCountsAccumulator.State.CountByMinuteByApp.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Ets.html","title":"AntikytheraCore.Ets","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.html#create_read_optimized_table/1","title":"AntikytheraCore.Ets.create_read_optimized_table/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.html#init_all/0","title":"AntikytheraCore.Ets.init_all/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.html","title":"AntikytheraCore.Ets.ConfigCache","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.html#init/0","title":"AntikytheraCore.Ets.ConfigCache.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.html#table_name/0","title":"AntikytheraCore.Ets.ConfigCache.table_name/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.Core.html","title":"AntikytheraCore.Ets.ConfigCache.Core","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.Core.html#read/0","title":"AntikytheraCore.Ets.ConfigCache.Core.read/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.Core.html#write/1","title":"AntikytheraCore.Ets.ConfigCache.Core.write/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.Gear.html","title":"AntikytheraCore.Ets.ConfigCache.Gear","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.Gear.html#read/1","title":"AntikytheraCore.Ets.ConfigCache.Gear.read/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.ConfigCache.Gear.html#write/2","title":"AntikytheraCore.Ets.ConfigCache.Gear.write/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.GearActionRunnerPools.html","title":"AntikytheraCore.Ets.GearActionRunnerPools","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.GearActionRunnerPools.html#init/0","title":"AntikytheraCore.Ets.GearActionRunnerPools.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.GearActionRunnerPools.html#table_name/0","title":"AntikytheraCore.Ets.GearActionRunnerPools.table_name/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.Memcache.html","title":"AntikytheraCore.Ets.Memcache","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.Memcache.html#delete/2","title":"AntikytheraCore.Ets.Memcache.delete/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.Memcache.html#init/0","title":"AntikytheraCore.Ets.Memcache.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.Memcache.html#read/2","title":"AntikytheraCore.Ets.Memcache.read/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.Memcache.html#table_name/0","title":"AntikytheraCore.Ets.Memcache.table_name/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.Memcache.html#write/5","title":"AntikytheraCore.Ets.Memcache.write/5","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.SystemCache.html","title":"AntikytheraCore.Ets.SystemCache","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.SystemCache.html#init/0","title":"AntikytheraCore.Ets.SystemCache.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.SystemCache.html#table_name/0","title":"AntikytheraCore.Ets.SystemCache.table_name/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.TenantToGearsMapping.html","title":"AntikytheraCore.Ets.TenantToGearsMapping","type":"module"},{"doc":"","ref":"AntikytheraCore.Ets.TenantToGearsMapping.html#associated?/2","title":"AntikytheraCore.Ets.TenantToGearsMapping.associated?/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.TenantToGearsMapping.html#init/0","title":"AntikytheraCore.Ets.TenantToGearsMapping.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.TenantToGearsMapping.html#set/2","title":"AntikytheraCore.Ets.TenantToGearsMapping.set/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.TenantToGearsMapping.html#table_name/0","title":"AntikytheraCore.Ets.TenantToGearsMapping.table_name/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Ets.TenantToGearsMapping.html#unset/1","title":"AntikytheraCore.Ets.TenantToGearsMapping.unset/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.html","title":"AntikytheraCore.ExecutorPool","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.html#apply_setting/2","title":"AntikytheraCore.ExecutorPool.apply_setting/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.html#kill_executor_pool/1","title":"AntikytheraCore.ExecutorPool.kill_executor_pool/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.html#start_executor_pool/2","title":"AntikytheraCore.ExecutorPool.start_executor_pool/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.html#start_link/3","title":"AntikytheraCore.ExecutorPool.start_link/3","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.html#start_per_gear_executor_pool/1","title":"AntikytheraCore.ExecutorPool.start_per_gear_executor_pool/1","type":"function"},{"doc":"A `GenServer` (whose lifecycle is managed by `PoolSup.Multi`) that executes gears' controller actions.","ref":"AntikytheraCore.ExecutorPool.ActionRunner.html","title":"AntikytheraCore.ExecutorPool.ActionRunner","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.ActionRunner.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.ActionRunner.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.ActionRunner.html#run/4","title":"AntikytheraCore.ExecutorPool.ActionRunner.run/4","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.ActionRunner.html#start_link/1","title":"AntikytheraCore.ExecutorPool.ActionRunner.start_link/1","type":"function"},{"doc":"A broker that finds a pair of \"a runnable job in job queue\" and \"an available `AsyncJobRunner` process in worker pool\".\n\nThis `GenServer` becomes `:active` when both antikythera and the job queue for the same executor pool are ready.\nWhen entering `:active` phase it notifies `AntikytheraCore.TerminationManager` of the pid\nso that it will be notified afterward.\nBefore host termination it becomes `:inactive` on receipt of a message from `AntikytheraCore.TerminationManager`;\nfrom then on it won't start new jobs.\n\nDuring `:active` phase it checks out a process from the async job worker pool of the same executor pool,\nfetches a job from the job queue, and tells the process to run the job.\n\nCommunications between broker and job queue are basically event-driven,\ni.e., no polling mechanism is needed for them to work, most of the time.\nHowever, in rare occasions, messages between them may be lost\ndue to e.g. failure of majority members in the job queue's consensus group.\nTherefore we also use periodic polling to recover from this kind of troubles.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.html","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.html#deactivate/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.deactivate/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.html#notify_job_registered/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.notify_job_registered/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.html#notify_pool_capacity_may_have_changed/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.notify_pool_capacity_may_have_changed/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.html#start_link/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.html","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.html#new/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.html#new!/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.html#valid?/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.html#values/0","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.values/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.html#t:t/0","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.Phase.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State","type":"module"},{"doc":"Creates a new instance of AntikytheraCore.ExecutorPool.AsyncJobBroker.State by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#new/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#new!/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.new!/1","type":"function"},{"doc":"Type-aware getter for phase.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#phase/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.phase/1","type":"function"},{"doc":"Type-aware setter for phase.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#phase/2","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.phase/2","type":"function"},{"doc":"Type-aware getter for pool_name.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#pool_name/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.pool_name/1","type":"function"},{"doc":"Type-aware setter for pool_name.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#pool_name/2","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.pool_name/2","type":"function"},{"doc":"Type-aware getter for queue_name.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#queue_name/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.queue_name/1","type":"function"},{"doc":"Type-aware setter for queue_name.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#queue_name/2","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.queue_name/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.ExecutorPool.AsyncJobBroker.State with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#update/2","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#update!/2","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#valid?/1","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.html#t:t/0","title":"AntikytheraCore.ExecutorPool.AsyncJobBroker.State.t/0","type":"type"},{"doc":"A `GenServer` for logging, which is used in `AntikytheraCore.ExecutorPool.AsyncJobRunner`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer.html","title":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer.html#info/1","title":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer.info/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer.html#start_link/1","title":"AntikytheraCore.ExecutorPool.AsyncJobLog.Writer.start_link/1","type":"function"},{"doc":"An ephemeral `GenServer` process under `PoolSup` that manages an execution of an async job.\n\nActual execution of gear's code is done within a separate process for the following reasons:\n- for cleaner error handling\n- to introduce timeout of job execution\n- to be responsive to system messages\n\nWhen the separate process terminates, this `GenServer` updates/deletes the locked job in the queue\ndepending on the result of the execution.\n\nNote that when no job is registered in the job queue, successfully checked-out process is immediately checked in.\nSee implementation of `AntikytheraCore.ExecutorPool.AsyncJobBroker`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobRunner.html","title":"AntikytheraCore.ExecutorPool.AsyncJobRunner","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobRunner.html#abandon_callback_max_duration/0","title":"AntikytheraCore.ExecutorPool.AsyncJobRunner.abandon_callback_max_duration/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.AsyncJobRunner.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.AsyncJobRunner.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobRunner.html#do_abandon/4","title":"AntikytheraCore.ExecutorPool.AsyncJobRunner.do_abandon/4","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobRunner.html#do_run/4","title":"AntikytheraCore.ExecutorPool.AsyncJobRunner.do_run/4","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobRunner.html#run/4","title":"AntikytheraCore.ExecutorPool.AsyncJobRunner.run/4","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.AsyncJobRunner.html#start_link/1","title":"AntikytheraCore.ExecutorPool.AsyncJobRunner.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Id.html","title":"AntikytheraCore.ExecutorPool.Id","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Id.html#validate_association/2","title":"AntikytheraCore.ExecutorPool.Id.validate_association/2","type":"function"},{"doc":"A `GenServer` to manage memcache for each executor pool.","ref":"AntikytheraCore.ExecutorPool.MemcacheWriter.html","title":"AntikytheraCore.ExecutorPool.MemcacheWriter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.MemcacheWriter.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.MemcacheWriter.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.MemcacheWriter.html#max_records/0","title":"AntikytheraCore.ExecutorPool.MemcacheWriter.max_records/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.MemcacheWriter.html#start_link/1","title":"AntikytheraCore.ExecutorPool.MemcacheWriter.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.MemcacheWriter.html#write/5","title":"AntikytheraCore.ExecutorPool.MemcacheWriter.write/5","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html","title":"AntikytheraCore.ExecutorPool.RegisteredName","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#action_runner_pool_multi/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.action_runner_pool_multi/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#action_runner_pool_multi_unsafe/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.action_runner_pool_multi_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#async_job_broker/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.async_job_broker/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#async_job_broker_unsafe/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.async_job_broker_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#async_job_queue/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.async_job_queue/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#async_job_queue_unsafe/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.async_job_queue_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#async_job_runner_pool/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.async_job_runner_pool/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#async_job_runner_pool_unsafe/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.async_job_runner_pool_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#memcache_writer/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.memcache_writer/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#memcache_writer_unsafe/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.memcache_writer_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#supervisor/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.supervisor/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#supervisor_unsafe/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.supervisor_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#websocket_connections_counter/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.websocket_connections_counter/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.RegisteredName.html#websocket_connections_counter_unsafe/1","title":"AntikytheraCore.ExecutorPool.RegisteredName.websocket_connections_counter_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Setting.html","title":"AntikytheraCore.ExecutorPool.Setting","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Setting.html#all/0","title":"AntikytheraCore.ExecutorPool.Setting.all/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Setting.html#default/0","title":"AntikytheraCore.ExecutorPool.Setting.default/0","type":"function"},{"doc":"Type-aware getter for n_pools_a.","ref":"AntikytheraCore.ExecutorPool.Setting.html#n_pools_a/1","title":"AntikytheraCore.ExecutorPool.Setting.n_pools_a/1","type":"function"},{"doc":"Type-aware setter for n_pools_a.","ref":"AntikytheraCore.ExecutorPool.Setting.html#n_pools_a/2","title":"AntikytheraCore.ExecutorPool.Setting.n_pools_a/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.ExecutorPool.Setting by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.ExecutorPool.Setting.html#new/1","title":"AntikytheraCore.ExecutorPool.Setting.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ExecutorPool.Setting.html#new!/1","title":"AntikytheraCore.ExecutorPool.Setting.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Setting.html#of_gear/1","title":"AntikytheraCore.ExecutorPool.Setting.of_gear/1","type":"function"},{"doc":"Type-aware getter for pool_size_a.","ref":"AntikytheraCore.ExecutorPool.Setting.html#pool_size_a/1","title":"AntikytheraCore.ExecutorPool.Setting.pool_size_a/1","type":"function"},{"doc":"Type-aware setter for pool_size_a.","ref":"AntikytheraCore.ExecutorPool.Setting.html#pool_size_a/2","title":"AntikytheraCore.ExecutorPool.Setting.pool_size_a/2","type":"function"},{"doc":"Type-aware getter for pool_size_j.","ref":"AntikytheraCore.ExecutorPool.Setting.html#pool_size_j/1","title":"AntikytheraCore.ExecutorPool.Setting.pool_size_j/1","type":"function"},{"doc":"Type-aware setter for pool_size_j.","ref":"AntikytheraCore.ExecutorPool.Setting.html#pool_size_j/2","title":"AntikytheraCore.ExecutorPool.Setting.pool_size_j/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.ExecutorPool.Setting with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.ExecutorPool.Setting.html#update/2","title":"AntikytheraCore.ExecutorPool.Setting.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ExecutorPool.Setting.html#update!/2","title":"AntikytheraCore.ExecutorPool.Setting.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.ExecutorPool.Setting.html#valid?/1","title":"AntikytheraCore.ExecutorPool.Setting.valid?/1","type":"function"},{"doc":"Type-aware getter for ws_max_connections.","ref":"AntikytheraCore.ExecutorPool.Setting.html#ws_max_connections/1","title":"AntikytheraCore.ExecutorPool.Setting.ws_max_connections/1","type":"function"},{"doc":"Type-aware setter for ws_max_connections.","ref":"AntikytheraCore.ExecutorPool.Setting.html#ws_max_connections/2","title":"AntikytheraCore.ExecutorPool.Setting.ws_max_connections/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Setting.html#t:t/0","title":"AntikytheraCore.ExecutorPool.Setting.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Sup.html","title":"AntikytheraCore.ExecutorPool.Sup","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.Sup.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.Sup.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.Sup.html#start_link/1","title":"AntikytheraCore.ExecutorPool.Sup.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html","title":"AntikytheraCore.ExecutorPool.TenantSetting","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#associate_with_gear/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.associate_with_gear/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#default/0","title":"AntikytheraCore.ExecutorPool.TenantSetting.default/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#disassociate_from_gear/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.disassociate_from_gear/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#fetch_all_modified/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.fetch_all_modified/1","type":"function"},{"doc":"Type-aware getter for gears.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#gears/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.gears/1","type":"function"},{"doc":"Type-aware setter for gears.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#gears/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.gears/2","type":"function"},{"doc":"Type-aware getter for n_pools_a.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#n_pools_a/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.n_pools_a/1","type":"function"},{"doc":"Type-aware setter for n_pools_a.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#n_pools_a/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.n_pools_a/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.ExecutorPool.TenantSetting by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#new/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#new!/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#persist_new_tenant_and_broadcast/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.persist_new_tenant_and_broadcast/2","type":"function"},{"doc":"Type-aware getter for pool_size_a.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#pool_size_a/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.pool_size_a/1","type":"function"},{"doc":"Type-aware setter for pool_size_a.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#pool_size_a/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.pool_size_a/2","type":"function"},{"doc":"Type-aware getter for pool_size_j.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#pool_size_j/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.pool_size_j/1","type":"function"},{"doc":"Type-aware setter for pool_size_j.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#pool_size_j/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.pool_size_j/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.ExecutorPool.TenantSetting with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#update/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#update!/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#valid?/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.valid?/1","type":"function"},{"doc":"Type-aware getter for ws_max_connections.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#ws_max_connections/1","title":"AntikytheraCore.ExecutorPool.TenantSetting.ws_max_connections/1","type":"function"},{"doc":"Type-aware setter for ws_max_connections.","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#ws_max_connections/2","title":"AntikytheraCore.ExecutorPool.TenantSetting.ws_max_connections/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TenantSetting.html#t:t/0","title":"AntikytheraCore.ExecutorPool.TenantSetting.t/0","type":"type"},{"doc":"A `GenServer` to periodically send command to the leader process of the corresponding async job queue.\n\nPurposes of the command are twofold:\n- to trigger executions of scheduled jobs that have become runnable, and\n- to get metrics from the queue.\n\nThe interaction between this `GenServer` and the leader is meaningful only when the leader resides in the same node.","ref":"AntikytheraCore.ExecutorPool.TimedJobStarter.html","title":"AntikytheraCore.ExecutorPool.TimedJobStarter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.TimedJobStarter.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.TimedJobStarter.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.TimedJobStarter.html#start_link/1","title":"AntikytheraCore.ExecutorPool.TimedJobStarter.start_link/1","type":"function"},{"doc":"A `GenServer` that periodically fetches usage of executor pool and submits it.\n\nDepends on\n- `AntikytheraCore.MetricsUploader` process for antikythera\n- `PoolSup.Multi` process for action runners in the same executor pool\n- `PoolSup` process for async job runners in the same executor pool\n- `AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter` process in the same executor pool","ref":"AntikytheraCore.ExecutorPool.UsageReporter.html","title":"AntikytheraCore.ExecutorPool.UsageReporter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.UsageReporter.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.UsageReporter.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.UsageReporter.html#start_link/1","title":"AntikytheraCore.ExecutorPool.UsageReporter.start_link/1","type":"function"},{"doc":"A `GenServer` to count number of websocket connections that belong to the executor pool.\n\nBefore termination of the current host, all monitored websocket connections will be disconnected\nin order to prompt clients to reconnect to other active nodes\n(this is executed within temporary processes but uses monitors created by this `GenServer`).","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html#child_spec/1","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html#gradually_terminate_all_ws_connections/0","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.gradually_terminate_all_ws_connections/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html#increment/2","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.increment/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html#set_max/2","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.set_max/2","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html#start_link/1","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html#start_terminating_all_ws_connections/0","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.start_terminating_all_ws_connections/0","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.html#stats/1","title":"AntikytheraCore.ExecutorPool.WebsocketConnectionsCounter.stats/1","type":"function"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WsConnectionsCapping.html","title":"AntikytheraCore.ExecutorPool.WsConnectionsCapping","type":"module"},{"doc":"","ref":"AntikytheraCore.ExecutorPool.WsConnectionsCapping.html#cap_based_on_available_memory/1","title":"AntikytheraCore.ExecutorPool.WsConnectionsCapping.cap_based_on_available_memory/1","type":"function"},{"doc":"","ref":"AntikytheraCore.FileSetup.html","title":"AntikytheraCore.FileSetup","type":"module"},{"doc":"","ref":"AntikytheraCore.FileSetup.html#setup_files_and_ets_tables/0","title":"AntikytheraCore.FileSetup.setup_files_and_ets_tables/0","type":"function"},{"doc":"","ref":"AntikytheraCore.FileSetup.html#write_initial_antikythera_instance_version_to_history_if_non_cloud/1","title":"AntikytheraCore.FileSetup.write_initial_antikythera_instance_version_to_history_if_non_cloud/1","type":"function"},{"doc":"","ref":"AntikytheraCore.FileUtil.html","title":"AntikytheraCore.FileUtil","type":"module"},{"doc":"Periodically polls changes in gear configs.\n\nGear configs are persisted in a shared data storage (e.g. NFS) and are cached in ETS in each erlang node.\nNote that gear configs are loaded (cached into ETS) at each step of\n\n- `AntikytheraCore.start/2`: all existing gear configs are loaded\n- each gear's `start/2`: the gear's gear config is loaded\n\nThus this GenServer's responsibility is just to keep up with changes in the shared data storage.\n\nDepends on `AntikytheraCore.GearManager` (when applying changes in gear configs results in update of cowboy routing).","ref":"AntikytheraCore.GearConfigPoller.html","title":"AntikytheraCore.GearConfigPoller","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.GearConfigPoller.html#child_spec/1","title":"AntikytheraCore.GearConfigPoller.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearConfigPoller.html#start_link/1","title":"AntikytheraCore.GearConfigPoller.start_link/1","type":"function"},{"doc":"Periodically polls gears' executor pool settings and apply changes.\n\nSettings of gear executor pools are embedded in antikythera core config.\nLoading core config file and caching into ETS is done by `CoreConfigPoller`; this `GenServer` watches for the cache.","ref":"AntikytheraCore.GearExecutorPoolsManager.html","title":"AntikytheraCore.GearExecutorPoolsManager","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.GearExecutorPoolsManager.html#child_spec/1","title":"AntikytheraCore.GearExecutorPoolsManager.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearExecutorPoolsManager.html#start_link/1","title":"AntikytheraCore.GearExecutorPoolsManager.start_link/1","type":"function"},{"doc":"Helpers to get/set context ID.\n\nContext IDs are mainly to be used for logging.\nIn order to keep `Logger`'s interface clean (i.e. not to make `context_id` trump data solely for logging purpose),\ncontext ID is stored in each process's process dictionary and fetched when a process emits a log message.","ref":"AntikytheraCore.GearLog.ContextHelper.html","title":"AntikytheraCore.GearLog.ContextHelper","type":"module"},{"doc":"","ref":"AntikytheraCore.GearLog.ContextHelper.html#get!/0","title":"AntikytheraCore.GearLog.ContextHelper.get!/0","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.ContextHelper.html#set/1","title":"AntikytheraCore.GearLog.ContextHelper.set/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html","title":"AntikytheraCore.GearLog.FileHandle","type":"module"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html#close/1","title":"AntikytheraCore.GearLog.FileHandle.close/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html#open/2","title":"AntikytheraCore.GearLog.FileHandle.open/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html#restore_write_to_terminal/1","title":"AntikytheraCore.GearLog.FileHandle.restore_write_to_terminal/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html#rotate/1","title":"AntikytheraCore.GearLog.FileHandle.rotate/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html#set_write_to_terminal/2","title":"AntikytheraCore.GearLog.FileHandle.set_write_to_terminal/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html#write/2","title":"AntikytheraCore.GearLog.FileHandle.write/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.html#t:t/0","title":"AntikytheraCore.GearLog.FileHandle.t/0","type":"opaque"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.SizeCheck.html","title":"AntikytheraCore.GearLog.FileHandle.SizeCheck","type":"module"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.SizeCheck.html#check_now?/2","title":"AntikytheraCore.GearLog.FileHandle.SizeCheck.check_now?/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.FileHandle.SizeCheck.html#exceeds_limit?/1","title":"AntikytheraCore.GearLog.FileHandle.SizeCheck.exceeds_limit?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html","title":"AntikytheraCore.GearLog.Level","type":"module"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#default/0","title":"AntikytheraCore.GearLog.Level.default/0","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#from_string/1","title":"AntikytheraCore.GearLog.Level.from_string/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#new/1","title":"AntikytheraCore.GearLog.Level.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#new!/1","title":"AntikytheraCore.GearLog.Level.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#valid?/1","title":"AntikytheraCore.GearLog.Level.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#values/0","title":"AntikytheraCore.GearLog.Level.values/0","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#write_to_log?/2","title":"AntikytheraCore.GearLog.Level.write_to_log?/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Level.html#t:t/0","title":"AntikytheraCore.GearLog.Level.t/0","type":"type"},{"doc":"A Module to write and rotate a log file using `AntikytheraCore.GearLog.FileHandle`.","ref":"AntikytheraCore.GearLog.LogRotation.html","title":"AntikytheraCore.GearLog.LogRotation","type":"module"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.html#init/3","title":"AntikytheraCore.GearLog.LogRotation.init/3","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.html#restore_write_to_terminal/1","title":"AntikytheraCore.GearLog.LogRotation.restore_write_to_terminal/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.html#rotate/1","title":"AntikytheraCore.GearLog.LogRotation.rotate/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.html#set_write_to_terminal/2","title":"AntikytheraCore.GearLog.LogRotation.set_write_to_terminal/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.html#terminate/1","title":"AntikytheraCore.GearLog.LogRotation.terminate/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.html#write_log/2","title":"AntikytheraCore.GearLog.LogRotation.write_log/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.State.html","title":"AntikytheraCore.GearLog.LogRotation.State","type":"module"},{"doc":"Type-aware getter for empty?.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#empty?/1","title":"AntikytheraCore.GearLog.LogRotation.State.empty?/1","type":"function"},{"doc":"Type-aware setter for empty?.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#empty?/2","title":"AntikytheraCore.GearLog.LogRotation.State.empty?/2","type":"function"},{"doc":"Type-aware getter for file_handle.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#file_handle/1","title":"AntikytheraCore.GearLog.LogRotation.State.file_handle/1","type":"function"},{"doc":"Type-aware setter for file_handle.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#file_handle/2","title":"AntikytheraCore.GearLog.LogRotation.State.file_handle/2","type":"function"},{"doc":"Type-aware getter for interval.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#interval/1","title":"AntikytheraCore.GearLog.LogRotation.State.interval/1","type":"function"},{"doc":"Type-aware setter for interval.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#interval/2","title":"AntikytheraCore.GearLog.LogRotation.State.interval/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.GearLog.LogRotation.State by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.GearLog.LogRotation.State.html#new/1","title":"AntikytheraCore.GearLog.LogRotation.State.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#new!/1","title":"AntikytheraCore.GearLog.LogRotation.State.new!/1","type":"function"},{"doc":"Type-aware getter for timer.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#timer/1","title":"AntikytheraCore.GearLog.LogRotation.State.timer/1","type":"function"},{"doc":"Type-aware setter for timer.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#timer/2","title":"AntikytheraCore.GearLog.LogRotation.State.timer/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.GearLog.LogRotation.State with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#update/2","title":"AntikytheraCore.GearLog.LogRotation.State.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#update!/2","title":"AntikytheraCore.GearLog.LogRotation.State.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.GearLog.LogRotation.State.html#valid?/1","title":"AntikytheraCore.GearLog.LogRotation.State.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.LogRotation.State.html#t:t/0","title":"AntikytheraCore.GearLog.LogRotation.State.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.GearLog.Message.html","title":"AntikytheraCore.GearLog.Message","type":"module"},{"doc":"","ref":"AntikytheraCore.GearLog.Message.html#t:t/0","title":"AntikytheraCore.GearLog.Message.t/0","type":"type"},{"doc":"A `GenServer` that writes log messages from each gear's `Logger` process into a gzipped file.\n\nThis `GenServer` is spawned per gear; each of which resides in the gear's supervision tree.\n\nAlthough opened log files are regularly rotated, this `GenServer` also supports on-demand log rotation.\nAfter each successful log rotation, old log file is uploaded to cloud storage.","ref":"AntikytheraCore.GearLog.Writer.html","title":"AntikytheraCore.GearLog.Writer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.GearLog.Writer.html#child_spec/1","title":"AntikytheraCore.GearLog.Writer.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#debug/2","title":"AntikytheraCore.GearLog.Writer.debug/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#debug/4","title":"AntikytheraCore.GearLog.Writer.debug/4","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#error/2","title":"AntikytheraCore.GearLog.Writer.error/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#error/4","title":"AntikytheraCore.GearLog.Writer.error/4","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#info/2","title":"AntikytheraCore.GearLog.Writer.info/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#info/4","title":"AntikytheraCore.GearLog.Writer.info/4","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#restore_write_to_terminal/1","title":"AntikytheraCore.GearLog.Writer.restore_write_to_terminal/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#rotate/1","title":"AntikytheraCore.GearLog.Writer.rotate/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#rotate_and_start_upload_in_all_nodes/1","title":"AntikytheraCore.GearLog.Writer.rotate_and_start_upload_in_all_nodes/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#set_min_level/2","title":"AntikytheraCore.GearLog.Writer.set_min_level/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#set_write_to_terminal/2","title":"AntikytheraCore.GearLog.Writer.set_write_to_terminal/2","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.html#start_link/1","title":"AntikytheraCore.GearLog.Writer.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.State.html","title":"AntikytheraCore.GearLog.Writer.State","type":"module"},{"doc":"Type-aware getter for log_state.","ref":"AntikytheraCore.GearLog.Writer.State.html#log_state/1","title":"AntikytheraCore.GearLog.Writer.State.log_state/1","type":"function"},{"doc":"Type-aware setter for log_state.","ref":"AntikytheraCore.GearLog.Writer.State.html#log_state/2","title":"AntikytheraCore.GearLog.Writer.State.log_state/2","type":"function"},{"doc":"Type-aware getter for min_level.","ref":"AntikytheraCore.GearLog.Writer.State.html#min_level/1","title":"AntikytheraCore.GearLog.Writer.State.min_level/1","type":"function"},{"doc":"Type-aware setter for min_level.","ref":"AntikytheraCore.GearLog.Writer.State.html#min_level/2","title":"AntikytheraCore.GearLog.Writer.State.min_level/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.GearLog.Writer.State by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.GearLog.Writer.State.html#new/1","title":"AntikytheraCore.GearLog.Writer.State.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.GearLog.Writer.State.html#new!/1","title":"AntikytheraCore.GearLog.Writer.State.new!/1","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.GearLog.Writer.State with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.GearLog.Writer.State.html#update/2","title":"AntikytheraCore.GearLog.Writer.State.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.GearLog.Writer.State.html#update!/2","title":"AntikytheraCore.GearLog.Writer.State.update!/2","type":"function"},{"doc":"Type-aware getter for uploader.","ref":"AntikytheraCore.GearLog.Writer.State.html#uploader/1","title":"AntikytheraCore.GearLog.Writer.State.uploader/1","type":"function"},{"doc":"Type-aware setter for uploader.","ref":"AntikytheraCore.GearLog.Writer.State.html#uploader/2","title":"AntikytheraCore.GearLog.Writer.State.uploader/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.GearLog.Writer.State.html#valid?/1","title":"AntikytheraCore.GearLog.Writer.State.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearLog.Writer.State.html#t:t/0","title":"AntikytheraCore.GearLog.Writer.State.t/0","type":"type"},{"doc":"A `GenServer` to keep track of names of currently running gears.","ref":"AntikytheraCore.GearManager.html","title":"AntikytheraCore.GearManager","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.GearManager.html#child_spec/1","title":"AntikytheraCore.GearManager.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearManager.html#gear_started/1","title":"AntikytheraCore.GearManager.gear_started/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearManager.html#gear_stopped/1","title":"AntikytheraCore.GearManager.gear_stopped/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearManager.html#running_gear_names/0","title":"AntikytheraCore.GearManager.running_gear_names/0","type":"function"},{"doc":"","ref":"AntikytheraCore.GearManager.html#start_link/1","title":"AntikytheraCore.GearManager.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html","title":"AntikytheraCore.GearModule","type":"module"},{"doc":"","ref":"AntikytheraCore.GearModule.html#alert_manager/1","title":"AntikytheraCore.GearModule.alert_manager/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#alert_manager_unsafe/1","title":"AntikytheraCore.GearModule.alert_manager_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#error_handler/1","title":"AntikytheraCore.GearModule.error_handler/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#error_handler_unsafe/1","title":"AntikytheraCore.GearModule.error_handler_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#logger/1","title":"AntikytheraCore.GearModule.logger/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#logger_unsafe/1","title":"AntikytheraCore.GearModule.logger_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#metrics_uploader/1","title":"AntikytheraCore.GearModule.metrics_uploader/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#metrics_uploader_unsafe/1","title":"AntikytheraCore.GearModule.metrics_uploader_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#request_helper_modules/1","title":"AntikytheraCore.GearModule.request_helper_modules/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#root_supervisor/1","title":"AntikytheraCore.GearModule.root_supervisor/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#root_supervisor_unsafe/1","title":"AntikytheraCore.GearModule.root_supervisor_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#router/1","title":"AntikytheraCore.GearModule.router/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#router_unsafe/1","title":"AntikytheraCore.GearModule.router_unsafe/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#template_module_from_context/1","title":"AntikytheraCore.GearModule.template_module_from_context/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearModule.html#top/1","title":"AntikytheraCore.GearModule.top/1","type":"function"},{"doc":"","ref":"AntikytheraCore.GearProcess.html","title":"AntikytheraCore.GearProcess","type":"module"},{"doc":"","ref":"AntikytheraCore.GearProcess.html#max_heap_size/0","title":"AntikytheraCore.GearProcess.max_heap_size/0","type":"function"},{"doc":"","ref":"AntikytheraCore.GearProcess.html#set_max_heap_size/0","title":"AntikytheraCore.GearProcess.set_max_heap_size/0","type":"function"},{"doc":"","ref":"AntikytheraCore.GearProcess.html#spawn_monitor/3","title":"AntikytheraCore.GearProcess.spawn_monitor/3","type":"function"},{"doc":"A much simplified version of `Task` module defined in the elixir standard library.\n\nWhile the standard `Task` requires the invoker to trap exit in order to handle errors (as it uses `spawn_link`),\nthis module doesn't link the invoker with the worker (uses `spawn_monitor` instead).\nAlso this version does not send stacktrace to error_logger (since it doesn't use :proc_lib functions to start child process).","ref":"AntikytheraCore.GearTask.html","title":"AntikytheraCore.GearTask","type":"module"},{"doc":"","ref":"AntikytheraCore.GearTask.html#exec_wait/4","title":"AntikytheraCore.GearTask.exec_wait/4","type":"function"},{"doc":"","ref":"AntikytheraCore.GearTask.html#t:mod_fun_args/0","title":"AntikytheraCore.GearTask.mod_fun_args/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.BodyParser.html","title":"AntikytheraCore.Handler.BodyParser","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.BodyParser.html#parse/1","title":"AntikytheraCore.Handler.BodyParser.parse/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html","title":"AntikytheraCore.Handler.CowboyReq","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html#method/1","title":"AntikytheraCore.Handler.CowboyReq.method/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html#path_info/1","title":"AntikytheraCore.Handler.CowboyReq.path_info/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html#query_params/2","title":"AntikytheraCore.Handler.CowboyReq.query_params/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html#request_body_pair/4","title":"AntikytheraCore.Handler.CowboyReq.request_body_pair/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html#with_conn/5","title":"AntikytheraCore.Handler.CowboyReq.with_conn/5","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html#t:result/1","title":"AntikytheraCore.Handler.CowboyReq.result/1","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyReq.html#t:routing_info/0","title":"AntikytheraCore.Handler.CowboyReq.routing_info/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyRouting.html","title":"AntikytheraCore.Handler.CowboyRouting","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyRouting.html#compiled_routes/2","title":"AntikytheraCore.Handler.CowboyRouting.compiled_routes/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyRouting.html#conflicts_with_default_domain?/2","title":"AntikytheraCore.Handler.CowboyRouting.conflicts_with_default_domain?/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyRouting.html#default_domain/2","title":"AntikytheraCore.Handler.CowboyRouting.default_domain/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.CowboyRouting.html#update_routing/2","title":"AntikytheraCore.Handler.CowboyRouting.update_routing/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.ExecutorPoolHelper.html","title":"AntikytheraCore.Handler.ExecutorPoolHelper","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.ExecutorPoolHelper.html#increment_ws_count/4","title":"AntikytheraCore.Handler.ExecutorPoolHelper.increment_ws_count/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.ExecutorPoolHelper.html#with_executor/4","title":"AntikytheraCore.Handler.ExecutorPoolHelper.with_executor/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.html","title":"AntikytheraCore.Handler.GearAction","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.html#split_path_to_segments/1","title":"AntikytheraCore.Handler.GearAction.split_path_to_segments/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.html#with_logging_and_metrics_reporting/3","title":"AntikytheraCore.Handler.GearAction.with_logging_and_metrics_reporting/3","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.G2g.html","title":"AntikytheraCore.Handler.GearAction.G2g","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.G2g.html#handle/3","title":"AntikytheraCore.Handler.GearAction.G2g.handle/3","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.Web.html","title":"AntikytheraCore.Handler.GearAction.Web","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.Web.html#t:http_reply/0","title":"AntikytheraCore.Handler.GearAction.Web.http_reply/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.GearAction.Web.html#t:ws_upgrade/0","title":"AntikytheraCore.Handler.GearAction.Web.ws_upgrade/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.GearError.html","title":"AntikytheraCore.Handler.GearError","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.GearError.html#bad_executor_pool_id/2","title":"AntikytheraCore.Handler.GearError.bad_executor_pool_id/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearError.html#bad_request/1","title":"AntikytheraCore.Handler.GearError.bad_request/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearError.html#error/3","title":"AntikytheraCore.Handler.GearError.error/3","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearError.html#internal_error_body/3","title":"AntikytheraCore.Handler.GearError.internal_error_body/3","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearError.html#no_route/1","title":"AntikytheraCore.Handler.GearError.no_route/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.GearError.html#ws_too_many_connections/1","title":"AntikytheraCore.Handler.GearError.ws_too_many_connections/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.Healthcheck.html","title":"AntikytheraCore.Handler.Healthcheck","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.Healthcheck.Initialized.html","title":"AntikytheraCore.Handler.Healthcheck.Initialized","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.Healthcheck.Uninitialized.html","title":"AntikytheraCore.Handler.Healthcheck.Uninitialized","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.HelperModules.html","title":"AntikytheraCore.Handler.HelperModules","type":"module"},{"doc":"Type-aware getter for logger.","ref":"AntikytheraCore.Handler.HelperModules.html#logger/1","title":"AntikytheraCore.Handler.HelperModules.logger/1","type":"function"},{"doc":"Type-aware setter for logger.","ref":"AntikytheraCore.Handler.HelperModules.html#logger/2","title":"AntikytheraCore.Handler.HelperModules.logger/2","type":"function"},{"doc":"Type-aware getter for metrics_uploader.","ref":"AntikytheraCore.Handler.HelperModules.html#metrics_uploader/1","title":"AntikytheraCore.Handler.HelperModules.metrics_uploader/1","type":"function"},{"doc":"Type-aware setter for metrics_uploader.","ref":"AntikytheraCore.Handler.HelperModules.html#metrics_uploader/2","title":"AntikytheraCore.Handler.HelperModules.metrics_uploader/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.Handler.HelperModules by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.Handler.HelperModules.html#new/1","title":"AntikytheraCore.Handler.HelperModules.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Handler.HelperModules.html#new!/1","title":"AntikytheraCore.Handler.HelperModules.new!/1","type":"function"},{"doc":"Type-aware getter for router.","ref":"AntikytheraCore.Handler.HelperModules.html#router/1","title":"AntikytheraCore.Handler.HelperModules.router/1","type":"function"},{"doc":"Type-aware setter for router.","ref":"AntikytheraCore.Handler.HelperModules.html#router/2","title":"AntikytheraCore.Handler.HelperModules.router/2","type":"function"},{"doc":"Type-aware getter for top.","ref":"AntikytheraCore.Handler.HelperModules.html#top/1","title":"AntikytheraCore.Handler.HelperModules.top/1","type":"function"},{"doc":"Type-aware setter for top.","ref":"AntikytheraCore.Handler.HelperModules.html#top/2","title":"AntikytheraCore.Handler.HelperModules.top/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.Handler.HelperModules with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.Handler.HelperModules.html#update/2","title":"AntikytheraCore.Handler.HelperModules.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Handler.HelperModules.html#update!/2","title":"AntikytheraCore.Handler.HelperModules.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.Handler.HelperModules.html#valid?/1","title":"AntikytheraCore.Handler.HelperModules.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.HelperModules.html#t:t/0","title":"AntikytheraCore.Handler.HelperModules.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.html","title":"AntikytheraCore.Handler.SystemInfoExporter","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken.html","title":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken.html#get/0","title":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken.get/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken.html#init/0","title":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken.html#with_valid_token/2","title":"AntikytheraCore.Handler.SystemInfoExporter.AccessToken.with_valid_token/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.ErrorCount.html","title":"AntikytheraCore.Handler.SystemInfoExporter.ErrorCount","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.Upgradability.html","title":"AntikytheraCore.Handler.SystemInfoExporter.Upgradability","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.SystemInfoExporter.Versions.html","title":"AntikytheraCore.Handler.SystemInfoExporter.Versions","type":"module"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html","title":"AntikytheraCore.Handler.WebsocketState","type":"module"},{"doc":"Type-aware getter for conn.","ref":"AntikytheraCore.Handler.WebsocketState.html#conn/1","title":"AntikytheraCore.Handler.WebsocketState.conn/1","type":"function"},{"doc":"Type-aware setter for conn.","ref":"AntikytheraCore.Handler.WebsocketState.html#conn/2","title":"AntikytheraCore.Handler.WebsocketState.conn/2","type":"function"},{"doc":"Type-aware getter for error_reason.","ref":"AntikytheraCore.Handler.WebsocketState.html#error_reason/1","title":"AntikytheraCore.Handler.WebsocketState.error_reason/1","type":"function"},{"doc":"Type-aware setter for error_reason.","ref":"AntikytheraCore.Handler.WebsocketState.html#error_reason/2","title":"AntikytheraCore.Handler.WebsocketState.error_reason/2","type":"function"},{"doc":"Type-aware getter for frames_received.","ref":"AntikytheraCore.Handler.WebsocketState.html#frames_received/1","title":"AntikytheraCore.Handler.WebsocketState.frames_received/1","type":"function"},{"doc":"Type-aware setter for frames_received.","ref":"AntikytheraCore.Handler.WebsocketState.html#frames_received/2","title":"AntikytheraCore.Handler.WebsocketState.frames_received/2","type":"function"},{"doc":"Type-aware getter for frames_sent.","ref":"AntikytheraCore.Handler.WebsocketState.html#frames_sent/1","title":"AntikytheraCore.Handler.WebsocketState.frames_sent/1","type":"function"},{"doc":"Type-aware setter for frames_sent.","ref":"AntikytheraCore.Handler.WebsocketState.html#frames_sent/2","title":"AntikytheraCore.Handler.WebsocketState.frames_sent/2","type":"function"},{"doc":"Type-aware getter for gear_impl_state.","ref":"AntikytheraCore.Handler.WebsocketState.html#gear_impl_state/1","title":"AntikytheraCore.Handler.WebsocketState.gear_impl_state/1","type":"function"},{"doc":"Type-aware setter for gear_impl_state.","ref":"AntikytheraCore.Handler.WebsocketState.html#gear_impl_state/2","title":"AntikytheraCore.Handler.WebsocketState.gear_impl_state/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#handle_client_message/2","title":"AntikytheraCore.Handler.WebsocketState.handle_client_message/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#handle_server_message/2","title":"AntikytheraCore.Handler.WebsocketState.handle_server_message/2","type":"function"},{"doc":"Type-aware getter for helper_modules.","ref":"AntikytheraCore.Handler.WebsocketState.html#helper_modules/1","title":"AntikytheraCore.Handler.WebsocketState.helper_modules/1","type":"function"},{"doc":"Type-aware setter for helper_modules.","ref":"AntikytheraCore.Handler.WebsocketState.html#helper_modules/2","title":"AntikytheraCore.Handler.WebsocketState.helper_modules/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#init/1","title":"AntikytheraCore.Handler.WebsocketState.init/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#make/3","title":"AntikytheraCore.Handler.WebsocketState.make/3","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.Handler.WebsocketState by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.Handler.WebsocketState.html#new/1","title":"AntikytheraCore.Handler.WebsocketState.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Handler.WebsocketState.html#new!/1","title":"AntikytheraCore.Handler.WebsocketState.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#terminate/2","title":"AntikytheraCore.Handler.WebsocketState.terminate/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.Handler.WebsocketState with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.Handler.WebsocketState.html#update/2","title":"AntikytheraCore.Handler.WebsocketState.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Handler.WebsocketState.html#update!/2","title":"AntikytheraCore.Handler.WebsocketState.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.Handler.WebsocketState.html#valid?/1","title":"AntikytheraCore.Handler.WebsocketState.valid?/1","type":"function"},{"doc":"Type-aware getter for ws_module.","ref":"AntikytheraCore.Handler.WebsocketState.html#ws_module/1","title":"AntikytheraCore.Handler.WebsocketState.ws_module/1","type":"function"},{"doc":"Type-aware setter for ws_module.","ref":"AntikytheraCore.Handler.WebsocketState.html#ws_module/2","title":"AntikytheraCore.Handler.WebsocketState.ws_module/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#t:callback_result/0","title":"AntikytheraCore.Handler.WebsocketState.callback_result/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#t:cowboy_ws_terminate_reason/0","title":"AntikytheraCore.Handler.WebsocketState.cowboy_ws_terminate_reason/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Handler.WebsocketState.html#t:t/0","title":"AntikytheraCore.Handler.WebsocketState.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Logger.html","title":"AntikytheraCore.Logger","type":"module"},{"doc":"","ref":"AntikytheraCore.Logger.html#error/1","title":"AntikytheraCore.Logger.error/1","type":"macro"},{"doc":"","ref":"AntikytheraCore.Logger.html#info/1","title":"AntikytheraCore.Logger.info/1","type":"macro"},{"doc":"Defines (1) a behaviour to compute metrics results from raw metrics data and (2) some implementations of the behaviour.\n\"Raw metrics data\" is values reported from other components (e.g. response time values).\n\"Metrics results\" are values that summarize characteristics of \"raw metrics data\" (e.g. average response time).\n\nAntikythera gear implementations must specify `name` of one of the strategies when submitting raw metric data.\nFor available `name` see the type definition in `AntikytheraCore.Metrics.AggregateStrategy.Name`.\nFor detailed information about each strategy refer to each module's documentation.","ref":"AntikytheraCore.Metrics.AggregateStrategy.html","title":"AntikytheraCore.Metrics.AggregateStrategy","type":"module"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#name_to_module/1","title":"AntikytheraCore.Metrics.AggregateStrategy.name_to_module/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#new/1","title":"AntikytheraCore.Metrics.AggregateStrategy.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#new!/1","title":"AntikytheraCore.Metrics.AggregateStrategy.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#valid?/1","title":"AntikytheraCore.Metrics.AggregateStrategy.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#values/0","title":"AntikytheraCore.Metrics.AggregateStrategy.values/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#t:data_t/0","title":"AntikytheraCore.Metrics.AggregateStrategy.data_t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#t:results_t/0","title":"AntikytheraCore.Metrics.AggregateStrategy.results_t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.html#t:t/0","title":"AntikytheraCore.Metrics.AggregateStrategy.t/0","type":"type"},{"doc":"Aggregate strategy that calculates the average value of incoming raw metrics data.","ref":"AntikytheraCore.Metrics.AggregateStrategy.Average.html","title":"AntikytheraCore.Metrics.AggregateStrategy.Average","type":"module"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour.html","title":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour","type":"behaviour"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour.html#c:init/1","title":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour.init/1","type":"callback"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour.html#c:merge/2","title":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour.merge/2","type":"callback"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour.html#c:results/1","title":"AntikytheraCore.Metrics.AggregateStrategy.Behaviour.results/1","type":"callback"},{"doc":"Aggregate strategy that simply takes the last value within each time window.","ref":"AntikytheraCore.Metrics.AggregateStrategy.Gauge.html","title":"AntikytheraCore.Metrics.AggregateStrategy.Gauge","type":"module"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Name.html","title":"AntikytheraCore.Metrics.AggregateStrategy.Name","type":"module"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Name.html#new/1","title":"AntikytheraCore.Metrics.AggregateStrategy.Name.new/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Name.html#new!/1","title":"AntikytheraCore.Metrics.AggregateStrategy.Name.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Name.html#valid?/1","title":"AntikytheraCore.Metrics.AggregateStrategy.Name.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Name.html#values/0","title":"AntikytheraCore.Metrics.AggregateStrategy.Name.values/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.AggregateStrategy.Name.html#t:t/0","title":"AntikytheraCore.Metrics.AggregateStrategy.Name.t/0","type":"type"},{"doc":"Aggregate strategy for request counts.\nThis calculates the following values from HTTP status codes of responses within a time window:\n\n- total number of requests\n- number of requests that result in 4XX response (client error)\n- number of requests that result in 5XX response (server error)\n\nNote that, strictly speaking, numbers generated by this aggreagate strategy do not include invalid requests\nthat are rejected by antikythera before arriving at gear's controller action,\ni.e. requests which do not match any of the defined routes, requests with malformed body, etc.","ref":"AntikytheraCore.Metrics.AggregateStrategy.RequestCount.html","title":"AntikytheraCore.Metrics.AggregateStrategy.RequestCount","type":"module"},{"doc":"Aggregate strategy that computes the sum of all values generated within each time window.","ref":"AntikytheraCore.Metrics.AggregateStrategy.Sum.html","title":"AntikytheraCore.Metrics.AggregateStrategy.Sum","type":"module"},{"doc":"Aggregate strategy for time distribution (such as response times).\nThis calculates the following values from time durations generated within a time window:\n\n- average\n- maximum\n- 95-percentile","ref":"AntikytheraCore.Metrics.AggregateStrategy.TimeDistribution.html","title":"AntikytheraCore.Metrics.AggregateStrategy.TimeDistribution","type":"module"},{"doc":"A bin-like data structure to hold per-minute, per-epool metrics data.\n\n    %{\n      {time_minute, epool_id} => %{\n        {metrics_type, strategy} => data_in_processing,\n        {metrics_type, strategy} => data_in_processing,\n        ...\n      },\n      ...\n    }","ref":"AntikytheraCore.Metrics.Buffer.html","title":"AntikytheraCore.Metrics.Buffer","type":"module"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#add/4","title":"AntikytheraCore.Metrics.Buffer.add/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#new/0","title":"AntikytheraCore.Metrics.Buffer.new/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#partition_ongoing_and_past/2","title":"AntikytheraCore.Metrics.Buffer.partition_ongoing_and_past/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#t:epool_id/0","title":"AntikytheraCore.Metrics.Buffer.epool_id/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#t:metrics_data_map/0","title":"AntikytheraCore.Metrics.Buffer.metrics_data_map/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#t:metrics_type/0","title":"AntikytheraCore.Metrics.Buffer.metrics_type/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#t:metrics_unit/0","title":"AntikytheraCore.Metrics.Buffer.metrics_unit/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#t:metrics_value/0","title":"AntikytheraCore.Metrics.Buffer.metrics_value/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#t:minute/0","title":"AntikytheraCore.Metrics.Buffer.minute/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Buffer.html#t:t/0","title":"AntikytheraCore.Metrics.Buffer.t/0","type":"type"},{"doc":"Data structure to represent metrics results computed from raw metrics data.\nResults are map of metrics data by minute-epool.\n\n    %{\n      {time_minute, epool_id} => %{\n        metrics_label => metrics_value,\n        metrics_label => metrics_value,\n        ...\n      },\n      ...\n    }","ref":"AntikytheraCore.Metrics.Results.html","title":"AntikytheraCore.Metrics.Results","type":"module"},{"doc":"","ref":"AntikytheraCore.Metrics.Results.html#compute_results/1","title":"AntikytheraCore.Metrics.Results.compute_results/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.Results.html#merge/2","title":"AntikytheraCore.Metrics.Results.merge/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.Results.html#new/0","title":"AntikytheraCore.Metrics.Results.new/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Metrics.Results.html#t:metrics_label/0","title":"AntikytheraCore.Metrics.Results.metrics_label/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Results.html#t:per_unit_results_map/0","title":"AntikytheraCore.Metrics.Results.per_unit_results_map/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Metrics.Results.html#t:t/0","title":"AntikytheraCore.Metrics.Results.t/0","type":"type"},{"doc":"A `GenServer` which buffers metrics data and periodically flushes the aggregated results to stable storage.","ref":"AntikytheraCore.MetricsUploader.html","title":"AntikytheraCore.MetricsUploader","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.MetricsUploader.html#child_spec/1","title":"AntikytheraCore.MetricsUploader.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.MetricsUploader.html#start_link/1","title":"AntikytheraCore.MetricsUploader.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.MetricsUploader.html#submit/3","title":"AntikytheraCore.MetricsUploader.submit/3","type":"function"},{"doc":"","ref":"AntikytheraCore.MetricsUploader.html#submit_custom_metrics/3","title":"AntikytheraCore.MetricsUploader.submit_custom_metrics/3","type":"function"},{"doc":"","ref":"AntikytheraCore.MetricsUploader.html#submit_with_time/4","title":"AntikytheraCore.MetricsUploader.submit_with_time/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Mix.AssetList.html","title":"AntikytheraCore.Mix.AssetList","type":"module"},{"doc":"","ref":"AntikytheraCore.Mix.AssetList.html#load_all/2","title":"AntikytheraCore.Mix.AssetList.load_all/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Mix.AssetList.html#write!/2","title":"AntikytheraCore.Mix.AssetList.write!/2","type":"function"},{"doc":"A GenServer that periodically removes extra nodes from mnesia schema.\n\nWhen a new node is started it automatically starts to sync both mnesia schema and data with other participating nodes.\nOn the other hand when a node is terminated it's not automatically removed from mnesia schema\n(because mnesia has no idea whether the node will re-join the cluster or not).\n\nThis GenServer periodically cleans up any already-terminated nodes from mnesia schema\nby using hosts information from the underlying infrastructure.\nWithout this cleanup the \"already terminated nodes\" would accumulate in mnesia schema\nand make startup of new nodes really slow (new node tries to sync with nonexisting nodes until timeout).\n\nDepends on `AntikytheraCore.ClusterHostsPoller`.","ref":"AntikytheraCore.MnesiaNodesCleaner.html","title":"AntikytheraCore.MnesiaNodesCleaner","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.MnesiaNodesCleaner.html#child_spec/1","title":"AntikytheraCore.MnesiaNodesCleaner.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.MnesiaNodesCleaner.html#start_link/1","title":"AntikytheraCore.MnesiaNodesCleaner.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.OsUtil.html","title":"AntikytheraCore.OsUtil","type":"module"},{"doc":"","ref":"AntikytheraCore.OsUtil.html#init/0","title":"AntikytheraCore.OsUtil.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.OsUtil.html#total_memory_size_in_bytes/0","title":"AntikytheraCore.OsUtil.total_memory_size_in_bytes/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html","title":"AntikytheraCore.Path","type":"module"},{"doc":"","ref":"AntikytheraCore.Path.html#antikythera_root_dir/0","title":"AntikytheraCore.Path.antikythera_root_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#atomic_write!/2","title":"AntikytheraCore.Path.atomic_write!/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#changed?/2","title":"AntikytheraCore.Path.changed?/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#compiled_core_dir/0","title":"AntikytheraCore.Path.compiled_core_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#compiled_gears_dir/0","title":"AntikytheraCore.Path.compiled_gears_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#config_encryption_key_path/0","title":"AntikytheraCore.Path.config_encryption_key_path/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#core_config_file_path/0","title":"AntikytheraCore.Path.core_config_file_path/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#core_log_file_path/1","title":"AntikytheraCore.Path.core_log_file_path/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#gear_config_dir/0","title":"AntikytheraCore.Path.gear_config_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#gear_config_file_path/1","title":"AntikytheraCore.Path.gear_config_file_path/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#gear_log_dir/1","title":"AntikytheraCore.Path.gear_log_dir/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#gear_log_file_path/1","title":"AntikytheraCore.Path.gear_log_file_path/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#gear_tmp_dir/0","title":"AntikytheraCore.Path.gear_tmp_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#history_dir/0","title":"AntikytheraCore.Path.history_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#list_modified_files/2","title":"AntikytheraCore.Path.list_modified_files/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#raft_persistence_dir_parent/0","title":"AntikytheraCore.Path.raft_persistence_dir_parent/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#system_info_access_token_path/0","title":"AntikytheraCore.Path.system_info_access_token_path/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#tenant_ids_file_path/0","title":"AntikytheraCore.Path.tenant_ids_file_path/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#tenant_setting_dir/0","title":"AntikytheraCore.Path.tenant_setting_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Path.html#tenant_setting_file_path/1","title":"AntikytheraCore.Path.tenant_setting_file_path/1","type":"function"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.MessageBuilder.html","title":"AntikytheraCore.PeriodicLog.MessageBuilder","type":"module"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.MessageBuilder.html#build_log/1","title":"AntikytheraCore.PeriodicLog.MessageBuilder.build_log/1","type":"function"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.MessageBuilder.html#init/0","title":"AntikytheraCore.PeriodicLog.MessageBuilder.init/0","type":"function"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.ReductionBuilder.html","title":"AntikytheraCore.PeriodicLog.ReductionBuilder","type":"module"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.ReductionBuilder.html#build_log/1","title":"AntikytheraCore.PeriodicLog.ReductionBuilder.build_log/1","type":"function"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.ReductionBuilder.html#init/0","title":"AntikytheraCore.PeriodicLog.ReductionBuilder.init/0","type":"function"},{"doc":"A `GenServer` that logs periodically","ref":"AntikytheraCore.PeriodicLog.Writer.html","title":"AntikytheraCore.PeriodicLog.Writer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.PeriodicLog.Writer.html#child_spec/1","title":"AntikytheraCore.PeriodicLog.Writer.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.Writer.html#start_link/1","title":"AntikytheraCore.PeriodicLog.Writer.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.Writer.State.html","title":"AntikytheraCore.PeriodicLog.Writer.State","type":"module"},{"doc":"Type-aware getter for build_mod.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#build_mod/1","title":"AntikytheraCore.PeriodicLog.Writer.State.build_mod/1","type":"function"},{"doc":"Type-aware setter for build_mod.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#build_mod/2","title":"AntikytheraCore.PeriodicLog.Writer.State.build_mod/2","type":"function"},{"doc":"Type-aware getter for build_state.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#build_state/1","title":"AntikytheraCore.PeriodicLog.Writer.State.build_state/1","type":"function"},{"doc":"Type-aware setter for build_state.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#build_state/2","title":"AntikytheraCore.PeriodicLog.Writer.State.build_state/2","type":"function"},{"doc":"Type-aware getter for log_state.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#log_state/1","title":"AntikytheraCore.PeriodicLog.Writer.State.log_state/1","type":"function"},{"doc":"Type-aware setter for log_state.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#log_state/2","title":"AntikytheraCore.PeriodicLog.Writer.State.log_state/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.PeriodicLog.Writer.State by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#new/1","title":"AntikytheraCore.PeriodicLog.Writer.State.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#new!/1","title":"AntikytheraCore.PeriodicLog.Writer.State.new!/1","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.PeriodicLog.Writer.State with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#update/2","title":"AntikytheraCore.PeriodicLog.Writer.State.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#update!/2","title":"AntikytheraCore.PeriodicLog.Writer.State.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#valid?/1","title":"AntikytheraCore.PeriodicLog.Writer.State.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.PeriodicLog.Writer.State.html#t:t/0","title":"AntikytheraCore.PeriodicLog.Writer.State.t/0","type":"type"},{"doc":"Generates a .appup file for the given application, start version, and upgrade version.\n\nThe appup file generated by `make/5` is much simpler than that by similar appup generation tools,\nas `make/5` does not use `update` instruction (which is called \"synchronized code replacement\").\nSynchronized code replacement (which uses `code_change/3`) is risky because\n\n- It's difficult to get it right, difficult to test.\n- Suspending target processes entails traversing the whole supervision tree.\n  All processes in the tree must respond to system messages in a timely manner.\n    - Processes governed by some libraries might not properly handle system messages;\n      if it's the case all OTP applications will be restarted by `:init.restart()`.\n    - Some process might not respond in a timely fashion; resulting in brutal kill of the unresponsive process.\n    - Suspending handling of non-system messages might cause performance issues.\n\nTherefore we simply don't use OTP's code change mechanism;\nto change data format of process state we\n- implement callbacks to check/convert the data format version, or\n- avoid hot code upgrade and replace the working instances with new ones.\n\nOn the other hand, `make/5` automatically resolves interdependencies between changed modules (which other tools don't).","ref":"AntikytheraCore.Release.Appup.html","title":"AntikytheraCore.Release.Appup","type":"module"},{"doc":"- `name`  : the application name as an atom\n- `v1`    : the start version, such as \"0.0.1\"\n- `v2`    : the upgrade version, such as \"0.0.2\"\n- `v1_dir`: the path to the v1 artifacts (rel/ /lib/ -0.0.1)\n- `v2_dir`: the path to the v2 artifacts (_build/prod/lib/ )","ref":"AntikytheraCore.Release.Appup.html#module-parameters-for-make-5","title":"Parameters for `make/5` - AntikytheraCore.Release.Appup","type":"module"},{"doc":"","ref":"AntikytheraCore.Release.Appup.html#make/5","title":"AntikytheraCore.Release.Appup.make/5","type":"function"},{"doc":"","ref":"AntikytheraCore.Request.html","title":"AntikytheraCore.Request","type":"module"},{"doc":"","ref":"AntikytheraCore.Request.html#make_from_cowboy_req/6","title":"AntikytheraCore.Request.make_from_cowboy_req/6","type":"function"},{"doc":"A `GenServer` which periodically removes already unused artifact files for running gears.\n\n\"Artifact files\" here means files created by unpacking OTP application tarballs of gears.\nSince gears as OTP applications are not included in OTP releases of antikythera instances,\nwe can remove the artifact files directly; no coordination with `:release_handler` is required.\n\nDepends on `AntikytheraCore.GearManager`.","ref":"AntikytheraCore.StaleGearArtifactCleaner.html","title":"AntikytheraCore.StaleGearArtifactCleaner","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.StaleGearArtifactCleaner.html#child_spec/1","title":"AntikytheraCore.StaleGearArtifactCleaner.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.StaleGearArtifactCleaner.html#start_link/1","title":"AntikytheraCore.StaleGearArtifactCleaner.start_link/1","type":"function"},{"doc":"Manages progress of startup procedure of antikythera.\n\nMost of initialization steps are done within `AntikytheraCore.start/2`.\nHowever, the following step is not done in `AntikytheraCore.start/2` and delayed:\n\n- Installing gears:\n  Starting a gear requires that the antikythera instance (as an OTP application) has started;\n  this step is delegated to `AntikytheraCore.VersionSynchronizer`.\n\nThis `GenServer` waits for the above procedures to complete and then changes the cowboy routing rules\nso that the current node can receive web requests from its upstream load balancer.","ref":"AntikytheraCore.StartupManager.html","title":"AntikytheraCore.StartupManager","type":"module"},{"doc":"","ref":"AntikytheraCore.StartupManager.html#all_gears_installed/0","title":"AntikytheraCore.StartupManager.all_gears_installed/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.StartupManager.html#child_spec/1","title":"AntikytheraCore.StartupManager.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.StartupManager.html#initialized?/0","title":"AntikytheraCore.StartupManager.initialized?/0","type":"function"},{"doc":"","ref":"AntikytheraCore.StartupManager.html#start_link/1","title":"AntikytheraCore.StartupManager.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.StartupManager.html#update_routing/1","title":"AntikytheraCore.StartupManager.update_routing/1","type":"function"},{"doc":"A `GenServer` that periodically submit metrics about the ErlangVM.\n\nNote that some values obtained from `:erlang.statistics/1` are cumulative; we have to take diff from the previous values.\n\n- reductions\n- bytes_in\n- bytes_out\n- gc_count\n- gc_words_reclaimed","ref":"AntikytheraCore.SystemMetricsReporter.html","title":"AntikytheraCore.SystemMetricsReporter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.SystemMetricsReporter.html#child_spec/1","title":"AntikytheraCore.SystemMetricsReporter.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.SystemMetricsReporter.html#start_link/1","title":"AntikytheraCore.SystemMetricsReporter.start_link/1","type":"function"},{"doc":"This is an implementation of `EEx.Engine` that auto-escape dynamic parts within HAML templates.","ref":"AntikytheraCore.TemplateEngine.html","title":"AntikytheraCore.TemplateEngine","type":"module"},{"doc":"","ref":"AntikytheraCore.TemplateEngine.html#to_safe_iodata/1","title":"AntikytheraCore.TemplateEngine.to_safe_iodata/1","type":"function"},{"doc":"A `GenServer` that caches tenant settings and apply changes to tenant executor pools.\n\nOriginal information of tenant settings is (typically) stored in a slow file storage.\nAs reading many files may take long, such kind of file operations are done by one-off processes.\n\nNote that `init/1` callback fetches all tenant settings and applies them in a blocking manner.\n\nNote also that there are two sources of tenant setting information that this GenServer receives:\n\n1. periodic scanning of files\n2. notification by `GenServer.call(server, {:apply, tenant_id, tenant_setting})` (may be called using `multi_call`)\n\nInformation given by the two sources may have conflicts, i.e. results from (1) may not include the latest info given by (2).\nIn order not to kill working tenant executor pools, we don't immediately remove/disassociate executor pool\nthat is not included in (1); those operations are delayed to the next `:check`.","ref":"AntikytheraCore.TenantExecutorPoolsManager.html","title":"AntikytheraCore.TenantExecutorPoolsManager","type":"module"},{"doc":"","ref":"AntikytheraCore.TenantExecutorPoolsManager.html#all/0","title":"AntikytheraCore.TenantExecutorPoolsManager.all/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.TenantExecutorPoolsManager.html#child_spec/1","title":"AntikytheraCore.TenantExecutorPoolsManager.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TenantExecutorPoolsManager.html#polling_interval/0","title":"AntikytheraCore.TenantExecutorPoolsManager.polling_interval/0","type":"function"},{"doc":"","ref":"AntikytheraCore.TenantExecutorPoolsManager.html#start_link/1","title":"AntikytheraCore.TenantExecutorPoolsManager.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TenantExecutorPoolsManager.OneOffFetcher.html","title":"AntikytheraCore.TenantExecutorPoolsManager.OneOffFetcher","type":"module"},{"doc":"","ref":"AntikytheraCore.TenantExecutorPoolsManager.OneOffFetcher.html#run/1","title":"AntikytheraCore.TenantExecutorPoolsManager.OneOffFetcher.run/1","type":"function"},{"doc":"A GenServer that periodically checks the status of `Node.self/0`.\n\nWhen this `GenServer` notices that this host is going to be terminated, it prepares for termination:\n\n- deactivate `RaftFleet` so that all consensus members in this node will be migrated to other active nodes\n- deactivate `AntikytheraCore.ExecutorPool.AsyncJobBroker`s so that they don't start new async jobs\n\nDepends on `AntikytheraCore.ClusterHostsPoller`.","ref":"AntikytheraCore.TerminationManager.html","title":"AntikytheraCore.TerminationManager","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.TerminationManager.html#child_spec/1","title":"AntikytheraCore.TerminationManager.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TerminationManager.html#register_broker/0","title":"AntikytheraCore.TerminationManager.register_broker/0","type":"function"},{"doc":"","ref":"AntikytheraCore.TerminationManager.html#start_link/1","title":"AntikytheraCore.TerminationManager.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TerminationManager.State.html","title":"AntikytheraCore.TerminationManager.State","type":"module"},{"doc":"Type-aware getter for brokers.","ref":"AntikytheraCore.TerminationManager.State.html#brokers/1","title":"AntikytheraCore.TerminationManager.State.brokers/1","type":"function"},{"doc":"Type-aware setter for brokers.","ref":"AntikytheraCore.TerminationManager.State.html#brokers/2","title":"AntikytheraCore.TerminationManager.State.brokers/2","type":"function"},{"doc":"Type-aware getter for in_service?.","ref":"AntikytheraCore.TerminationManager.State.html#in_service?/1","title":"AntikytheraCore.TerminationManager.State.in_service?/1","type":"function"},{"doc":"Type-aware setter for in_service?.","ref":"AntikytheraCore.TerminationManager.State.html#in_service?/2","title":"AntikytheraCore.TerminationManager.State.in_service?/2","type":"function"},{"doc":"Type-aware getter for log_flushed?.","ref":"AntikytheraCore.TerminationManager.State.html#log_flushed?/1","title":"AntikytheraCore.TerminationManager.State.log_flushed?/1","type":"function"},{"doc":"Type-aware setter for log_flushed?.","ref":"AntikytheraCore.TerminationManager.State.html#log_flushed?/2","title":"AntikytheraCore.TerminationManager.State.log_flushed?/2","type":"function"},{"doc":"","ref":"AntikytheraCore.TerminationManager.State.html#new/0","title":"AntikytheraCore.TerminationManager.State.new/0","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.TerminationManager.State by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.TerminationManager.State.html#new/1","title":"AntikytheraCore.TerminationManager.State.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.TerminationManager.State.html#new!/1","title":"AntikytheraCore.TerminationManager.State.new!/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TerminationManager.State.html#next/2","title":"AntikytheraCore.TerminationManager.State.next/2","type":"function"},{"doc":"Type-aware getter for not_in_service_count.","ref":"AntikytheraCore.TerminationManager.State.html#not_in_service_count/1","title":"AntikytheraCore.TerminationManager.State.not_in_service_count/1","type":"function"},{"doc":"Type-aware setter for not_in_service_count.","ref":"AntikytheraCore.TerminationManager.State.html#not_in_service_count/2","title":"AntikytheraCore.TerminationManager.State.not_in_service_count/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.TerminationManager.State with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.TerminationManager.State.html#update/2","title":"AntikytheraCore.TerminationManager.State.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.TerminationManager.State.html#update!/2","title":"AntikytheraCore.TerminationManager.State.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.TerminationManager.State.html#valid?/1","title":"AntikytheraCore.TerminationManager.State.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TerminationManager.State.html#t:t/0","title":"AntikytheraCore.TerminationManager.State.t/0","type":"type"},{"doc":"A GenServer that keeps track of user pids of temporary directories.\n\nTemporary directories are created via calls to `Antikythera.Tmpdir.make/2`.\nThis GenServer communicates with the caller process and monitors its death to make sure that the directories are eventually deleted.\n\nCurrently we do not impose upper limit on volume and I/O usage\nsince number of concurrently running processes is capped by executor pools.\nAs a result we also skip checking association of \"executor pool\" and \"calling code (gear)\".","ref":"AntikytheraCore.TmpdirTracker.html","title":"AntikytheraCore.TmpdirTracker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.TmpdirTracker.html#child_spec/1","title":"AntikytheraCore.TmpdirTracker.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.html#finished/0","title":"AntikytheraCore.TmpdirTracker.finished/0","type":"function"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.html#get/1","title":"AntikytheraCore.TmpdirTracker.get/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.html#request/1","title":"AntikytheraCore.TmpdirTracker.request/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.html#start_link/1","title":"AntikytheraCore.TmpdirTracker.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.State.html","title":"AntikytheraCore.TmpdirTracker.State","type":"module"},{"doc":"Type-aware getter for gear_tmp_dir.","ref":"AntikytheraCore.TmpdirTracker.State.html#gear_tmp_dir/1","title":"AntikytheraCore.TmpdirTracker.State.gear_tmp_dir/1","type":"function"},{"doc":"Type-aware setter for gear_tmp_dir.","ref":"AntikytheraCore.TmpdirTracker.State.html#gear_tmp_dir/2","title":"AntikytheraCore.TmpdirTracker.State.gear_tmp_dir/2","type":"function"},{"doc":"Type-aware getter for map.","ref":"AntikytheraCore.TmpdirTracker.State.html#map/1","title":"AntikytheraCore.TmpdirTracker.State.map/1","type":"function"},{"doc":"Type-aware setter for map.","ref":"AntikytheraCore.TmpdirTracker.State.html#map/2","title":"AntikytheraCore.TmpdirTracker.State.map/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.TmpdirTracker.State by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.TmpdirTracker.State.html#new/1","title":"AntikytheraCore.TmpdirTracker.State.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.TmpdirTracker.State.html#new!/1","title":"AntikytheraCore.TmpdirTracker.State.new!/1","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.TmpdirTracker.State with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.TmpdirTracker.State.html#update/2","title":"AntikytheraCore.TmpdirTracker.State.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.TmpdirTracker.State.html#update!/2","title":"AntikytheraCore.TmpdirTracker.State.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.TmpdirTracker.State.html#valid?/1","title":"AntikytheraCore.TmpdirTracker.State.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.State.html#t:t/0","title":"AntikytheraCore.TmpdirTracker.State.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.State.Map.html","title":"AntikytheraCore.TmpdirTracker.State.Map","type":"module"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.State.Map.html#valid?/1","title":"AntikytheraCore.TmpdirTracker.State.Map.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraCore.TmpdirTracker.State.Map.html#t:t/0","title":"AntikytheraCore.TmpdirTracker.State.Map.t/0","type":"type"},{"doc":"","ref":"AntikytheraCore.Version.html","title":"AntikytheraCore.Version","type":"module"},{"doc":"","ref":"AntikytheraCore.Version.html#current_version/1","title":"AntikytheraCore.Version.current_version/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.html#read_from_app_file/2","title":"AntikytheraCore.Version.read_from_app_file/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Artifact.html","title":"AntikytheraCore.Version.Artifact","type":"module"},{"doc":"","ref":"AntikytheraCore.Version.Artifact.html#copy_core_release_tgz/1","title":"AntikytheraCore.Version.Artifact.copy_core_release_tgz/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Artifact.html#gears_dir/0","title":"AntikytheraCore.Version.Artifact.gears_dir/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Artifact.html#unpack_gear_tgz/2","title":"AntikytheraCore.Version.Artifact.unpack_gear_tgz/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Core.html","title":"AntikytheraCore.Version.Core","type":"module"},{"doc":"","ref":"AntikytheraCore.Version.Core.html#upgrade_to_next_version/0","title":"AntikytheraCore.Version.Core.upgrade_to_next_version/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Gear.html","title":"AntikytheraCore.Version.Gear","type":"module"},{"doc":"","ref":"AntikytheraCore.Version.Gear.html#gear_dependencies_from_app_file/2","title":"AntikytheraCore.Version.Gear.gear_dependencies_from_app_file/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Gear.html#install_gears_at_startup/1","title":"AntikytheraCore.Version.Gear.install_gears_at_startup/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Gear.html#install_gears_whose_deps_met/4","title":"AntikytheraCore.Version.Gear.install_gears_whose_deps_met/4","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.Gear.html#install_or_upgrade_to_next_version/1","title":"AntikytheraCore.Version.Gear.install_or_upgrade_to_next_version/1","type":"function"},{"doc":"Functions to get information from \"history\" files.\n\nAntikythera uses history files to detect new deployable versions of antikythera instance and gears.\nEach of antikythera instance and gears has its own history file.\nThe files reside in `AntikytheraCore.Path.history_dir/0`.\n\nWhen deploying a new version, after successfully generated a compiled artifact (tarball),\nantikythera's deploy script instructs the ErlangVM nodes by appending a line to the history file of the target OTP application.\nEach line takes either of the following forms:\n\n|                                          | format of line                          | installable    | upgradable    |\n| deploy with hot code upgrade             | ` `                             | true           | true          |\n| deploy with hot code upgrade (canary)    | `  canary= `              | false          | host in hosts |\n| deploy without hot code upgrade          | `  noupgrade`                   | true           | false         |\n| deploy without hot code upgrade (canary) | `  noupgrade_canary= ` | now < deadline | false         |\n\n- \"installable\" means that the version can be installed into a node that doesn't have the OTP application.\n    - See also `script/in_cloud/erl/boot_impl.sh`, which chooses an appropriate version of\n      OTP release of the antikythera instance at startup of a newly created host.\n- \"upgradable\" means that the version can be applied by hot code upgrade to a node that has a previous version of the OTP application.\n\nVersions in a history file are expected to monotonically increase.","ref":"AntikytheraCore.Version.History.html","title":"AntikytheraCore.Version.History","type":"module"},{"doc":"","ref":"AntikytheraCore.Version.History.html#all_deployable_gear_names/0","title":"AntikytheraCore.Version.History.all_deployable_gear_names/0","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.html#find_all_modified_history_files/1","title":"AntikytheraCore.Version.History.find_all_modified_history_files/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.html#latest_installable_gear_version/1","title":"AntikytheraCore.Version.History.latest_installable_gear_version/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.html#next_upgradable_version/2","title":"AntikytheraCore.Version.History.next_upgradable_version/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.Entry.html","title":"AntikytheraCore.Version.History.Entry","type":"module"},{"doc":"","ref":"AntikytheraCore.Version.History.Entry.html#canary?/1","title":"AntikytheraCore.Version.History.Entry.canary?/1","type":"function"},{"doc":"Type-aware getter for canary_target_hosts.","ref":"AntikytheraCore.Version.History.Entry.html#canary_target_hosts/1","title":"AntikytheraCore.Version.History.Entry.canary_target_hosts/1","type":"function"},{"doc":"Type-aware setter for canary_target_hosts.","ref":"AntikytheraCore.Version.History.Entry.html#canary_target_hosts/2","title":"AntikytheraCore.Version.History.Entry.canary_target_hosts/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.Entry.html#from_line/1","title":"AntikytheraCore.Version.History.Entry.from_line/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.Entry.html#installable?/1","title":"AntikytheraCore.Version.History.Entry.installable?/1","type":"function"},{"doc":"Type-aware getter for installable_until.","ref":"AntikytheraCore.Version.History.Entry.html#installable_until/1","title":"AntikytheraCore.Version.History.Entry.installable_until/1","type":"function"},{"doc":"Type-aware setter for installable_until.","ref":"AntikytheraCore.Version.History.Entry.html#installable_until/2","title":"AntikytheraCore.Version.History.Entry.installable_until/2","type":"function"},{"doc":"Creates a new instance of AntikytheraCore.Version.History.Entry by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraCore.Version.History.Entry.html#new/1","title":"AntikytheraCore.Version.History.Entry.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Version.History.Entry.html#new!/1","title":"AntikytheraCore.Version.History.Entry.new!/1","type":"function"},{"doc":"Type-aware getter for noupgrade.","ref":"AntikytheraCore.Version.History.Entry.html#noupgrade/1","title":"AntikytheraCore.Version.History.Entry.noupgrade/1","type":"function"},{"doc":"Type-aware setter for noupgrade.","ref":"AntikytheraCore.Version.History.Entry.html#noupgrade/2","title":"AntikytheraCore.Version.History.Entry.noupgrade/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraCore.Version.History.Entry with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraCore.Version.History.Entry.html#update/2","title":"AntikytheraCore.Version.History.Entry.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraCore.Version.History.Entry.html#update!/2","title":"AntikytheraCore.Version.History.Entry.update!/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.Entry.html#upgradable?/1","title":"AntikytheraCore.Version.History.Entry.upgradable?/1","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraCore.Version.History.Entry.html#valid?/1","title":"AntikytheraCore.Version.History.Entry.valid?/1","type":"function"},{"doc":"Type-aware getter for version.","ref":"AntikytheraCore.Version.History.Entry.html#version/1","title":"AntikytheraCore.Version.History.Entry.version/1","type":"function"},{"doc":"Type-aware setter for version.","ref":"AntikytheraCore.Version.History.Entry.html#version/2","title":"AntikytheraCore.Version.History.Entry.version/2","type":"function"},{"doc":"","ref":"AntikytheraCore.Version.History.Entry.html#t:t/0","title":"AntikytheraCore.Version.History.Entry.t/0","type":"type"},{"doc":"A `GenServer` which periodically polls deployable versions of antikythera instance and gears.\n\nOn startup of antikythera, this `GenServer` waits for completion of `AntikytheraCore.start/2`,\ninstalls all available gears, notifies to `StartupManager`, and then starts periodic polling.\nDepends on `StartupManager` and `VersionUpgradeTaskQueue`.","ref":"AntikytheraCore.VersionSynchronizer.html","title":"AntikytheraCore.VersionSynchronizer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.VersionSynchronizer.html#child_spec/1","title":"AntikytheraCore.VersionSynchronizer.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.VersionSynchronizer.html#start_link/1","title":"AntikytheraCore.VersionSynchronizer.start_link/1","type":"function"},{"doc":"A `GenServer` to serialize tasks to \"upgrade of antikythera instance\" and \"installation or upgrade of gears\".\n\nThis is implemented as a `GenServer` for the purpose of preventing multiple install/upgrade from running concurrently.\nAs each installation/upgrade can take much longer than 5-seconds timeout of system messages,\nthe actual task is delegated to one-off processes.","ref":"AntikytheraCore.VersionUpgradeTaskQueue.html","title":"AntikytheraCore.VersionUpgradeTaskQueue","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AntikytheraCore.VersionUpgradeTaskQueue.html#child_spec/1","title":"AntikytheraCore.VersionUpgradeTaskQueue.child_spec/1","type":"function"},{"doc":"","ref":"AntikytheraCore.VersionUpgradeTaskQueue.html#core_updated/0","title":"AntikytheraCore.VersionUpgradeTaskQueue.core_updated/0","type":"function"},{"doc":"","ref":"AntikytheraCore.VersionUpgradeTaskQueue.html#disable/0","title":"AntikytheraCore.VersionUpgradeTaskQueue.disable/0","type":"function"},{"doc":"","ref":"AntikytheraCore.VersionUpgradeTaskQueue.html#enable/0","title":"AntikytheraCore.VersionUpgradeTaskQueue.enable/0","type":"function"},{"doc":"","ref":"AntikytheraCore.VersionUpgradeTaskQueue.html#gear_updated/1","title":"AntikytheraCore.VersionUpgradeTaskQueue.gear_updated/1","type":"function"},{"doc":"","ref":"AntikytheraCore.VersionUpgradeTaskQueue.html#start_link/1","title":"AntikytheraCore.VersionUpgradeTaskQueue.start_link/1","type":"function"},{"doc":"","ref":"AntikytheraCore.Vm.html","title":"AntikytheraCore.Vm","type":"module"},{"doc":"","ref":"AntikytheraCore.Vm.html#count_messages_in_all_mailboxes/0","title":"AntikytheraCore.Vm.count_messages_in_all_mailboxes/0","type":"function"},{"doc":"Environment abstraction layer: pluggable interfaces defined in antikythera.\n\n(This module exists solely for documentation purpose and thus contains only this `@moduledoc`.)\n\nSome of antikythera's features are pluggable so that one can use environment-specific implementations.\nFor example, in order to maintain cluster membership, list of currently running ErlangVM nodes\nmay be obtained from the IaaS platform API.\n\nAdministrators of an antikythera instance must provide the set of concrete callback modules\nsuitable for their environments.\nThe callback modules to be used by antikythera must be specified as `:eal_impl_modules` key in mix configs.\nNote that `:eal_impl_modules` are resolved at compile time.\nThis means:\n\n- compiling antikythera requires the callback modules for all behaviours, and\n- to change any of the callback modules you have to recompile antikythera.","ref":"AntikytheraEal.html","title":"AntikytheraEal","type":"module"},{"doc":"All pluggable interfaces are defined as behaviour modules and named as\n`AntikytheraEal.SomeFeature.Behaviour` (e.g. `AntikytheraEal.ClusterConfiguration.Behaviour`).\nEach behaviour comes with a concrete callback module which is used for tests of antikythera\n(e.g. `AntikytheraEal.ClusterConfiguration.StandAlone`).\nThe parent module (e.g. `AntikytheraEal.ClusterConfiguration`) works as a proxy module\nwhen invoking the callback functions of the module specified in mix config\n(see also `AntikytheraEal.ImplChooser`).","ref":"AntikytheraEal.html#module-module-naming-scheme-for-each-interface","title":"Module naming scheme for each interface - AntikytheraEal","type":"module"},{"doc":"","ref":"AntikytheraEal.AlertMailer.html","title":"AntikytheraEal.AlertMailer","type":"module"},{"doc":"Interface to email delivery backend for core/gear alerts.\n\nSee `AntikytheraEal` for common information about pluggable interfaces defined in antikythera.","ref":"AntikytheraEal.AlertMailer.Behaviour.html","title":"AntikytheraEal.AlertMailer.Behaviour","type":"behaviour"},{"doc":"Sends an alert email.\n\nCC/BCC and HTML mail is omitted for simplicity.\n\nThis callback is called in `AntikytheraCore.Alert.Handler.Email`, that is,\nwhen an alert email about error(s) in either antikythera core or gear is sent.","ref":"AntikytheraEal.AlertMailer.Behaviour.html#c:deliver/1","title":"AntikytheraEal.AlertMailer.Behaviour.deliver/1","type":"callback"},{"doc":"","ref":"AntikytheraEal.AlertMailer.Mail.html","title":"AntikytheraEal.AlertMailer.Mail","type":"module"},{"doc":"Type-aware getter for body.","ref":"AntikytheraEal.AlertMailer.Mail.html#body/1","title":"AntikytheraEal.AlertMailer.Mail.body/1","type":"function"},{"doc":"Type-aware setter for body.","ref":"AntikytheraEal.AlertMailer.Mail.html#body/2","title":"AntikytheraEal.AlertMailer.Mail.body/2","type":"function"},{"doc":"Type-aware getter for from.","ref":"AntikytheraEal.AlertMailer.Mail.html#from/1","title":"AntikytheraEal.AlertMailer.Mail.from/1","type":"function"},{"doc":"Type-aware setter for from.","ref":"AntikytheraEal.AlertMailer.Mail.html#from/2","title":"AntikytheraEal.AlertMailer.Mail.from/2","type":"function"},{"doc":"Creates a new instance of AntikytheraEal.AlertMailer.Mail by using the given `dict`.\n\nReturns `{:ok, valid_struct}` or `{:error, reason}`.\n\nThe values in the `dict` are validated by each field's `valid?/1` function.\nIf the value was invalid, it will be passed to `new/1` of the field\n\nFor missing fields, followings will be tried:\n- `default/0` of each field type\n- `new/1` of each field type, with empty map as input","ref":"AntikytheraEal.AlertMailer.Mail.html#new/1","title":"AntikytheraEal.AlertMailer.Mail.new/1","type":"function"},{"doc":"A variant of `new/1` which returns `t` or raise if validation fails.\n\nIn other words, `new/1` followed by `Croma.Result.get!/1`.","ref":"AntikytheraEal.AlertMailer.Mail.html#new!/1","title":"AntikytheraEal.AlertMailer.Mail.new!/1","type":"function"},{"doc":"Type-aware getter for subject.","ref":"AntikytheraEal.AlertMailer.Mail.html#subject/1","title":"AntikytheraEal.AlertMailer.Mail.subject/1","type":"function"},{"doc":"Type-aware setter for subject.","ref":"AntikytheraEal.AlertMailer.Mail.html#subject/2","title":"AntikytheraEal.AlertMailer.Mail.subject/2","type":"function"},{"doc":"Type-aware getter for to.","ref":"AntikytheraEal.AlertMailer.Mail.html#to/1","title":"AntikytheraEal.AlertMailer.Mail.to/1","type":"function"},{"doc":"Type-aware setter for to.","ref":"AntikytheraEal.AlertMailer.Mail.html#to/2","title":"AntikytheraEal.AlertMailer.Mail.to/2","type":"function"},{"doc":"Updates an existing instance of AntikytheraEal.AlertMailer.Mail with the given `dict`.\nThe values in the `dict` are validated by each field's `valid?/1` function.\nReturns `{:ok, valid_struct}` or `{:error, reason}`.","ref":"AntikytheraEal.AlertMailer.Mail.html#update/2","title":"AntikytheraEal.AlertMailer.Mail.update/2","type":"function"},{"doc":"A variant of `update/2` which returns `t` or raise if validation fails.\nIn other words, `update/2` followed by `Croma.Result.get!/1`.","ref":"AntikytheraEal.AlertMailer.Mail.html#update!/2","title":"AntikytheraEal.AlertMailer.Mail.update!/2","type":"function"},{"doc":"Checks if the given value belongs to `t:t/0` or not.","ref":"AntikytheraEal.AlertMailer.Mail.html#valid?/1","title":"AntikytheraEal.AlertMailer.Mail.valid?/1","type":"function"},{"doc":"","ref":"AntikytheraEal.AlertMailer.Mail.html#t:t/0","title":"AntikytheraEal.AlertMailer.Mail.t/0","type":"type"},{"doc":"","ref":"AntikytheraEal.AlertMailer.MemoryInbox.html","title":"AntikytheraEal.AlertMailer.MemoryInbox","type":"module"},{"doc":"","ref":"AntikytheraEal.AlertMailer.MemoryInbox.html#clean/0","title":"AntikytheraEal.AlertMailer.MemoryInbox.clean/0","type":"function"},{"doc":"","ref":"AntikytheraEal.AlertMailer.MemoryInbox.html#get/0","title":"AntikytheraEal.AlertMailer.MemoryInbox.get/0","type":"function"},{"doc":"","ref":"AntikytheraEal.AssetStorage.html","title":"AntikytheraEal.AssetStorage","type":"module"},{"doc":"Interface to work with storage of asset files.\n\nSee `AntikytheraEal` for common information about pluggable interfaces defined in antikythera.\n\nAll callbacks defined in this behaviour are called only within operational mix tasks;\nthey are not used at runtime.\n\nAsset files stored in the storage may be delivered via CDN.\nIt's the callback module's responsibility to properly set headers such as `cache-control` for each asset file.","ref":"AntikytheraEal.AssetStorage.Behaviour.html","title":"AntikytheraEal.AssetStorage.Behaviour","type":"behaviour"},{"doc":"Deletes an asset file identified by the given `key`.","ref":"AntikytheraEal.AssetStorage.Behaviour.html#c:delete/1","title":"AntikytheraEal.AssetStorage.Behaviour.delete/1","type":"callback"},{"doc":"Lists all already-stored asset files for the specified gear.","ref":"AntikytheraEal.AssetStorage.Behaviour.html#c:list/1","title":"AntikytheraEal.AssetStorage.Behaviour.list/1","type":"callback"},{"doc":"Lists common key prefixes of all already-stored asset files for all gears.\n\nBased on the key format defined in `Antikythera.Asset`,\nthe prefixes are expected to be the list of gear names which have already-stored asset files.","ref":"AntikytheraEal.AssetStorage.Behaviour.html#c:list_toplevel_prefixes/0","title":"AntikytheraEal.AssetStorage.Behaviour.list_toplevel_prefixes/0","type":"callback"},{"doc":"Uploads the given asset file to the storage.\n\nParameters:\n\n- `path` : File path of the asset file to upload.\n- `key`  : Upload location (typically path part of URL) for the asset file (computed by `Antikythera.Asset`).\n- `mime` : MIME type of the asset file.\n- `gzip?`: Whether gzip compression is beneficial for the asset file.","ref":"AntikytheraEal.AssetStorage.Behaviour.html#c:upload/4","title":"AntikytheraEal.AssetStorage.Behaviour.upload/4","type":"callback"},{"doc":"","ref":"AntikytheraEal.AssetStorage.NoOp.html","title":"AntikytheraEal.AssetStorage.NoOp","type":"module"},{"doc":"","ref":"AntikytheraEal.ClusterConfiguration.html","title":"AntikytheraEal.ClusterConfiguration","type":"module"},{"doc":"Interface for cluster membership.\n\nSee `AntikytheraEal` for common information about pluggable interfaces defined in antikythera.","ref":"AntikytheraEal.ClusterConfiguration.Behaviour.html","title":"AntikytheraEal.ClusterConfiguration.Behaviour","type":"behaviour"},{"doc":"Returns the number of seconds for which the initial health check to this host is delayed to\nprovide ample warm-up time for the host.\n\nThis callback is called only at startup and the return value is used to calculate the trial count for\nestablishing connections to other nodes.","ref":"AntikytheraEal.ClusterConfiguration.Behaviour.html#c:health_check_grace_period_in_seconds/0","title":"AntikytheraEal.ClusterConfiguration.Behaviour.health_check_grace_period_in_seconds/0","type":"callback"},{"doc":"Lists running ErlangVM nodes for the current deployment.\n\nOn success this callback must return a map where\neach key is hostname (`String.t`) of a running node and\neach value is a boolean value representing whether the node is \"active\" or not.\n\"Inactive\" node is treated as during decommission process.\n\nThis callback is at the startup of each node in order to join the existing cluster.\nAfter successful startup, this callback is called periodically so that\n\n- all running nodes in the cluster maintain connections to each other, and\n- each node can notice and cleanup when it is about to be terminated.","ref":"AntikytheraEal.ClusterConfiguration.Behaviour.html#c:running_hosts/0","title":"AntikytheraEal.ClusterConfiguration.Behaviour.running_hosts/0","type":"callback"},{"doc":"Returns identifier of the data center zone in which current node is running.\n\nThis callback is called only at startup and the return value is passed to `RaftFleet.activate/1`.","ref":"AntikytheraEal.ClusterConfiguration.Behaviour.html#c:zone_of_this_host/0","title":"AntikytheraEal.ClusterConfiguration.Behaviour.zone_of_this_host/0","type":"callback"},{"doc":"","ref":"AntikytheraEal.ClusterConfiguration.StandAlone.html","title":"AntikytheraEal.ClusterConfiguration.StandAlone","type":"module"},{"doc":"","ref":"AntikytheraEal.ImplChooser.html","title":"AntikytheraEal.ImplChooser","type":"module"},{"doc":"","ref":"AntikytheraEal.ImplChooser.html#extract_impl_module/1","title":"AntikytheraEal.ImplChooser.extract_impl_module/1","type":"function"},{"doc":"","ref":"AntikytheraEal.ImplChooser.html#make_vars/2","title":"AntikytheraEal.ImplChooser.make_vars/2","type":"function"},{"doc":"","ref":"AntikytheraEal.LogStorage.html","title":"AntikytheraEal.LogStorage","type":"module"},{"doc":"Interface to work with storage of gear log files.\n\nSee `AntikytheraEal` for common information about pluggable interfaces defined in antikythera.\n\nGear's log messages are written to a file in local filesystem (by `AntikytheraCore.GearLog.Writer`).\nLog files are then rotated after certain times and then uploaded to log storage.\n\nTypically rotated log files are uploaded by a script that runs outside of antikythera,\nin order to correctly upload logs even after ErlangVM crashes.\n`upload_rotated_logs/1` can also be used to upload logs to the storage, but this callback is\nbasically intended for on-demand (i.e. gear administrator initiated) uploads.","ref":"AntikytheraEal.LogStorage.Behaviour.html","title":"AntikytheraEal.LogStorage.Behaviour","type":"behaviour"},{"doc":"Generates download URLs of multiple log files specified by list of keys.\n\nKeys can be obtained by `list/2`.","ref":"AntikytheraEal.LogStorage.Behaviour.html#c:download_urls/1","title":"AntikytheraEal.LogStorage.Behaviour.download_urls/1","type":"callback"},{"doc":"Lists already-uploaded log files, whose last log messages are generated on the specified date, of a gear.\n\n`date_str` is of the form of `\"20180101\"`.\n\nReturns list of pairs, where each pair consists of\n\n- key which identifies the log file\n- size of the log file","ref":"AntikytheraEal.LogStorage.Behaviour.html#c:list/2","title":"AntikytheraEal.LogStorage.Behaviour.list/2","type":"callback"},{"doc":"Upload all log files which have already been rotated but not yet uploaded.","ref":"AntikytheraEal.LogStorage.Behaviour.html#c:upload_rotated_logs/1","title":"AntikytheraEal.LogStorage.Behaviour.upload_rotated_logs/1","type":"callback"},{"doc":"","ref":"AntikytheraEal.LogStorage.FileSystem.html","title":"AntikytheraEal.LogStorage.FileSystem","type":"module"},{"doc":"","ref":"AntikytheraEal.MetricsStorage.html","title":"AntikytheraEal.MetricsStorage","type":"module"},{"doc":"","ref":"AntikytheraEal.MetricsStorage.html#save/2","title":"AntikytheraEal.MetricsStorage.save/2","type":"function"},{"doc":"","ref":"AntikytheraEal.MetricsStorage.html#t:metrics_per_unit/0","title":"AntikytheraEal.MetricsStorage.metrics_per_unit/0","type":"type"},{"doc":"Interface to storage for metrics data.\n\nSee `AntikytheraEal` for common information about pluggable interfaces defined in antikythera.\n\nMetrics data generated during runtime is buffered for a while by `AntikytheraCore.MetricsUploader`\nprocesses and then transferred to metrics storage using `upload/3` callback.","ref":"AntikytheraEal.MetricsStorage.Behaviour.html","title":"AntikytheraEal.MetricsStorage.Behaviour","type":"behaviour"},{"doc":"Uploads the metrics data to the metrics storage.\n\n`otp_app_name` is either a gear name or `:antikythera`.\n`node_id` is the name of the current node and callback implementation may include `node_id`\nin the body of the upload.","ref":"AntikytheraEal.MetricsStorage.Behaviour.html#c:upload/3","title":"AntikytheraEal.MetricsStorage.Behaviour.upload/3","type":"callback"},{"doc":"","ref":"AntikytheraEal.MetricsStorage.Memory.html","title":"AntikytheraEal.MetricsStorage.Memory","type":"module"},{"doc":"Retrieve metrics in memory for testing.","ref":"AntikytheraEal.MetricsStorage.Memory.html#download/4","title":"AntikytheraEal.MetricsStorage.Memory.download/4","type":"function"},{"doc":"","ref":"AntikytheraLocal.Cmd.html","title":"AntikytheraLocal.Cmd","type":"module"},{"doc":"","ref":"AntikytheraLocal.Cmd.html#exec_and_output_log!/3","title":"AntikytheraLocal.Cmd.exec_and_output_log!/3","type":"function"},{"doc":"","ref":"AntikytheraLocal.NodeName.html","title":"AntikytheraLocal.NodeName","type":"module"},{"doc":"","ref":"AntikytheraLocal.NodeName.html#get/0","title":"AntikytheraLocal.NodeName.get/0","type":"function"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html","title":"AntikytheraLocal.RunningEnvironment","type":"module"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html#currently_running_os_process_ids/0","title":"AntikytheraLocal.RunningEnvironment.currently_running_os_process_ids/0","type":"function"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html#prepare_new_version_of_core/1","title":"AntikytheraLocal.RunningEnvironment.prepare_new_version_of_core/1","type":"function"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html#prepare_new_version_of_gear/2","title":"AntikytheraLocal.RunningEnvironment.prepare_new_version_of_gear/2","type":"function"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html#setup/1","title":"AntikytheraLocal.RunningEnvironment.setup/1","type":"function"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html#teardown/0","title":"AntikytheraLocal.RunningEnvironment.teardown/0","type":"function"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html#unpacked_gears_dir/0","title":"AntikytheraLocal.RunningEnvironment.unpacked_gears_dir/0","type":"function"},{"doc":"","ref":"AntikytheraLocal.RunningEnvironment.html#wait_until_upgrade_applied/2","title":"AntikytheraLocal.RunningEnvironment.wait_until_upgrade_applied/2","type":"function"},{"doc":"","ref":"AntikytheraLocal.StartScript.html","title":"AntikytheraLocal.StartScript","type":"module"},{"doc":"","ref":"AntikytheraLocal.StartScript.html#run/2","title":"AntikytheraLocal.StartScript.run/2","type":"function"},{"doc":"Generates a mix project for a new gear application.","ref":"Mix.Tasks.Antikythera.Gear.New.html","title":"mix antikythera.gear.new","type":"task"},{"doc":"This mix task must be run from a mix project directory of either\nyour antikythera instance or one of gears in your antikythera instance.\n\n    mix antikythera.gear.new PATH\n\nBasename of `PATH` becomes the name of your gear.\nFor example, if `~/workspace/my_gear` is used, `my_gear` becomes your gear name.\n\nNewly generated gear automatically belongs to an antikythera instance\ninferred from the mix project you are currently in.\n\nYou can also manually specify your antikythera instance via `ANTIKYTHERA_INSTANCE_DEP` environment variable like so:\n\n    ANTIKYTHERA_INSTANCE_DEP='{:instance_name, [git: \"git@github.com:your-organization/instance_name.git\"]}' mix antikythera.gear.new PATH\n\n`ANTIKYTHERA_INSTANCE_DEP` must be a valid dependency tuple used in `mix.exs` files.\nSee [`mix deps` doc](https://hexdocs.pm/mix/Mix.Tasks.Deps.html) for available options.","ref":"Mix.Tasks.Antikythera.Gear.New.html#module-usage","title":"Usage - mix antikythera.gear.new","type":"task"},{"doc":"In order to sync development environment configurations with your antikythera instance and other gears,\nthis generator creates symbolic links to these files, if they exist in the current directory:\n\n- `.credo.exs`\n    - Configuration file for static code analysis tool [Credo](https://github.com/rrrene/credo)\n- `.tool-versions`\n    - Local versions file for version manager tool [asdf](https://github.com/asdf-vm/asdf)","ref":"Mix.Tasks.Antikythera.Gear.New.html#module-symbolic-links-to-files-in-your-antikythera-instance","title":"Symbolic links to files in your antikythera instance - mix antikythera.gear.new","type":"task"},{"doc":"","ref":"Mix.Tasks.Antikythera.Gear.New.html#run/1","title":"Mix.Tasks.Antikythera.Gear.New.run/1","type":"function"},{"doc":"Prepares static assets for your gear.\n\nThis mix task must be called before compilations of the gear if you need asset preparation.\nIt ensures all static assets of your gear reside in `priv/static/` directory before compilations of `YourGear.Asset` module.\nAssets in `priv/static/` directory must be uploaded to cloud storage by `mix antikythera_core.upload_new_asset_versions` for serving via CDN if you run the gear on cloud.\nSee `Antikythera.Asset` and `Mix.Tasks.AntikytheraCore.UploadNewAssetVersions` for details.\n\nIf your gear uses some kind of preprocessing tools to generate asset files (JS, CSS, etc.),\nyou have to set up the supported asset preparation method described in the next section.\n\nThis mix task invokes the preprocessing tools with `ANTIKYTHERA_COMPILE_ENV` environment variable\n(see also `Antikythera.Env`).\nYou can use this environment variable to distinguish for which environment current script is running.","ref":"Mix.Tasks.Antikythera.PrepareAssets.html","title":"mix antikythera.prepare_assets","type":"task"},{"doc":"Asset preparation process is split into 3 steps: package installation step, auditing step and build step.\nIf any part of the preparation process resulted in failure (non-zero exit code),\nthe whole task will abort and thus auto-deploy will fail.\n\n- Prerequisite: `package.json` file and `antikythera_prepare_assets` script in it\n\nNote that if the above prerequisite is not present,\n**the whole asset preparation process will be skipped** since package installation is unnecessary.\n\n#","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#module-supported-asset-preparation-method","title":"Supported Asset Preparation Method - mix antikythera.prepare_assets","type":"task"},{"doc":"Use [`yarn`](https://yarnpkg.com/en/) if `yarn.lock` file exists, otherwise use `npm install`.\n\n#","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#module-package-installation","title":"Package Installation - mix antikythera.prepare_assets","type":"task"},{"doc":"Run `yarn audit` or `npm audit` and abort if one or more critical packages found.\n\n#","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#module-auditing-the-packages","title":"Auditing the packages - mix antikythera.prepare_assets","type":"task"},{"doc":"- Command: `npm run antikythera_prepare_assets`\n- Within `antikythera_prepare_assets` script, you may execute any asset-related actions such as:\n    - Linting\n    - Type Checking\n    - Testing\n    - Compiling/Transpiling\n    - Uglifying/Minifying\n    - etc...\n- How to organize these actions is up to you. You may use whatever tools available in `npm`,\n  such as [`webpack`](https://webpack.js.org/) or [`browserify`](http://browserify.org/).\n\n##","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#module-build-using-npm-scripts-https-docs-npmjs-com-misc-scripts","title":"Build using [npm-scripts](https://docs.npmjs.com/misc/scripts) - mix antikythera.prepare_assets","type":"task"},{"doc":"This step is responsible for placing finalized static assets into `priv/static/` directory.\n\nIn old versions of `gear_generator`, Node.js packages that depend on\n[`compass`](http://compass-style.org/) was used by default in generated gear.\n`compass` gem was (and is) globally installed on antikythera jenkins server to enable them.\n\nHowever, [`compass` is no longer maintained](https://github.com/Compass/compass/commit/dd74a1cfef478a896d03152b2c2a3b93839d168e).\nWe recommend you to consider using alternative SASS/SCSS processor.\n`compass` gem will be kept installed for backward compatibility.","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#module-note-on-implementation","title":"Note on implementation - mix antikythera.prepare_assets","type":"task"},{"doc":"","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#dump_asset_file_paths/0","title":"Mix.Tasks.Antikythera.PrepareAssets.dump_asset_file_paths/0","type":"function"},{"doc":"","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#run_command/3","title":"Mix.Tasks.Antikythera.PrepareAssets.run_command/3","type":"function"},{"doc":"","ref":"Mix.Tasks.Antikythera.PrepareAssets.html#run_command!/3","title":"Mix.Tasks.Antikythera.PrepareAssets.run_command!/3","type":"function"},{"doc":"","ref":"Mix.Tasks.Antikythera.PrintVersion.html","title":"mix antikythera.print_version","type":"task"},{"doc":"","ref":"Mix.Tasks.Antikythera.PrintVersion.html#run/1","title":"Mix.Tasks.Antikythera.PrintVersion.run/1","type":"function"},{"doc":"Deletes obsolete asset files in asset storage.\n\nTo judge whether each asset file should be kept or deleted we use \"asset list file\"s.\nEach asset list file is created during deployment of a gear version (see `Mix.Tasks.AntikytheraCore.UploadNewAssetVersions`).\n\nBased on the asset retention policy described in the moduledoc of `Antikythera.Asset`,\nit can be seen that, within all existing asset list files, only the following ones are relevant:\n\n1. asset list files created within the latest 90 days\n2. the latest asset list file among ones created up to 90 days before\n\nAssets included in any of the relevant asset list files should be kept.\nAssets not included in all of the relevant asset list files are \"obsolete\"; they should be deleted.","ref":"Mix.Tasks.AntikytheraCore.DeleteObsoleteAssets.html","title":"mix antikythera_core.delete_obsolete_assets","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraCore.DeleteObsoleteAssets.html#run/1","title":"Mix.Tasks.AntikytheraCore.DeleteObsoleteAssets.run/1","type":"function"},{"doc":"Generates an appup file using the previous version located at the specified directory.\n\nThis task is invoked during gear deployments.","ref":"Mix.Tasks.AntikytheraCore.GenerateAppup.html","title":"mix antikythera_core.generate_appup","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraCore.GenerateAppup.html#run/1","title":"Mix.Tasks.AntikytheraCore.GenerateAppup.run/1","type":"function"},{"doc":"Generates a new release tarball for antikythera instance using `mix release`.\n\nNotes on the implementation details of this task:\n\n- Release generation is basically done under `rel_erlang-*/`.\n  During tests (i.e. when `ANTIKYTHERA_COMPILE_ENV=local`), files are generated under `rel_local_erlang-*/`.\n    - Erlang/OTP major version number is included in the path in order to distinguish artifacts generated by different OTP releases.\n      No binary compatibility is maintained by major version release of Erlang/OTP due to precompilation of Elixir's regex sigils.\n      For more details see [documentation for Regex module](https://hexdocs.pm/elixir/Regex.html#module-precompilation).\n- Generated releases are placed under `rel(_local)_erlang-*/ /releases/`.\n- If no previous releases found, this task generates a new release from scratch (i.e. without relup).\n  If any previous releases exist, relup file to upgrade from the latest existing release to the current release is also generated.\n- Making a new release tarball consists of the following steps:\n    - Preparation\n        - Template files (`rel/*.eex`) used by `mix release` to create `env.sh` and `vm.args`.\n        - Configuration for `mix release` provided by `Elixir.Mix.Tasks.AntikytheraCore.GenerateRelease.config_for_mix_release/0`.\n    - Release generation\n        - Ensure that source files are compiled and ` .app` is up-to-date.\n        - If a previous release is found, generate `*.appup` and `relup` files.\n            - `*.appup` files are generated before assembling.\n            - `relup` file is generated after assembling.\n        - Generate a new release by `:assemble` step of `mix release`.\n        - Generate `RELEASES` file, which is required by `:release_handler`.\n    - Cleanup\n        - Make a tarball by `:tar` step of `mix release`, then move it to the version directory.","ref":"Mix.Tasks.AntikytheraCore.GenerateRelease.html","title":"mix antikythera_core.generate_release","type":"task"},{"doc":"Generates a release configuration required by `mix antikythera_core.generate_release`.\n\nA release name for the configuration must be identical to the antikythera instance name.\nThe following is an example configuration for `antikythera_instance_example`:\n\n```\ndef project do\n  [\n    releases: [\n      antikythera_instance_example: &Elixir.Mix.Tasks.AntikytheraCore.GenerateRelease.config_for_mix_release/0,\n\n      ...\n    ]\n  ]\nend\n```","ref":"Mix.Tasks.AntikytheraCore.GenerateRelease.html#config_for_mix_release/0","title":"Mix.Tasks.AntikytheraCore.GenerateRelease.config_for_mix_release/0","type":"function"},{"doc":"Uploads assets to asset storage so that they can be delivered via CDN.\n\nThis mix task is invoked during gear deployment from within each gear project.","ref":"Mix.Tasks.AntikytheraCore.UploadNewAssetVersions.html","title":"mix antikythera_core.upload_new_asset_versions","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraCore.UploadNewAssetVersions.html#run/1","title":"Mix.Tasks.AntikytheraCore.UploadNewAssetVersions.run/1","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.PrepareCore.html","title":"mix antikythera_local.prepare_core","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.PrepareCore.html#run/1","title":"Mix.Tasks.AntikytheraLocal.PrepareCore.run/1","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.PrepareGear.html","title":"mix antikythera_local.prepare_gear","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.PrepareGear.html#run/1","title":"Mix.Tasks.AntikytheraLocal.PrepareGear.run/1","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.Start.html","title":"mix antikythera_local.start","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.Start.html#run/1","title":"Mix.Tasks.AntikytheraLocal.Start.run/1","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.Stop.html","title":"mix antikythera_local.stop","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.Stop.html#run/1","title":"Mix.Tasks.AntikytheraLocal.Stop.run/1","type":"function"},{"doc":"Runs tests to ensure that antikythera's upgrade doesn't break backward-compatibility.\n\nIt works in the following sequence:\n\n- Starts an antikythera instance of a version specified by a git ref, with gears in specified directories installed\n- (If [testgear] is used, establishes websocket connection to [testgear]. Periodically sends echo requests)\n- Upgrades the antikythera instance to new version specified by another git ref\n- Run blackbox tests defined in each gear, checking whether their external behaviors are kept intact\n- (Raises if the websocket connection established above was abnormally closed before here)\n\n[testgear]: https://github.com/access-company/testgear","ref":"Mix.Tasks.AntikytheraLocal.UpgradeCompatibilityTest.html","title":"mix antikythera_local.upgrade_compatibility_test","type":"task"},{"doc":"$ mix antikythera_local.upgrade_compatibility_test   [git_ref_from[ git_ref_to]]\n\n- `comma_separated_gear_directories` can be absolute or relative paths to directories, such as `../testgear,/path/to/another_gear`.\n    - It assumes basenames of paths exactly match gear_names, for [testgear] detection.\n- `git_ref_from` defaults to `HEAD^`. `git_ref_to` defaults to `HEAD`.","ref":"Mix.Tasks.AntikytheraLocal.UpgradeCompatibilityTest.html#module-usage","title":"Usage - mix antikythera_local.upgrade_compatibility_test","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.UpgradeCompatibilityTest.html#run/1","title":"Mix.Tasks.AntikytheraLocal.UpgradeCompatibilityTest.run/1","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.UpgradeCompatibilityTest.WS.html","title":"mix antikythera_local.upgrade_compatibility_test.ws","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.UpgradeCompatibilityTest.WS.html#send_loop/0","title":"Mix.Tasks.AntikytheraLocal.UpgradeCompatibilityTest.WS.send_loop/0","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html","title":"mix antikythera_local.version_upgrade_test","type":"task"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#check_application_configs/0","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.check_application_configs/0","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#check_healthcheck_and_gear_endpoints/0","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.check_healthcheck_and_gear_endpoints/0","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#check_new_version_is_applied/4","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.check_new_version_is_applied/4","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#check_new_version_with_noupgrade_is_not_applied/4","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.check_new_version_with_noupgrade_is_not_applied/4","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#check_repository_is_clean/1","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.check_repository_is_clean/1","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#check_testgear_artifact_dirs/1","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.check_testgear_artifact_dirs/1","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#check_version/2","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.check_version/2","type":"function"},{"doc":"","ref":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.html#run/1","title":"Mix.Tasks.AntikytheraLocal.VersionUpgradeTest.run/1","type":"function"},{"doc":"Ensures only gear applications are specifed in `gear_deps/0`.","ref":"Mix.Tasks.Compile.EnsureGearDependencies.html","title":"mix compile.ensure_gear_dependencies","type":"task"},{"doc":"Statically checks issues in gear's source code.\n\nSince antikythera is designed to run multiple gears within an ErlangVM, it assumes that each gear\n\n- is implemented so that it can run side-by-side with other gears, and\n- does not disturb resource allocation controlled by antikythera.\n\nThis task is integrated into `mix compile` command so that these issues are detected each time gear's source code is compiled.","ref":"Mix.Tasks.Compile.GearStaticAnalysis.html","title":"mix compile.gear_static_analysis","type":"task"},{"doc":"","ref":"Mix.Tasks.Compile.GearStaticAnalysis.html#run/1","title":"Mix.Tasks.Compile.GearStaticAnalysis.run/1","type":"function"},{"doc":"Touches source files that need to be recompiled.\n\nNormally mix automatically resolves which source files to recompile based on file modification times.\nHowever some macro-related code modifications do not trigger recompilation.\nThis mix task finds those kinds of modifications and touch files to compile so that subsequent task recompiles them.\n\nNote that `@external_resource` module attribute does not support directory as external dependency of a module.\nFor a module that depends on all files under a directory, we have to manually check the latest modification time\nof the whole directory tree.\n\nCurrent targets of this task are:\n\n- `mix.exs` files in gear projects depend on `mix_common.exs` in the antikythera repository.\n  Mix assumes that project configurations reside only in `mix.exs` and `config/*.exs`,\n  so even if `mix_common.exs` has been modified mix does not notice.\n  Note that antikythera's `mix.exs` also depends on `mix_common.exs` but it's handled differently\n  (at the beginning of `mix.exs`), because we can't run mix tasks before it's compiled!\n- `SomeGear.Template` module depends on all existing HAML templates in `web/template/`.\n- Similarly, `SomeGear.Asset` module depends on all asset files in `priv/static/`.\n  Although `SomeGear.Asset` is recompiled after modifying an asset file, it doesn't automatically trigger\n  recompilation/reload in iex session because `priv/static/**` is not monitored by `:exsync` (in current configuration).","ref":"Mix.Tasks.Compile.PropagateFileModifications.html","title":"mix compile.propagate_file_modifications","type":"task"},{"doc":"","ref":"Mix.Tasks.Compile.PropagateFileModifications.html#run/1","title":"Mix.Tasks.Compile.PropagateFileModifications.run/1","type":"function"},{"doc":"# Antikythera Framework\n\n[![hex badge](https://img.shields.io/hexpm/v/antikythera.svg)](https://hex.pm/packages/antikythera)\n\n**Antikythera** is an [Elixir] framework to build your own in-house PaaS (Platform as a Service).\n\nYou can run multiple web services while managing only a single cluster of [ErlangVM][Erlang] nodes.\nFrom the viewpoint of service developers antikythera provides FaaS(Function as a Service)-like development experiences.\n\n[Elixir]: https://elixir-lang.org/\n[Erlang]: https://www.erlang.org/\n\n**Note:** Antikythera is already used in production at [ACCESS],\nthough as an OSS, it is still in early development stage and missing many components.\nPlease be aware of that, and stay tuned for further additions!\n\n[ACCESS]: https://www.access-company.com/","ref":"readme.html","title":"Antikythera Framework","type":"extras"},{"doc":"- As a platform for multiple services:\n    - Resource control for multi-service and multi-tenant use cases\n    - Automated deployment without affecting other running services\n    - Built-in logging, monitoring and configuration management\n    - Service-to-service communications without network overhead\n- As a web framework:\n    - HTTP request processing with arbitrary Elixir code with WebSocket support\n    - Domain-based routing for multiple services; path-based routing within single service\n    - CDN support for static contents, HAML template for dynamic web pages\n- As an asynchronous job executor:\n    - Built-in distributed job queues\n    - Running arbitrary Elixir code for each job","ref":"readme.html#features","title":"Features - Antikythera Framework","type":"extras"},{"doc":"![Antikythera Architecture](assets/AntikytheraArchitecture.png)\n\n- We refer to each antikythera cluster as an **\"antikythera instance\"**,\n  and each web service running within an antikythera instance as a **\"gear\"**.\n    - Antikythera Instance example: [antikythera_instance_example](https://github.com/access-company/antikythera_instance_example)\n    - Gear example: [testgear](https://github.com/access-company/testgear)\n    - These two projects are also used for antikythera's test.\n- Antikythera is heavily dependent on the power of [ErlangVM][Erlang].\n    - Antikythera is written in [Elixir], and all gears must also be written in Elixir.\n- Multiple gears are co-located and executed within the same ErlangVMs of an antikythera instance.\n    - This type of co-location of multiple applications are sometimes referred to as \"nano-services architecture\".\n    - This way we can eliminate the infrastructure/operational cost for having multiple web application stacks.\n    - Also, inter-service communications in a micro-services architecture become gear-to-gear communications in this nano-services architecture.\n      Unlike micro-services architecture, it doesn't require network roundtrip as all gears are running within a single ErlangVM, resulting in lower communication overhead.\n    - Although gears share the same computing resources, they are executed in an independent manner thanks to the Erlang's process model.\n    - Antikythera instance and its gears are all separate [mix](https://hexdocs.pm/mix/Mix.html) projects.\n        - Developers of different gears can work on their own projects, independently.\n        - Library dependencies are defined per-antikythera-instance, and all gears inherit those dependencies.","ref":"readme.html#basic-architecture","title":"Basic architecture - Antikythera Framework","type":"extras"},{"doc":"See our [Getting Started](https://hexdocs.pm/antikythera/getting_started.html) guide!","ref":"readme.html#getting-started","title":"Getting Started - Antikythera Framework","type":"extras"},{"doc":"- [For antikythera instance administrators](https://hexdocs.pm/antikythera/instance_administrators.html) (TBD)\n- [For gear developers (i.e. developers of web services)](https://hexdocs.pm/antikythera/gear_developers.html)\n- [API Reference](https://hexdocs.pm/antikythera/api-reference.html)","ref":"readme.html#documentation","title":"Documentation - Antikythera Framework","type":"extras"},{"doc":"We encourage you to contribute to antikythera! Please check out our [Contributing Guide](https://github.com/access-company/antikythera/blob/master/CONTRIBUTING.md).\n\nFor both core and gear development, please make use of our [Style Guide](https://github.com/access-company/antikythera/blob/master/STYLE_GUIDE.md).\n\n- [Issue Tracker](https://github.com/access-company/antikythera/issues)\n- [Users Mailing List](https://groups.google.com/forum/#!forum/antikythera_users)","ref":"readme.html#contributing","title":"Contributing - Antikythera Framework","type":"extras"},{"doc":"Copyright(c) 2015-2023 [ACCESS CO., LTD][ACCESS]. All rights reserved.\n\nAntikythera source code is licensed under the [Apache License version 2.0](./LICENSE).","ref":"readme.html#copyright-and-license","title":"Copyright and License - Antikythera Framework","type":"extras"},{"doc":"# Antikythera Style Guide\n\nOverall policy:\n\n1. Be consistent within a chunk of your changes.\n2. Be consistent with surrounding code.\n3. Make use of this (and linked) Style Guides when in doubt.\n4. Ask and discuss!","ref":"style_guide.html","title":"Antikythera Style Guide","type":"extras"},{"doc":"We are *mostly* following [Credo's Elixir Style Guide](https://github.com/rrrene/elixir-style-guide).\n\nStyles marked as **preferred** in the Credo's guide are mostly used.\n\nIf you find yourself lost about coding style, always search in existing code for similarities.\nOr just ask in a Pull Request or [Users ML](https://groups.google.com/forum/#!forum/antikythera_users).\n\nAs a side-reading, [A community driven style guide for Elixir](https://github.com/levionessa/elixir_style_guide) is also available.\nWe are not compliant with all of it, but it provides good advices.\n\nNote: Although we are mostly following its style guide,\nwe are not completely relying on [Credo](https://github.com/rrrene/credo) static analysis.","ref":"style_guide.html#guidelines","title":"Guidelines - Antikythera Style Guide","type":"extras"},{"doc":"Typespecs are useful for both documentation purpose and success typing analysis ([dialyzer](https://www.erlang.org/doc/man/dialyzer.html)).\nSo it's preferred to explicitly declare typespecs of functions, especially for public ones.\n\nWe are utilizing [`Croma.Defun`](https://hexdocs.pm/croma/Croma.Defun.html) series for easy typespec notations.\n\nExample:\n\n```elixir\n@type timed_id_t :: {pos_integer, reference}\n\ndefun five_params_fun(key      :: v[atom],\n                      some_str :: v[String.t],\n                      key_list :: [atom],\n                      timed_id :: timed_id_t,\n                      state    :: v[map]) :: :ok | {:error, term} do\n  # Do something\nend\n```\n\nAlso, use `v[]` validation where applicable. See [`Croma.Defun`](https://hexdocs.pm/croma/Croma.Defun.html) for details.\nNote that validations of arguments by `v[]` are disabled when compiling for production environment.\n\nDo not hesitate to put typespecs on complicated private functions too.","ref":"style_guide.html#typespec","title":"Typespec - Antikythera Style Guide","type":"extras"},{"doc":"For module names we simply use CamelCase even if they contain acronyms.\nFor instance we use `Antikythera.Url`, not `Antikythera.URL`.\nThis is to eliminate exceptional case in our naming rules,\nwhich leads to easier conversions between module aliases and strings.","ref":"style_guide.html#module-names","title":"Module names - Antikythera Style Guide","type":"extras"},{"doc":"Module aliases **MUST** be ordered in their generality;\nmodules with general use cases **MUST** come above modules with limited use cases.\n\n1. External modules\n2. `Antikythera`\n3. `AntikytheraCore`\n4. `AntikytheraEal`\n\nFor sub-modules under namespaces listed above, there is no specific order defined.\n\nYou SHOULD bundle modules of the same level:\n\n```elixir\nalias Antikythera.{GearName, GearNameStr}\n```","ref":"style_guide.html#order-of-module-aliases","title":"Order of module aliases - Antikythera Style Guide","type":"extras"},{"doc":"`import`ing a module will allow you to use its functions without `SomeModule.` prefix.\nHowever it SHOULD NOT be abused because it can lead to name conflicts.\n\n- Use only when you are absolutely sure it is safe and useful there.\n- Limit its targets with `:only` option.\n- Consider limiting its scope by placing it inside a specific scope.","ref":"style_guide.html#do-not-abuse-import","title":"Do not abuse `import` - Antikythera Style Guide","type":"extras"},{"doc":"You MUST NOT omit parentheses in function calls.\n\n```elixir\n# Okay\nresult = zero_arity_function()\nresult = one_arity_function(arg1)\n\n# Not okay as it gets warned by elixir compiler\nresult = zero_arity_function\n\n# Not okay (less readable especially when some args are long and not pipe-ready)\nresult = some_function arg1, arg2, arg3, arg4\n\n# Okay\nresult = some_function(arg1, arg2, arg3, arg4)\n```\n\nAlso you MUST NOT omit parentheses when you define functions.\nNote that you MAY omit parentheses when invoking macros.","ref":"style_guide.html#function-calling-syntax","title":"Function calling syntax - Antikythera Style Guide","type":"extras"},{"doc":"Pipe operator improves code readability when 1st argument is a complex expression\nbecause it reorders expressions in the evaluation order.\n\n```elixir\n# You have to read inside-out, as function arguments are evaluated before function invocation\nEnum.sort(Enum.filter(list, &some_predicate/1))\n\n# Much readable as you can now naturally follow the evaluation order from left to right\nEnum.filter(list, &some_predicate/1) |> Enum.sort()\n```\n\nAlso pipe operator makes separation between \"data\" and \"its transformation\" clearer.\n\n```elixir\nlist\n|> Enum.map(some_func)\n|> Enum.filter(some_predicate)\n|> Enum.reduce(acc0, some_reducer)\n```\n\nIf you have neither of the above benefits in mind, you SHOULD NOT use pipe operator.\n\n```elixir\n# absurd\nkeys = some_map |> Map.keys()\n\n# simple enough\nkeys = Map.keys(some_map)\n```","ref":"style_guide.html#using-operator","title":"Using `|>` operator - Antikythera Style Guide","type":"extras"},{"doc":"# Contributing Guide\n\nAs an ever-expanding application platform, antikythera is always accepting contributions.\n\nHere, contributions can be of several forms:\n\n- Asking questions\n- Finding bugs\n- Making feature suggestions\n- Committing code\n- Improving documentations\n\nAll of them are helpful! Follow the guidelines provided here.","ref":"contributing.html","title":"Contributing Guide","type":"extras"},{"doc":"- First, read up [documentations](https://hexdocs.pm/antikythera) before asking questions/making suggestions.\n- Also, search in [antikythera API reference](https://hexdocs.pm/antikythera/api-reference.html)\n  to find detailed information about modules/functions.\n- Make sure you are using latest antikythera with latest supported Erlang/Elixir version.\n    - Refresh dependencies if dependency/compile related error occurred:\n\n      ```shell\n      mix deps.clean\n      mix deps.update --all\n      mix deps.get\n      ```\n\n- If you need further help, found bug in antikythera code, or came up with feature suggestions, follow the steps below.\n\n#","ref":"contributing.html#surveying-and-discussions","title":"Surveying and Discussions - Contributing Guide","type":"extras"},{"doc":"- Your problem might already be encountered and solved by other developers, and they could offer you a help.\n    - Even if they have not solved the problem, they may still be able to provide additional information,\n      which can be quite useful for bug fixes.\n- If your problem is just an oversight or an error, it can be solved here.\n- Discussions in the Users ML can be referenced in the future, making it an effective knowledge base.\n- If the problem is reported to ML first, other developers can recognize the problem and reduce their cognitive load.\n- For feature suggestions:\n    - Other developers can agree/disagree/reinforce/provide additional points to your suggestions.\n    - Core developers can respond with design philosophies/decisions behind antikythera development,\n      and provide proper directions to your suggestion:\n        - Implement it as a new feature\n        - Achieve it with existing features\n        - Not going to support as an antikythera feature\n- If you think your mail is not getting attentions, reply to the thread to request answers.\n  People just might have overlooked the mail.\n    - Or, your post might be too trivial to get answered. Make sure you check out documentations mentioned in the previous section.\n- If you are working closely to the core developer team, you MAY just tell your concern to them and open an issue.\n    - But, as described above, posting a mail to Users ML is itself beneficial to the whole antikythera community, so it is preferred.\n\n#","ref":"contributing.html#post-in-antikythera-framework-users-mailing-list-users-ml-https-groups-google-com-forum-forum-antikythera_users","title":"Post in [Antikythera Framework Users Mailing List (Users ML)](https://groups.google.com/forum/#!forum/antikythera_users) - Contributing Guide","type":"extras"},{"doc":"- If your problem is indeed marked as a bug, or your feature suggestions are accepted,\n  you or one of core developers can open issues.\n- If you are absolutely certain that you found a bug, you MAY just report an issue directly.\n    - However, dropping a descriptive mail in Users ML is always helpful and preferred.\n- **DO NOT** make feature issues directly. Discuss in ML first.\n    - This is to keep backlog as small and manageable as possible.\n    - If feature issues are randomly opened, core developers must periodically review and organize them, which is costly.\n\n#","ref":"contributing.html#report-in-issue-tracker-https-github-com-access-company-antikythera-issues","title":"Report in [Issue Tracker](https://github.com/access-company/antikythera/issues) - Contributing Guide","type":"extras"},{"doc":"- Try to be as descriptive as possible.\n    - **NOT** just \"I can not do XXX\" or \"It says YYY on compile\".\n    - These information are essential:\n        - What are you going to achieve? (And Why?)\n        - What changes/circumstances cause the problem?\n        - What does the compiler/log say?\n        - Which version(commit hash) of antikythera you use?\n        - Can you reproduce it and how?\n    - (Not all of them are always required, but they surely make your posts more descriptive)","ref":"contributing.html#mail-and-issue-manners","title":"Mail and Issue manners - Contributing Guide","type":"extras"},{"doc":"- Reported issues are evaluated by their urgency and assigned/tagged appropriately.\n- Basically those issues are assigned to core developers and implemented.\n  However, it is welcomed to make your own effort to pick up one of them and implement it.\n    - Just drop comments in the issue you are willing to tackle with.\n      Then the core team will assign it to you.\n- If you try to implement features, follow the instruction explained in this section.\n\n#","ref":"contributing.html#contributing-to-the-antikythera-code","title":"Contributing to the antikythera code - Contributing Guide","type":"extras"},{"doc":"- Fork this repository and setup antikythera project:\n\n  ```shell\n  cd path/to/antikythera/\n  mix deps.get\n  mix test\n  ```\n\n- After you have made sure that you can run antikythera tests, check out a feature branch from `master` branch.\n    - Antikythera and its related projects use [GitHub-flow](https://scottchacon.com/2011/08/31/github-flow/);\n      anything on `master` are automatically tested (though it is not actually \"deployed\" to somewhere).\n    - For branch name and commit messages, look up recent branches/commits and follow their conventions\n      (be sufficiently descriptive and simple).\n- Implement the feature with tests.\n    - At least, test behavior of public interfaces of the modules,\n      [as advised by Elixir author, José Valim](https://stackoverflow.com/a/20949676/5421126).\n    - Always confirm that the whole test passes.\n      [`mix test.watch`](https://github.com/lpil/mix-test.watch) is useful to monitor file changes and automatically run the test.\n    - If your change requires an actual gear for testing, see the next section.\n- Implement by a (relatively) small chunk.\n    - Your feature might require quite a large addition or changes.\n    - Such a large patch is hard to read and review, leads to blocking of reviewers' activities.\n    - Keep your components (modules/functions) loosely-coupled and follow the single responsibility principle,\n      so that they can be implemented/tested separately, thus can be reviewed/merged steadily.\n- Make use of our [style guide](https://github.com/access-company/antikythera/blob/master/STYLE_GUIDE.md).\n\n##","ref":"contributing.html#developing","title":"Developing - Contributing Guide","type":"extras"},{"doc":"- Sometimes it is essential to test features through an actual gear.\n    - For example, when you implement a new [`Plug`](./lib/web/controller/plug.ex).\n    - Or, your implementation touches existing codes and might break current functionalities.\n- In such cases, we use [`testgear`](https://github.com/access-company/testgear) and\n  [`antikythera_instance_example`](https://github.com/access-company/antikythera_instance_example)\n- Since these are separate projects, you will likely write tests using `testgear` as a separate step from implementing features in antikythera.\n- Fork `testgear` repository and setup like the following:\n\n  ```shell\n  $ cd path/to/testgear/\n  $ export ANTIKYTHERA_INSTANCE_DEP='{:antikythera_instance_example, [git: \"git@github.com:access-company/antikythera_instance_example.git\"]}'\n  $ mix deps.get\n  $ mix deps.get                      # Fetch dev/test-only dependencies declared in antikythera_instance_example\n  $ mix test                          # Whitebox test\n  $ iex -S mix\n  (from another terminal)\n  $ export ANTIKYTHERA_INSTANCE_DEP='{:antikythera_instance_example, [git: \"git@github.com:access-company/antikythera_instance_example.git\"]}'\n  $ TEST_MODE=blackbox_local mix test # Blackbox test\n  ```\n\n- Then, you set up a controller that uses new feature, and test its integrity via request and response.\n    - Consult [the document for testing in gears](https://hexdocs.pm/antikythera/testing.html) as a reference.\n\n#","ref":"contributing.html#use-testgear-for-testing-antikythera-s-features","title":"Use `testgear` for testing antikythera's features - Contributing Guide","type":"extras"},{"doc":"- Branches are Pull Requested against (mostly) `master` branch, then reviewed by core team, and merged when approved by **ALL MEMBERS**.\n- When you are confident about your development chunk, prepare your Pull Request.\n    - First, you **MUST** self-review the changes you made in the branch. This is particularly important.\n        - Follow the coding conventions described in the [style guide](https://github.com/access-company/antikythera/blob/master/STYLE_GUIDE.md).\n        - Make sure your code fit in existing antikythera code.\n    - Provide sufficient `@moduledoc`, `@doc` and `@typedoc`, especially if it is publicly available.\n    - If your code contains workaround or hacky solution, put an explanation comments.\n    - Run the following commands and fix errors or apply the fixes:\n        - `$ mix format`\n        - `$ mix compile`\n        - `$ mix dialyzer`\n        - `$ mix credo -a --strict`\n            - Make sure you don't add warnings or errors.\n        - `$ mix test`\n        - `$ TEST_MODE=blackbox_local mix test`\n        - `$ mix antikythera_local.upgrade_compatibility_test`\n        - `$ mix antikythera_local.version_upgrade_test`\n        - `$ markdownlint-cli2 --fix \"**/*.md\"`\n        - `$ mix docs`\n            - Make sure you don't add warnings or errors.\n    - If reviewers deemed the Pull Request is not self-reviewed well, it CAN be rejected.\n- When you are confident on your branch, push it to your fork repository, then open Pull Request.\n    - Pull Request title can be just branch name, or summarized description.\n    - Make sure the Pull Request and the issue are mutually linked.\n- When your Pull Request is ready, you **MUST** post description either in the Pull Request itself, or in the issue.\n    - In the description:\n        - Briefly state what is implemented in the patch (no need to explain procedures step by step, just outline).\n        - If the patch is incomplete and a part of the whole feature, make sure what part of the feature is implemented\n          and how it works in the whole picture (and optionally, what is not yet implemented and what comes in next chunk).\n        - If you think some part of the patch is hard to understand, any additional notes will be helpful for reviewers.\n\n#","ref":"contributing.html#pull-request","title":"Pull Request - Contributing Guide","type":"extras"},{"doc":"- For each comments made by reviewers, you **MUST**:\n    - Fix the specified error/style/logic/etc..., OR,\n    - Explain decision factor/reason behind code, and discuss, THEN,\n    - Fix, leave as-is or put comments on that part, AND FINALLY,\n    - Reply to the comments with fixing commit hash, or closing words (e.g. \"Addressed in \\ \" or \"As discussed above, we keep this as-is\").\n- When all reviewers approved the Pull Request, one of core developers will take care of the rest:\n    - The branch will be merged with \"squash\" (in most cases from GitHub UI, i.e. \"button merge\").\n        - In squash merge commit, `Author` is the contributor, and `Committer` is a member who performed the merge.\n          So be assured that your contributions are always properly counted!\n        - When successfully merged, remote branch can be safely deleted (for your fork repository branches, it is up you).\n    - When the new `master` is successfully tested:\n        - If the issue is solved/the feature is fully implemented, it will be closed. Thanks for your contribution!\n        - If the issue is still in progress, you resume your development.\n    - Sometimes tests can fail due to unexpected side effects or errors.\n        - You can tackle with fixing it, but in most cases core team will take care of such complicated failures.","ref":"contributing.html#review-and-merge","title":"Review and Merge - Contributing Guide","type":"extras"},{"doc":"# Getting Started\n\n- Set up your [development environment](https://hexdocs.pm/antikythera/development_environment.html).\n    - Basically you need to (1) install Erlang and Elixir, and (2) prepare for [domain-based routing](https://hexdocs.pm/antikythera/routing.html).\n- Then, you need your own antikythera instance.\n    - **Documentation about this part is in preparation. Please be patient!**\n    - If you don't have one yet, you can start by using [`antikythera_instance_example`](https://github.com/access-company/antikythera_instance_example).\n\n      ```shell\n      $ git clone https://github.com/access-company/antikythera_instance_example.git\n      $ cd antikythera_instance_example\n      $ mix deps.get\n      (If you encountered an error complaining about required Erlang and/or Elixir versions, run `asdf install` here)\n      $ mix deps.get # fetch additional dependencies declared in antikythera_instance_example\n      $ mix compile\n      ```\n\n- Now you can generate your own gear project using your antikythera instance.\n    - From local clone of your antikythera instance:\n\n      ```shell\n      $ mix antikythera.gear.new ~/path/to/my_gear\n      $ cd ~/path/to/my_gear\n      # In order for versioning utility of antikythera to work, git repository and an initial commit are necessary.\n      $ git init\n      $ git add .\n      $ git commit -m 'Initial commit'\n      $ mix deps.get\n      $ mix deps.get # fetch additional dependencies declared in your antikythera instance\n      $ iex -S mix\n      ```\n\n    - Open `http://my-gear.localhost:8080/hello` in web browser, see if it is working.\n- Congratulations! Now your first gear is up and running.\n    - Take a look around and play around with it.\n    - Check out documentations and see what you can do with antikythera and gears.\n        - Guide for instance administrators[](./instance_administrators/) (TBD)\n        - [Guide for gear developers (i.e. developers of web services)](https://hexdocs.pm/antikythera/gear_developers.html)\n        - [API References](https://hexdocs.pm/antikythera/api-reference.html) (To be published)","ref":"getting_started.html","title":"Getting Started","type":"extras"},{"doc":"# Development Environment","ref":"development_environment.html","title":"Development Environment","type":"extras"},{"doc":"- Before anything else, do note that **an antikythera instance and its gears run in the same ErlangVM cluster**.\n  Therefore, obviously, all development AND deployment environments must have the same Erlang and Elixir versions!\n- We strongly recommend [asdf](https://github.com/asdf-vm/asdf) to manage this synchronization.\n- Antikythera itself doesn't impose any version restrictions other than the one specified in `mix.exs`.\n  Individual antikythera instance project may enforce a more fine-grained restrictions on language versions.\n- Other requirements:\n    - Some dependencies contain C source code and thus a reasonably new C compiler is required.\n    - In macOS you may be warned by outdated GNU make version during compiling antikythera's dependencies.\n      In that case install GNU make using e.g. [homebrew](https://brew.sh/).\n      The installed executable is `gmake`; create a symlink named `make` in somewhere in your `$PATH`\n      so that the desired version of make command is selected.\n    - Install development header of [Expat](https://libexpat.github.io/), which is required by [fast_xml](https://github.com/processone/fast_xml) library.\n        - In macOS this should be already installed. If not, install `expat` with homebrew.\n    - On Linux you have to install `inotify-tools` to enable file system watching tools\n      such as [exsync](https://github.com/falood/exsync) and [mix_test_watch](https://github.com/lpil/mix-test.watch).\n- [Install asdf as explained in the official README.md](https://github.com/asdf-vm/asdf#setup)\n- Install asdf plugins\n    - `$ asdf plugin-add erlang`\n    - `$ asdf plugin-add elixir`\n    - `$ asdf plugin-add nodejs`\n        - Node.js installation is optional. You do not need it if you are not using asset-related features.\n- Install language versions\n    - In order to invoke initial `mix deps.get`, you need to globally install reasonably new Erlang and Elixir versions.\n        - `$ asdf install erlang x.y.z && asdf global erlang x.y.z`\n        - `$ asdf install elixir x.y.z && asdf global elixir x.y.z`\n    - Working on your antikythera instance:\n        - If `.tool-versions` already exists, `$ asdf install`\n        - If not,\n            - `$ asdf local erlang x.y.z && asdf local elixir x.y.z`\n            - then commit the generated `.tool-versions` file.\n    - Working on a gear:\n        - Make sure you have a valid symlink named `.tool-versions` which points to `deps/your_antikythera_instance/.tool-versions`\n            - This way you can make your gear's language versions in sync with your antikythera instance.\n            - If your gear is generated using [`mix antikythera.gear.new`](https://github.com/access-company/antikythera/blob/master/lib/mix/gear.new.ex),\n              symlink to `.tool-versions` is automatically created.\n        - `$ asdf install`\n\n#","ref":"development_environment.html#install-compilers-and-build-tools","title":"Install Compilers and Build Tools - Development Environment","type":"extras"},{"doc":"- Although `iex` is able to remember commands executed in previous sessions, the feature is not enabled by default.\n- It is highly recommended to set the following environment variable during development:\n    - `export ERL_AFLAGS=\"-kernel shell_history enabled\"` (for bash family)","ref":"development_environment.html#shell-history-in-iex","title":"Shell history in `iex` - Development Environment","type":"extras"},{"doc":"- Antikythera routes web requests by subdomains and paths. See [routing](https://hexdocs.pm/antikythera/routing.html).\n- In local development, you must be able to resolve subdomain of localhost (e.g. `your-gear.localhost`) into loopback address (`127.0.0.1`).\n    - The easiest way is to add a line like `127.0.0.1 your-gear.localhost` to your `/etc/hosts`.\n    - Alternatively, you can setup a local DNS server (such as [dnsmasq](https://thekelleys.org.uk/dnsmasq/doc.html)) to resolve all subdomains of localhost.","ref":"development_environment.html#prepare-for-domain-based-routing","title":"Prepare for Domain-based Routing - Development Environment","type":"extras"},{"doc":"- In your local environment you can customize some of antikythera's default settings using the following environment variables:\n    - Runtime environment variables:\n        - `BASE_DOMAIN`:\n            - Suffix part of domain names to be used when clients interact with running gears. Defaults to `localhost`.\n            - For example, when you are running `:some_gear` as `BASE_DOMAIN=somedomain iex -S mix`,\n              you can browse the top page of the gear at `http://some-gear.somedomain:8080/` (assuming that the default port number is used; see below).\n            - Note that when using `BASE_DOMAIN` other than `localhost`, you may need to tell the DNS resolver on the client side about the gear's domain name.\n        - `PORT`:\n            - Port number to receive incoming web requests from. Defaults to `8081` during `mix test`, and `8080` otherwise\n              (thus one can run both `iex -S mix` and `mix test` at the same time).\n            - Explicitly set this when you run multiple antikythera servers within your machine.\n        - `TEST_PORT`:\n            - Port number to send HTTP requests to during `blackbox_local` tests. Defaults to `8080`.\n        - `NO_LISTEN`:\n            - A boolean flag for disabling web server functionality of antikythera. Defaults to `false`.\n            - `true` is implicitly set when using `Antikythera.Mix.Task.prepare_antikythera_instance/0` in your mix task.\n        - `LOG_LEVEL`:\n            - Log level of all gears. Defaults to `info`. See also [logging](https://hexdocs.pm/antikythera/logging.html).\n        - `TEST_LOG_ON_TERMINAL`:\n            - A boolean flag for enabling output gear log on terminal during `mix test`. Defaults to `false`.\n        - `SOME_GEAR_CONFIG_JSON`:\n            - Gear config of `:some_gear`. See also [gear_config](https://hexdocs.pm/antikythera/gear_config.html).\n    - Compile-time environment variables: (to change the followings you need to recompile antikythera)\n        - `GEAR_ACTION_TIMEOUT`:\n            - Milliseconds to wait until gear action finishes as a default. Defaults to `10000`. See also [controller](https://hexdocs.pm/antikythera/controller.html).\n        - `GEAR_PROCESS_MAX_HEAP_SIZE`:\n            - Maximum size of per-process heap memory in words. Defaults to `50000000` (400MB in 64-bit architecture).\n\n- If you develop mix task, you can use the following environment variables:\n    - Runtime environment variables:\n        - `ANTIKYTHERA_MIX_TASK_MODE`:\n            - A boolean flag whether Antikythera runs for mix task. Defaults to `false`.\n            - You must set `true` even if your command is mix task.\n            - If you set a value other than `local` to `ANTIKYTHERA_RUNTIME_ENV`, Antikythera thinks it was built for a release package and is deployed to a cloud.\n              Then, the path to the config files will be broken, and Antikythera will try to use cloud services even if your mix task running enrivonment doesn't allow them.\n              If this variable is `true`, Antikythera won't do them.\n    - Compile-time environment variables: (to change the followings you need to recompile antikythera)\n        - `ANTIKYTHERA_MIX_TASK_MODE`:\n            - A boolean flag whether Antikythera is compiled for mix task. Defaults to `false`.\n            - You must set `true` even if your command is mix task and automatically compile Antikythera.\n            - If you set a value other than `local` to `ANTIKYTHERA_COMPILE_ENV`, Antikythera thinks it is built for a release package and will be deployed to a cloud.\n              Then, the path to the config files will be broken, and Antikythera will try to use cloud services even if your mix task running enrivonment doesn't allow them.\n              If this variable is `true`, Antikythera won't do them.","ref":"development_environment.html#environment-variables-to-tweak-behavior-of-antikythera","title":"Environment variables to tweak behavior of antikythera - Development Environment","type":"extras"},{"doc":"# Dependency Management\n\n- Elixir ships with the official build tool [`mix`](https://hexdocs.pm/mix/Mix.html) and it manages dependencies for elixir projects.\n  Open source libraries written in Elixir or Erlang are basically published at [hex.pm](https://hex.pm/).\n  Usually elixir/mix project specifies its dependencies in `mix.exs` file.\n- In antikythera, library dependencies are defined **in your antikythera instance**.\n  Gear projects inherit dependencies of their antikythera instance, and have no means of specifying non-gear dependencies.\n- This is the design decision made based on the following reasons:\n    - Antikythera instances host multiple gears that are independent of each other.\n    - If each gear could specify its own dependencies, those dependencies would conflict and disrupt execution of other gears.\n    - In cases where multiple library candidates are found for a specific purpose,\n      it is beneficial to choose one of them and share it by all gears in that you can reuse knowledge of that library.\n- Therefore all libraries used by both an antikythera instance and gears are managed solely by the antikythera instance.\n    - Conversely, it is the duty of antikythera instance administrators to carefully choose good libraries and manage their versions.\n- In order to keep gears' dependencies in-sync with your antikythera instance,\n  gear developers should periodically update their gears with `$ mix deps.update antikythera_instance_name`.\n    - Update cycle should be determined by how often your antikythera instance is updated.\n    - Note that unless some breaking changes are introduced to your antikythera instance by e.g. updating a library to an incompatible version,\n      already compiled gears should continue to work.\n    - This is an essential feature of antikythera that allows you to independently update antikythera instances and its gears!\n      See [Deployment](https://hexdocs.pm/antikythera/deployment.html) for details.","ref":"library_dependencies.html","title":"Dependency Management","type":"extras"},{"doc":"- Available libraries must be defined in your antikythera instance's `mix.exs`, just as you do in normal mix projects.\n- A gear project inherits its antikythera instance's dependencies, so all libraries listed in the antikythera instance are also available to its gears.\n- If you have requests of libraries to include, consult your antikythera instance administrators.\n\n### \"Indirect\" dependencies\n\n- In antikythera instances, you can declare some dependencies as `:indirect` ones like so:\n\n  ```elixir\n  {:ranch, \"1.4.0\" , [indirect: true]},\n  ```\n\n- As the name suggests, you can use this special option to \"lock\" indirect dependencies defined in your dependencies.\n    - Without this, versions of indirect dependencies become ambiguous in gear projects, potentially leading to unexpected behaviors.\n    - Startup of indirect dependencies as runtime OTP applications are controlled by their \"parent\" dependencies.\n      Neither antikythera instances nor their gears explicitly start them.","ref":"library_dependencies.html#libraries-available-from-gears","title":"Libraries available from gears - Dependency Management","type":"extras"},{"doc":"- Although gears cannot specify dependencies in general, they can declare dependency to other gears.\n- For this purpose you can specify the names of gears that your gear depends on by `gear_deps/0` in gears' `mix.exs`.\n    - For example, the following `gear_deps/0` declares dependency to `gear_a` and `gear_b`\n\n      ```elixir\n      defp gear_deps() do\n        [\n          {:gear_a, [git: \"git@github.com:your-organization/gear_a.git\"]},\n          {:gear_b, \"1.0.0\", [organization: \"your-organization\"]}, # Private packages; see https://hex.pm/docs/private\n        ]\n      end\n      ```\n\n    - Note that `gear_deps/0` function can be module private.\n- If you have any gear dependencies, you should also update them periodically,\n  e.g. `$ mix deps.update antikythera_instance_name gear_a gear_b`.","ref":"library_dependencies.html#specifying-other-gears-as-dependencies","title":"Specifying other gears as dependencies - Dependency Management","type":"extras"},{"doc":"- Some development/test libraries are useful and recommended to include in your antikythera instance.\n- Also check out [antikythera's `mix.exs`](https://github.com/access-company/antikythera/blob/master/mix.exs) as a reference.\n\n#","ref":"library_dependencies.html#tools","title":"Tools - Dependency Management","type":"extras"},{"doc":"- With [dialyxir](https://github.com/jeremyjh/dialyxir), you can run typecheck of your code by `$ mix dialyzer`.\n    - `dialyxir` wraps the `dialyzer`, allowing it to conveniently run from mix projects.\n- See [here](https://learnyousomeerlang.com/dialyzer) for details of success typing analysis.\n\n### [ex_doc](https://github.com/elixir-lang/ex_doc) for documentations\n\n- With `ex_doc`, you can generate nicely rendered HTML documents from your Elixir code.\n- Elixir developers are always encouraged to add `@moduledoc`/`@doc`/`@typedoc` attributes to modules/functions/types to clearly express their intentions.\n- You should consider hosting your generated HTMLs at somewhere visible to your organization.\n\n#","ref":"library_dependencies.html#static-analysis-using-dialyzer-https-www-erlang-org-doc-man-dialyzer-html","title":"Static analysis using [dialyzer](https://www.erlang.org/doc/man/dialyzer.html) - Dependency Management","type":"extras"},{"doc":"- Test coverage reports are also essential to monitor code quality.\n    - Antikythera uses [`excoveralls`](https://github.com/parroty/excoveralls) as its coverage tool.\n      See the documentation of the tool for configuration options using `coveralls.json` file.","ref":"library_dependencies.html#test-coverage-reports","title":"Test coverage reports - Dependency Management","type":"extras"},{"doc":"# For Gear Developers\n\n## [Getting Started Guide](https://hexdocs.pm/antikythera/getting_started.html)\n\n## [Setting up your development environment](https://hexdocs.pm/antikythera/development_environment.html)\n\n## [Library dependencies](https://hexdocs.pm/antikythera/library_dependencies.html)\n\n## [Automatic deployment](https://hexdocs.pm/antikythera/deployment.html)\n\n## [Web request routing](https://hexdocs.pm/antikythera/routing.html)\n\n## [Gear to gear communication](https://hexdocs.pm/antikythera/g2g.html)\n\n## [Implementing controller action](https://hexdocs.pm/antikythera/controller.html)\n\n## [Dynamic HTML generation using templates](https://hexdocs.pm/antikythera/dynamic_html.html)\n\n## [Implementing websocket server](https://hexdocs.pm/antikythera/websocket.html)\n\n## [Asynchronous jobs](https://hexdocs.pm/antikythera/async_job.html)\n\n## [Executor pool for controlling computing resources](https://hexdocs.pm/antikythera/executor_pool.html)\n\n## [Logging](https://hexdocs.pm/antikythera/logging.html)\n\n## [Metrics Reporting](https://hexdocs.pm/antikythera/metrics_reporting.html)\n\n## [Alerting](https://hexdocs.pm/antikythera/alerting.html)\n\n## [Gear config](https://hexdocs.pm/antikythera/gear_config.html)\n\n## [Internationalization](https://hexdocs.pm/antikythera/i18n.html)\n\n## [Testing](https://hexdocs.pm/antikythera/testing.html)\n\n## [What gear implementations must not do](https://hexdocs.pm/antikythera/must_nots.html)\n\n## [Limitations](https://hexdocs.pm/antikythera/limitations.html)","ref":"gear_developers.html","title":"For Gear Developers","type":"extras"},{"doc":"# Deployment to Antikythera Instance\n\n**Note:** This page is being updated for OSS release. Please be patient.\nCurrent contents describes how our antikythera instance and gears are deployed in [ACCESS](https://www.access-company.com).","ref":"deployment.html","title":"Deployment to Antikythera Instance","type":"extras"},{"doc":"- Antikythera itself is regularly deployed to both dev and prod environments in the same way as gears described below.\n- For the following purposes antikythera expects each gear to be deployed at least once per month to both dev and prod environments.\n    - to keep gears and antikythera in sync,\n    - to smoothly phase out deprecated symbols (especially for macros as they require re-compilation and re-deploy of all running gears),\n      without too much communication overhead.","ref":"deployment.html#antikythera-s-deployment-policy","title":"Antikythera's deployment policy - Deployment to Antikythera Instance","type":"extras"},{"doc":"- For automatic deployment, antikythera assumes that all gears' code is managed as follows:\n    - Repository\n        - Every gear implementation is expected to have its own repository in GitHub (dev/prod).\n        - The repository in GitHub must belong to [access-company/Antikythera team](https://github.com/orgs/access-company/teams/antikythera)\n          so that the antikythera team and antikythera's auto-deploy component have read/write access to it.\n    - Branches\n        - Antikythera treats the following 2 branches as deploy targets; the other branches are ignored.\n            - `master` branch means \"ready to deploy to dev environment\".\n            - `release` branch means \"ready to deploy to prod environment\".\n        - Updates in these branches will be automatically deployed to the running antikythera system.","ref":"deployment.html#gear-s-git-repository","title":"Gear's git repository - Deployment to Antikythera Instance","type":"extras"},{"doc":"- Each of dev and prod environments runs its own jenkins server for automatic testing and deployment.\n\n![Deployment Flow](assets/AntikytheraDeploymentFlow.png)\n\n- Note that, in order to run a gear within live antikythera environments, it is necessary for the gear to use the latest version of antikythera.\n    - The antikythera version used by gear is tracked by `mix.lock` file.\n      You can manually update to the latest antikythera version by issuing `$ mix deps.update antikythera`.\n    - If antikythera's auto-deploy component detects that a gear is referring to an outdated antikythera version, then\n        - in dev environment it automatically tries to update the antikythera dependency (i.e. perform git-push to `origin/master`) and try again.\n        - in prod environment: it rejects the gear version.","ref":"deployment.html#deployment-flow","title":"Deployment flow - Deployment to Antikythera Instance","type":"extras"},{"doc":"- Jenkins servers in dev/prod environment contain deployment jobs per existing gear (and a few jobs for antikythera system operations).\n- The settings of these jenkins jobs are pre-configured and should not be altered, aside from the following exception.\n    - If you want to trigger deployment to prod after successful deployment to dev,\n      you can setup \"Git publisher\" of your gear's deploy job so that it automatically propagate changes in `master` to `release`.\n      (This configuration is currently active by default)","ref":"deployment.html#about-jenkins-job-settings","title":"About jenkins job settings - Deployment to Antikythera Instance","type":"extras"},{"doc":"- All the following tests are executed in the same way as normal development environment:\n    - Before deployment: `$ mix test`\n    - After deployment: `$ TEST_MODE=blackbox_dev mix test` or `$ TEST_MODE=blackbox_prod mix test`\n- (See [here](https://hexdocs.pm/antikythera/testing.html) for more about whitebox/blackbox tests)","ref":"deployment.html#tests-executed-by-jenkins-during-deploy-jobs","title":"Tests executed by jenkins during deploy jobs - Deployment to Antikythera Instance","type":"extras"},{"doc":"- If you encounter an error about antikythera's mix configuration during deployment of your gear,\n  it might be caused by changes in `mix.exs` and/or `mix_common.exs` in antikythera repository.\n  Please re-run the deploy job a few times and see if it works.","ref":"deployment.html#troubleshoot-errors-during-deploy","title":"Troubleshoot errors during deploy - Deployment to Antikythera Instance","type":"extras"},{"doc":"# Web Requests Routing\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- Antikythera's web request router consists of the following 2 stages:\n    - Domain-based routing\n    - Path-based routing\n- This process is schematically shown in the following flow diagram:\n\n![Request Handling Flow](assets/RequestHandlingFlow.png)","ref":"routing.html","title":"Web Requests Routing","type":"extras"},{"doc":"- Antikythera hosts multiple running gears and incoming web requests are routed based on the domain part of the URL.\n- When a gear is started, antikythera's routing layer registers the domains of the gear and subsequent requests to the domains will be routed to the gear.\n- Each gear's domain is configurable and defaults to the following subdomain (note that `_`s in gear name are replaced by `-`s):\n    - local environment: ` .localhost:8080` (when you run `$ iex -S mix` in your gear project)\n    - dev   environment: ` .development.example.com`\n    - prod  environment: ` .production.example.com`","ref":"routing.html#domain-based-routing-to-a-specific-gear","title":"Domain-based routing to a specific gear - Web Requests Routing","type":"extras"},{"doc":"- Once an appropriate gear for an web request is determined based on the domain part,\n  antikythera then routes the request to a controller action by matching URL path and HTTP method.\n- Gears must define the routing rules in `web/router.ex`.\n    - Refer to `Antikythera.Router` for details of the routing DSL.","ref":"routing.html#path-based-routing-to-a-specific-controller-action","title":"Path-based routing to a specific controller action - Web Requests Routing","type":"extras"},{"doc":"# Gear-to-Gear (g2g) Communication\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- Antikythera defines an interface for g2g communication.\n  The interface closely resembles the HTTP protocol: it has method, path, body, headers, etc.\n  This introduces a few nice things:\n    - Gear developers can focus on standard HTTP terminology. They can be written/tested in the same way.\n    - The same interface enhances code reuse: e.g. code for web request automatically implements gear request handling.\n- Antikythera routes a g2g request to its target gear's controller action by matching its method and path, as is the case for web request\n  (see also [routing](https://hexdocs.pm/antikythera/routing.html)).\n- You can send request to another gear by using `TargetGear.G2g` module.\n  For usage of `G2g` modules, refer to [test code in testgear](https://github.com/access-company/testgear/tree/master/test/g2g_test.exs).","ref":"g2g.html","title":"Gear-to-Gear (g2g) Communication","type":"extras"},{"doc":"Suppose you are implementing a gear controller action that acts as a simple proxy to another gear,\ni.e., returns a g2g response returned by another gear's action as it is.\nYou would implement the action as follows (assuming JSON encoded body):\n\n```elixir\n# Bad: don't do this\ndef action_xyz(conn) do\n  g2g_res = AnotherGear.G2g.send(conn)\n  json(conn, 200, g2g_res.body)\nend\n```\n\nThis implementation involves unnecessary decoding/encoding of `g2g_res.body` because\n\n- `G2g.send/{1,2}` automatically decodes the response body, and\n- `Antikythera.Conn.json/3` encodes the body.\n\nInstead you should bypass the decoding and re-encoding using `G2g.send_without_decoding/{1,2}`.\n\n```elixir\n# Good\ndef some_action(conn) do\n  g2g_res = AnotherGear.G2g.send_without_decoding(conn)\n  conn\n  |> Conn.put_status(200)\n  |> Conn.put_resp_body(g2g_res.body)\n  |> Conn.put_resp_headers(g2g_res.headers)\nend\n```","ref":"g2g.html#avoiding-unnecessary-body-decoding-encoding","title":"Avoiding unnecessary body decoding/encoding - Gear-to-Gear (g2g) Communication","type":"extras"},{"doc":"# Implementing Controller\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- Controller action must receives a `Antikythera.Conn` struct and returns a `Antikythera.Conn`.\n  In other words, type signature of a controller action must be `acion(Antikythera.Conn.t) :: Antikythera.Conn.t`.\n- What controller action should do is, based on the request information in `Conn` struct,\n  to construct a new `Conn` filled with response information (HTTP status code, body, headers, etc.).\n  Antikythera picks up the response information from the returned `Conn` and sends back to the request sender.\n- For this purpose many utility functions are defined in `Antikythera.Controller`.\n  Those functions are auto-imported when you `use Antikythera.Controller` in your controller module.\n- Controller action must return a response within 10 seconds.\n    - It is recommended that you set the client timeout value to 15 seconds or more. This is because the [executor pool](https://hexdocs.pm/antikythera/executor_pool.html) waits up to 5 seconds and the controller takes up to 10 seconds.\n- Gzip compression of responses is done transparently when a request contains `accept-encoding: gzip` header.","ref":"controller.html","title":"Implementing Controller","type":"extras"},{"doc":"- By default antikythera returns a simple error response to request sender in the following situations:\n    - An error occurs during execution of controller action.\n    - No controller action matches the request's method/path.\n    - Format of the request body does not conform to the value of `content-type` header.\n    - Websocket connections limit is reached.\n- To customize response on error, you can define your gear's custom error handlers by:\n    - Add `YourGear.Controller.Error` module.\n    - Inside the module define the following functions:\n        - `error/2`\n        - `no_route/1`\n        - `bad_request/1`\n        - `bad_executor_pool_id/2` (optional)\n        - `ws_too_many_connections/1` (optional)\n    - These functions must return a `Antikythera.Conn.t` as in regular controller actions.\n- Note that custom error handlers should do as little task as possible to avoid further troubles.","ref":"controller.html#handling-errors","title":"Handling errors - Implementing Controller","type":"extras"},{"doc":"- Before returning an HTTP response antikythera automatically adds the following response headers for security reasons:\n    - `x-frame-options` : `DENY`\n    - `x-xss-protection` : `1; mode=block`\n    - `x-content-type-options` : `nosniff`\n    - `strict-transport-security` : `max-age=31536000`\n- Gear implementations can explicitly set these headers;\n  in this case antikythera respects the value and don't overwrite it.","ref":"controller.html#response-headers","title":"Response headers - Implementing Controller","type":"extras"},{"doc":"# Templates for Dynamic HTML\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- Gears can use [HAML templates](https://haml.info/) to render dynamic HTML.\n- We are using [calliope](https://github.com/nurugger07/calliope) as the HAML engine.\n  Please refer to calliope's documentation for its syntax.\n- To use HAML:\n    - Confirm that `web/template.ex` is present (it's automatically generated by the gear generator).\n    - Place your HAML files under `web/template/`.\n    - HAML templates are processed during `$ mix compile` and content of each file\n      is packed into a clause of `YourGear.Template.content_for/2`.\n    - Render HTML responses by using `render/5` function in your controller action.\n- HTML-escaping in HAML templates\n    - Dynamic parts in HAML templates are automatically HTML-escaped\n      (i.e. special characters are replaced with character entity references).\n    - If you want to insert elements without escaping, use `Antikythera.TemplateSanitizer.raw/1`.\n      `Antikythera.TemplateSanitizer` is automatically `import`ed and thus you can just write\n      e.g. `= raw \" \"` in your HAML templates.\n- Inserting other templates in a template (a.k.a. partial rendering)\n    - You can simply call `content_for/2` (without `YourGear.Template.` prefix) from within your HAML templates.\n      e.g. `= content_for \"path/to/other/template\", param1: value1`","ref":"dynamic_html.html","title":"Templates for Dynamic HTML","type":"extras"},{"doc":"# Websocket\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- [Websocket](https://en.wikipedia.org/wiki/WebSocket) is an extension to HTTP that emulates TCP connections.\n  By using websocket, communication pattern between server and client is no longer restricted to plain HTTP's request-response model.\n- Antikythera supports the websocket protocol to enable gears to interactively communicate with their clients.","ref":"websocket.html","title":"Websocket","type":"extras"},{"doc":"- To use websocket in your gear you need to\n    1. declare a websocket route in `web/router.ex`\n        - The route is for handshake; GET requests that match the path pattern are processed as websocket handshake requests.\n        - See reference of `Antikythera.Router`.\n    2. implement a callback module of `Antikythera.Websocket` behaviour in `web/websocket.ex`\n        - For detailed explanations, see `Antikythera.Websocket`.","ref":"websocket.html#implementing-websocket-enabled-gear","title":"Implementing websocket-enabled gear - Websocket","type":"extras"},{"doc":"- Once a websocket connection is established between antikythera and a client,\n  antikythera allocates a dedicated process for the connection.\n    - Note that the connection process is created on the node that received the handshake request in the cluster.\n    - This connection process handles all incoming/outgoing frames between the server and the client.\n        - To push a frame to the client, you must first send an Erlang message to the connection process.\n          You can use `Antikythera.Registry.Unique` and/or `Antikythera.Registry.Group` for this purpose.\n\n#","ref":"websocket.html#websocket-connection-process","title":"Websocket connection process - Websocket","type":"extras"},{"doc":"- In order to prevent a specific gear from monopolizing resources in the cluster,\n  antikythera imposes an upper limit on number of websocket connections per [executor pool](https://hexdocs.pm/antikythera/executor_pool.html).\n    - During handshake, antikythera calls `YourGear.executor_pool_for_web_request/1` to ask which executor pool this client belongs to.\n    - Trying to connect more than the limit results in an HTTP 503 (Service Unavailable) error.\n        - The error can be customized as you wish; see [handling errors](https://hexdocs.pm/antikythera/controller.html#handling-errors).\n\n#","ref":"websocket.html#connections-limit","title":"Connections limit - Websocket","type":"extras"},{"doc":"- A websocket connection established between an antikythera gear and a client is automatically closed after 60 seconds of idle time.\n- If you want to keep websocket connections during idle time it's recommended that clients periodically send ping frames.\n    - It's also possible that the server sends periodic pings but in most cases\n      it's better to distribute computational costs associated with the timers for large number of clients.\n- You don't have to implement the server to reply with pong frame; ping frames are automatically handled by antikythera.\n\n#","ref":"websocket.html#idle-timeout","title":"Idle timeout - Websocket","type":"extras"},{"doc":"- For maintenance reason, a subset of antikythera servers in the cluster may be shut down at any time.\n  Before it is actually terminated, each target server will automatically send a `close` frame with status code `1001`\n  to all the connected websocket clients.\n  Client implementations should be prepared with these situations by properly reconnecting to other working servers in the cluster.","ref":"websocket.html#disconnection-from-soon-to-be-terminated-server","title":"Disconnection from soon-to-be-terminated server - Websocket","type":"extras"},{"doc":"# Asynchronous Jobs\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\nAntikythera provides gears with a way to asynchronously run arbitrary code.\nThis feature is referred to as \"async job\" throughout documentations of antikythera.\nAsync jobs are executed in background of HTTP request/response interactions.\n\nAsync jobs are useful for tasks such as:\n\n- batch inserts\n- sending many emails\n- download and process files\n- scheduled jobs\n\nAntikythera's async job processing comes with the following features:\n\n- Automatic retry of failed jobs\n- Support of timed jobs (jobs that start at specified times)\n- Support of recurring schedule (using crontab format)\n- Timeout of job execution\n- Load balancing of job executions within ErlangVMs in the cluster\n- Resource capping (implemented in terms of [executor pools](https://hexdocs.pm/antikythera/executor_pool.html))","ref":"async_job.html","title":"Asynchronous Jobs","type":"extras"},{"doc":"To use async job, gear implementation must define a module that `use`s `Antikythera.AsyncJob`.\n\n```elixir\ndefmodule YourGear.SomeAsyncJob do\n  use Antikythera.AsyncJob\n\n  @impl true\n  def run(payload, _job_id, _context) do\n    # put arbitrary code here\n    YourGear.Logger.info(inspect(payload))\n  end\nend\n```\n\nThen invoke `YourGear.SomeAsyncJob.register/3`.\nYou can register new jobs from within both controller actions and async jobs.\nIn your local development environment you can try within `$ iex -S mix`.\n\n```elixir\nYourGear.SomeAsyncJob.register(%{\"arbitrary\" => \"map\"}, {:gear, :your_gear})\n```\n\nInternally, it inserts the job to a job queue for executor pool `{:gear, :your_gear}`,\nthe job queue notifies a broker process of the newly registered job,\nthe broker spawns a worker to run the job,\nand then the following call to `YourGear.SomeAsyncJob.run/3` is evaluated within the worker:\n\n```elixir\nYourGear.SomeAsyncJob.run(%{\"arbitrary\" => \"map\"}, job_metadata, context_for_this_job_execution)\n```\n\nFor details of the async job API please refer to `Antikythera.AsyncJob`.","ref":"async_job.html#usage-example","title":"Usage example - Asynchronous Jobs","type":"extras"},{"doc":"# “Executor Pool” for Resource Management\n\n**Note:** This page is being updated for OSS release. Please be patient.","ref":"executor_pool.html","title":"“Executor Pool” for Resource Management","type":"extras"},{"doc":"Antikythera supports multi-application (multi-gear) and multi-tenant workloads.\nThis means that multiple gears and multiple tenants share the same computing resources in an antikythera cluster.\nIn order to mitigate influences of heavily-loaded gear and/or tenant on the other gears/tenants,\nantikythera controls execution of simultaneously running tasks using \"executor pool\"s.\n\nEach executor pool consists of the following pools of Erlang processes:\n\n- Processes for web request processing\n- Processes for websocket connections\n- Processes for [async job](https://hexdocs.pm/antikythera/async_job.html) execution\n\nNumber of processes in the pools are managed by the antikythera core.\nThese types of pools in an executor pool are independent of each other,\ni.e., web request processing does not affect async job execution.\n\nExecutor pools are classified into the following two categories:\n\n- Gear executor pool:\n    - An executor pool that associates with exactly one gear.\n      This type of executor pools is for e.g.\n        - Gears that do not have multi-tenant use cases,\n        - Gears that do support multi-tenant but need to process tasks which do not belong to any tenants.\n    - Every time a gear starts/stops (i.e. antikythera installs/uninstalls a gear) the gear's executor pool is also created/terminated.\n      Thus each gear can always use its own gear executor pool.\n- Tenant executor pool:\n    - An executor pool that associates with a \"tenant\" which represents a set of request senders.\n      This type of executor pools is intended for gears that provide service to multiple tenants.\n    - TODO: write about life-cycle of tenant executor pools\n    - Note that this type of executor pools can be used by multiple gears.\n\nEach task (e.g. web request processing) is executed by an Erlang process in the executor pool that corresponds to the task;\nthus simultaneous task executions are capped by the number of processes in each executor pool.","ref":"executor_pool.html#purpose","title":"Purpose - “Executor Pool” for Resource Management","type":"extras"},{"doc":"#","ref":"executor_pool.html#how-to-choose-target-executor-pool-for-the-given-task","title":"How to choose target executor pool for the given task - “Executor Pool” for Resource Management","type":"extras"},{"doc":"- To handle an incoming web request, gear implementations must choose an appropriate executor pool for the request.\n    - The target executor pool must be specified by `YourGear.executor_pool_for_web_request/1`\n      which receives a `Antikythera.Conn.t` and returns a `Antikythera.ExecutorPool.Id.t`.\n    - Implementations of `executor_pool_for_web_request/1` must be [pure](https://en.wikipedia.org/wiki/Pure_function);\n      i.e. return value must be computed solely from the given `Antikythera.Conn.t`.\n    - When the executor pool specified by `executor_pool_for_web_request/1` is overloaded and\n      no available executor process found within 5 seconds, the request results in an error.\n- In development environment (more precisely, when `Antikythera.Env.compiling_for_release?() == true`),\n  the returned value from `YourGear.executor_pool_for_web_request/1` is basically neglected and\n  gear executor pool is used for all requests to simplify setting-up of gear development.\n- For gear-to-gear requests it's not necessary to choose an executor pool to use;\n  no additional executor pool is necessary to process gear-to-gear requests.\n\n##","ref":"executor_pool.html#web-requests","title":"Web requests - “Executor Pool” for Resource Management","type":"extras"},{"doc":"1. A gear that does not support multi-tenant use cases:\n  uses its gear executor pool for all requests\n\n    ```elixir\n    def executor_pool_for_web_request(_conn) do\n      {:gear, :your_gear}\n    end\n    ```\n\n2. A gear that serves to multiple tenants:\n  specifies tenant ID included in the request path\n\n    ```elixir\n    def executor_pool_for_web_request(%Conn{request: req}) do\n      {:tenant, req.path_matches[:tenant_id]}\n    end\n    ```\n\n### [Async jobs](https://hexdocs.pm/antikythera/async_job.html)\n\n- When you register your async jobs, you must specify which executor pool to use as arguments to `YourGear.SomeAsyncJob.register/3`.\n  See [API reference](https://hexdocs.pm/antikythera/Antikythera.AsyncJob.html) for detailed explanation.","ref":"executor_pool.html#example-implementations-of-executor_pool_for_web_request-1","title":"Example implementations of `executor_pool_for_web_request/1` - “Executor Pool” for Resource Management","type":"extras"},{"doc":"- After choosing an executor pool for a task (by e.g. `SomeGear.executor_pool_for_web_request/1`),\n  antikythera verifies that the gear is allowed to use the specified executor pool.\n    - The primary purpose of this verification is to protect from DoS attack by requests containing invalid tenant IDs.\n- Available executor pools for a gear are:\n    - the gear executor pool of the gear\n    - tenant executor pools associated with the gear\n- Therefore developer of a gear that uses tenant executor pool must establish associations\n  between his/her gear and target tenant executor pools to use.","ref":"executor_pool.html#associations-between-gears-and-tenant-executor-pools","title":"Associations between gears and tenant executor pools - “Executor Pool” for Resource Management","type":"extras"},{"doc":"- Capacities of executor pools are determined by `AntikytheraCore.ExecutorPool.Setting`.\n- Each field of `AntikytheraCore.ExecutorPool.Setting` means following.\n    - The product of `n_pools_a` and `pool_size_a`\n        - The number of processes for web request processing.\n    - `ws_max_connections`\n        - The number of processes for websocket connections.\n    - `pool_size_j`\n        - The number of processes for [async job](https://hexdocs.pm/antikythera/async_job.html) execution.","ref":"executor_pool.html#managing-capacity-of-executor-pools","title":"Managing capacity of executor pools - “Executor Pool” for Resource Management","type":"extras"},{"doc":"- In addition to capping of simultaneous processes by executor pools,\n  antikythera imposes memory usage limit (up to 400MB on 64bit machine) on each process that runs under an executor pool.\n    - Any process that violates this upper limit will be killed.\n- This limit, together with the limit on number of processes, protects the entire ErlangVM from running out of memory.","ref":"executor_pool.html#limits-for-gear-processes","title":"Limits for gear processes - “Executor Pool” for Resource Management","type":"extras"},{"doc":"# Logging\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- From any part of your gear code, you can put messages to log files.\n  Pass a `String.t` to one of the following functions:\n    - `YourGear.Logger.error/1`\n    - `YourGear.Logger.info/1`\n    - `YourGear.Logger.debug/1`\n- `error`, `info`, `debug` are the log levels that represent severity of each message.\n- You can configure your logger setting to include which levels of messages to be included in your logs.\n  Default setting is `info` (i.e., `error` and `info` are included, `debug` is not included).\n- On your local development environment, you can set your log level by setting `LOG_LEVEL` environment variable.\n- Developers will be notified when any `error` level logs are reported. See [Alerting](https://hexdocs.pm/antikythera/alerting.html) for details.","ref":"logging.html","title":"Logging","type":"extras"},{"doc":"- Information about incoming web requests is automatically logged, as in the following format:\n\n  ```text\n  2016-01-26T00:40:22.557+00:00 [info] context=20160126-004022.557_ip-172-31-5-176_0.684.0 GET /path?query=params from=xxx.xxx.xxx.xxx START encoding=gzip, deflate, sdch\n  2016-01-26T00:40:22.567+00:00 [info] context=20160126-004022.557_ip-172-31-5-176_0.684.0 GET /path?query=params from=xxx.xxx.xxx.xxx END status=200 time=10ms\n  ```\n\n    - `context` field is a context ID of the current request processing, in the form of ` _ _ `.\n    - Query parameters are shown in the URL-decoded form.\n    - `encoding` field is the value of `accept-encoding` request header. This field is shown mainly to check whether the client uses gzip/deflate compression.\n- On start/end of [async job](https://hexdocs.pm/antikythera/async_job.html) executions the following logs are generated:\n\n  ```text\n  2016-08-06T09:17:24.043+00:00 [info] context=20160806-091724.043_ip-172-31-22-164_0.662.0   module=Testgear.TestAsyncJob job_id=P97tnfACMEivaKW-3zey attempt=1th/3 run_at=2016-08-06T09:17:24.042+00:00 START\n  2016-08-06T09:17:24.144+00:00 [info] context=20160806-091724.043_ip-172-31-22-164_0.662.0   module=Testgear.TestAsyncJob job_id=P97tnfACMEivaKW-3zey attempt=1th/3 run_at=2016-08-06T09:17:24.042+00:00 END status=success time=71ms\n  ```\n\n    - `context` is the same as in the case of web requests (see above).\n    - `module` is the name of the module which defines the job.\n    - `attempt` is the number of job executions (including retries) for this job, together with its upper limit.\n    - `run_at` is the start time specified by the job's schedule.\n    - `status` is one of `success`, `failure_retry` and `failure_abandon`.\n    - By providing implementation of `inspect_payload/1` callback, you can additionally include information of `payload`.\n      This can be useful to easily identify jobs in logs.\n      See documentation for `Antikythera.AsyncJob` for detail.\n- Also websocket connected/disconnected events are logged:\n\n  ```text\n  2016-11-09T04:47:19.089+00:00 [info] context=20161109-044719.089_ip-172-31-15-109_0.1164.0   CONNECTED\n  2016-11-09T04:47:19.831+00:00 [info] context=20161109-044719.089_ip-172-31-15-109_0.1164.0   DISCONNECTED connected_at=2016-11-09T04:47:19.089+00:00 frames_received=0 frames_sent=0\n  ```\n\n    - `frames_received` is the number of frames the connection received from the client.\n      Some control frames are automatically handled by antikythera and not included in the number.\n    - `frames_sent` is the number of frames the connection sent to the client.\n      Some control frames are automatically sent by antikythera and not included in the number.\n- In addition to the default log messages explained above, antikythera automatically logs errors which occur during execution of gear code.","ref":"logging.html#auto-generated-log-messages","title":"Auto-generated log messages - Logging","type":"extras"},{"doc":"- **(TBD)**\n- Note that there are multiple running nodes in the antikythera instance and as such the log files are created on a per-node basis.\n- Note also that log files will become downloadable in a few hours:\n    - log files are rotated on every 2 hours, and\n    - rotated logs are uploaded to cloud storage on every 30 minutes.","ref":"logging.html#obtaining-log-files","title":"Obtaining log files - Logging","type":"extras"},{"doc":"# Metrics Reporting\n\n**Note:** This page is being updated for OSS release. Please be patient.\nCurrent contents describes how we are handling our antikythera instance's and gears' metrics data in [ACCESS](https://www.access-company.com).\n\n- In antikythera core and gears, statistical metrics can be generated.\n- Generated metrics are buffered, aggregated, then flushed to stable storage **every minute**.\n- By default, antikythera uses Elasticsearch as its metrics storage,\n  utilizing its search/aggregation feature and visualization with Kibana.\n- All metrics are stored in `metrics-YYYY.MM.DD` indices under `antikythera` subspace.h\n- All metrics documents come with following predefined fields:\n    - `@timestamp`\n    - `node_id`\n    - `otp_app_name`\n        - `antikythera` or gear name\n    - `epool_id`\n        - `gear- ` or `tenant- `\n        - Exists only in [executor pool](https://hexdocs.pm/antikythera/executor_pool.html) related metrics\n- Metrics are stored for 1 month. Older indices will be automatically deleted.","ref":"metrics_reporting.html","title":"Metrics Reporting","type":"extras"},{"doc":"- The following metrics are automatically gathered by antikythera:\n    - Metrics about web/[g2g](https://hexdocs.pm/antikythera/g2g.html) requests\n        - Number of processed requests (`web/g2g_request_count_*`)\n        - Time distribution of response times (`web/g2g_response_time_ms_*`)\n        - Executor pool checkout failure count (`web_timeout_in_epool_checkout_sum`)\n        - Number of working processes for web request handling in an executor pool (`epool_working_action_runner_*`)\n    - Metrics about [websocket](https://hexdocs.pm/antikythera/websocket.html) interactions\n        - Number of active connections (`epool_websocket_connections_*`)\n        - Number of rejected attempts to establish websocket connections (`epool_websocket_rejected_count`)\n        - Number of received/sent websocket frames (`websocket_frames_received/sent`)\n    - Metrics about [async job](https://hexdocs.pm/antikythera/async_job.html)\n        - Number of completed/failed jobs (`async_job_success/failure_sum`)\n        - Time distribution of async job executions (`async_job_execution_time_ms_*`)\n        - Number of waiting jobs in job queue (`epool_waiting_job_count`)\n        - Number of runnable jobs in job queue (`epool_runnable_job_count`)\n        - Number of running jobs in job queue (`epool_running_job_count`)\n        - Number of waiting job brokers (`epool_waiting_broker_count`)\n        - Number of working processes for async job in an executor pool (`epool_working_job_runner_*`)","ref":"metrics_reporting.html#auto-collected-metrics","title":"Auto-collected metrics - Metrics Reporting","type":"extras"},{"doc":"- You can generate and collect arbitrary numeric metrics from your gear's code.\n- Field names for such metrics are prefixed with `custom_`\n  and can be searched/visualized just like auto-generated metrics.\n- Use `YourGear.MetricsUploader.submit/2` from anywhere in your code to generate metrics.\n  `YourGear.MetricsUploader` process will then buffer and aggregate them before uploading.\n    - First argument is metrics data list (`Antikythera.Metrics.DataList.t`).\n        - `strategy` for each data must be an atom representation of one of aggregation strategies.\n          Currently available strategies are:\n            - `:average`\n            - `:sum`\n            - `:time_distribution`\n                - Generates average, max and 95-percentile values\n            - `:gauge`\n                - Takes last value of each time window\n    - Second argument is the context (`Antikythera.Context.t`) from which the currently used executor pool ID will be extracted.","ref":"metrics_reporting.html#custom-metrics","title":"Custom metrics - Metrics Reporting","type":"extras"},{"doc":"- Simply search `metrics-*` indices with filtering on `otp_app_name` field using your gear name.\n- Note that all metrics are aggregated in uploader process **per node**, then uploaded to Elasticsearch.\n  There you will aggregate those metrics **across nodes** (mostly via Kibana query).\n    - This two-phase aggregation provides the entire view of metrics from applications running in multiple nodes,\n      while reducing volume of network traffic.\n    - However, total metrics could be inaccurate, even with a coherent strategy is used.\n      e.g. A sum of per-node sums is an accurate total sum, but an average of per-node averages is not an accurate total average.","ref":"metrics_reporting.html#metrics-search-and-visualization","title":"Metrics search and visualization - Metrics Reporting","type":"extras"},{"doc":"# Alerting\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- When error logs are reported via `YourGear.Logger.error/1` function,\n  the errors are sent to developers as alerts.\n- It is also possible to send alerts without logging,\n  by calling `YourGear.AlertManager.notify/1` function from any part of your gear code.\n- Currently, only email alerting interface is defined as a behaviour `AntikytheraEal.AlertMailer`.\n    - [Slack](https://slack.com) and [Twillio](https://www.twilio.com)\n      are candidates for alternative notification channels to support. Requests and contributions are welcomed.","ref":"alerting.html","title":"Alerting","type":"extras"},{"doc":"- To prevent excessive number of alerts from being sent, alert handlers have a built-in buffering mechanism.\n- Occasional errors will be sent in shorter period of buffering (called `fast_interval`, 1 minute by default).\n- If errors keep occurring beyond `fast_interval`,\n  they will be buffered for longer period (called `delayed_interval`, 30 minutes by default) before sent.\n- These intervals are configured per alert methods.\n- Required and optional settings (such as addresses for email alert), along with intervals above,\n  can be configured in `:alerts` field of [gear config](https://hexdocs.pm/antikythera/gear_config.html).","ref":"alerting.html#alert-behavior-and-configurations","title":"Alert behavior and configurations - Alerting","type":"extras"},{"doc":"# Gear Config\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- In general web services need some kind of \"secret token\"s, such as API key for external services.\n  Also, it's sometimes useful to change configurations on the fly (i.e. without changing source code and without deployment).\n- For these purposes antikythera provides \"gear config\".\n    - You can think of gear config as something like environment variables for gears.\n    - Each gear's gear config is an arbitrary JSON object.","ref":"gear_config.html","title":"Gear Config","type":"extras"},{"doc":"- From gear implementation, gear config is accessble by:\n    - `YourGear.get_all_env/0`: Returns a map of JSON-parsed gear config.\n    - `YourGear.get_env/2`: A convenience to just return a single value for the given key in gear config.\n      Use this when you just want a single value; if you need multiple values use `get_all_env/0` for better performance.","ref":"gear_config.html#getting-gear-config","title":"Getting gear config - Gear Config","type":"extras"},{"doc":"- In cloud (dev/prod environment):\n    - **(TBD)**\n    - After update, new values of gear config become visible from gear code, running within multiple ErlangVM nodes, in about a few minutes.\n- In local machine\n    - Use environment variable to specify gear config for your locally-running antikythera.\n      For instance, set `SOME_GEAR_CONFIG_JSON` for `some_gear` and so on.\n- During tests:\n    - You can use `Antikythera.Test.GearConfigHelper.set_config/1` to manipulate gear config from test code.","ref":"gear_config.html#setting-gear-config","title":"Setting gear config - Gear Config","type":"extras"},{"doc":"# Internationalization\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\nSometimes antikythera gears want to show translated messages to users.\nFor this purpose antikythera comes with Elixir's [gettext](https://github.com/elixir-lang/gettext) package.\nDocumentation for gettext package is available [here](https://hexdocs.pm/gettext/Gettext.html).\n\n[Gettext](https://www.gnu.org/software/gettext/) itself is one of GNU projects, and used in many other languages.","ref":"i18n.html","title":"Internationalization","type":"extras"},{"doc":"- To use gettext you must define `YourGear.Gettext`.\n    - Antikythera's gear generator script automatically sets up `YourGear.Gettext`.\n- Translation files are located in `priv/gettext/${locale}/LC_MESSAGES/${domain}.po`.\n    - `${locale}` is a locale such as `en`, `jp`, etc. Default locale is `en`.\n    - `${domain}` is the domain that can be used with `dgettext` and defaults to `default`.","ref":"i18n.html#preparing-translations","title":"Preparing translations - Internationalization","type":"extras"},{"doc":"- You can also use [`mix gettext.extract`](https://hexdocs.pm/gettext/Mix.Tasks.Gettext.Extract.html#content)\n  and [`mix gettext.merge`](https://hexdocs.pm/gettext/Mix.Tasks.Gettext.Merge.html#content) to manage translation files.\n- As written in Gettext's [README](https://github.com/elixir-lang/gettext#workflow), the basic workflow will be as follows:\n    1. Put `gettext/1,2`, `dgettext/2,3` or other translation **macro** calls in your sources code.\n    2. Run `$ mix gettext.extract`. All messages will be extracted into template files (`priv/gettext/*.pot`).\n        - This is basically an Elixir-equivalent of GNU's [xgettext](https://www.gnu.org/software/gettext/manual/gettext.html#xgettext-Invocation).\n    3. Run `$ mix gettext.merge priv/gettext/ --locale  `. Template files will be merged into translation files.\n        - Similarly, this corresponds to GNU's [msginit](https://www.gnu.org/software/gettext/manual/gettext.html#msginit-Invocation)\n          or [msgmerge](https://www.gnu.org/software/gettext/manual/gettext.html#msgmerge-Invocation).\n        - If the task is run for the first time, translation files will be generated from the template files.\n    4. Add translations in translation files.\n- Notice that you need to put **macro** calls in your code to use these features.\n    - In other words, messages appearing in translation **function** calls (e.g. `Gettext.gettext(YourGear.Gettext, \"Some message\")`),\n      cannot be extracted by the `gettext.extract` task.\n- And importantly, messages passed to macro calls must be **compile-time strings**.\n    - You must use function calls to translate messages that dynamically change on runtime.\n- Consider these trade-offs before deciding whether you use this extract-and-edit workflow, or just manually manage translation files.","ref":"i18n.html#workflow-using-gettext-mix-tasks-and-macros","title":"Workflow using Gettext mix tasks and macros - Internationalization","type":"extras"},{"doc":"- First you have to determine where to receive locale information from your users.\n  Popular options are:\n    - URL path: `/some/site/ja/hello.html`\n    - query parameter: `/some/site/hello.html?locale=ja`\n- Auto-generated `YourGear.Gettext` module comes with `put_locale/1`,\n  which is a convenient wrapper around `Gettext.put_locale/2`.\n  Invoke `put_locale/1` within your controller action as follows (here in the case of query parameter):\n\n  ```elixir\n  YourGear.Gettext.put_locale(conn.request.query_params[\"locale\"] || \"ja\")\n  ```\n\n  The specified locale is stored within the process's process dictionary and will be used in later `*gettext` calls.","ref":"i18n.html#setting-locale-for-each-web-request","title":"Setting locale for each web request - Internationalization","type":"extras"},{"doc":"# Testing\n\n**Note:** This page is being updated for OSS release. Please be patient.","ref":"testing.html","title":"Testing","type":"extras"},{"doc":"- You can write tests for your gear using standard [`ExUnit` test framework](https://hexdocs.pm/ex_unit/ExUnit.html).\n- [`StreamData`](https://github.com/whatyouhide/stream_data) (powerful property-based testing library) is also available out of the box.\n  Consider using property-based style, especially when testing side-effect free functions.\n- For testing purposes antikythera provides some helpers.\n  See modules prefixed with `Antikythera.Test.`.\n  Note that those modules are solely for test code; don't use these modules in your production code.","ref":"testing.html#libraries","title":"Libraries - Testing","type":"extras"},{"doc":"- Antikythera defines the following two ways to run your gear's tests.\n    - Start an ErlangVM to run both production code and test code of your gear.\n      This type of test execution is called **whitebox testing** as test code may touch the internal of the gear implementation.\n    - Start an ErlangVM to run only test code which interacts with an already-running ErlangVM that is executing your gear's production code.\n      The tests in this mode check behavior of the target only through public interface of your gear (e.g. HTTP) and thus it's called **blackbox testing**.\n- Running tests\n    - whitebox tests:\n        - `$ mix test`\n    - blackbox tests:\n        - `$ TEST_MODE=blackbox_local mix test`\n            - Test against locally running ErlangVM; `Antikythera.Test.Config.base_url/0` points to your gear's URL in your local environment.\n        - `$ TEST_MODE=blackbox_dev mix test`\n            - Test against dev environment; `Antikythera.Test.Config.base_url/0` points to your gear's URL in the antikythera dev environment.\n        - `$ TEST_MODE=blackbox_prod mix test`\n            - Test against prod environment; `Antikythera.Test.Config.base_url/0` points to your gear's URL in the antikythera prod environment.\n- You can specify when to run each of your tests by putting [tags](https://hexdocs.pm/ex_unit/ExUnit.Case.html#module-tags).\n    - `@tag :blackbox` : runs during both whitebox and blackbox testing\n    - `@tag :blackbox_only` : runs during blackbox testing; omitted during whitebox testing\n    - (no tag) : runs during whitebox testing; omitted during blackbox testing\n- Recommendation on which tag to use\n    - In general, your tests can be classified based on the following properties:\n        - target interface\n            1. internal (Elixir) function\n            2. interface which users consume, such as HTTP\n        - interaction with external service(s) during test\n            1. do not use any external service\n            2. use stubbed/mocked version of external service(s) to test in isolation\n            3. use real external service(s) for integration testing\n    - Use the following table as a general rule of thumb.\n\n      target interface | external services | whitebox testing | blackbox testing | tag\n      ---------------- | ----------------- | ---------------- | ---------------- | ---\n      a (Elixir)   | a (no)   | O | X | none\n      a (Elixir)   | b (stub) | O | X | none\n      a (Elixir)   | c (yes)  | X | X | (high- and low-level concepts are mixed; don't write this kind of tests!)\n      b (end-user) | a (no)   | O | O | `@blackbox`\n      b (end-user) | b (stub) | O | X | none\n      b (end-user) | c (yes)  | X | O | `@blackbox_only`\n\n- Reference: See also `Antikythera.Test.Config`","ref":"testing.html#whitebox-blackbox-testing-and-test-modes","title":"Whitebox/blackbox testing and test modes - Testing","type":"extras"},{"doc":"- Sometimes tests require certain kind of secret that should not be hardcoded in tests.\n    - e.g. API key for an external service\n- To pass such information to test runner process, antikythera defines `WHITEBOX_TEST_SECRET_JSON` and `BLACKBOX_TEST_SECRET_JSON` environment variable.\n- In your test cases you can obtain contents of the environment variable via\n  `Antikythera.Test.Config.whitebox_test_secret/0` or `Antikythera.Test.Config.blackbox_test_secret/0` which returns JSON-parsed variable.","ref":"testing.html#secret-in-testing","title":"Secret in testing - Testing","type":"extras"},{"doc":"# What Gears Must Not Do\n\n**Note:** This page is being updated for OSS release. Please be patient.\n\n- (Some of the issues listed here are checked by `compile.antikythera_gear_static_analysis` every time you compile your gear's code.)\n- Modules prefixed with `Antikythera` define interface functions to be used by gear implementations.\n  On the other hand, modules with the other prefixes (`AntikytheraCore`, `AntikytheraEal` and `AntikytheraLocal`)\n  are internal to antikythera and not for gear implementations.\n  Gears must not directly depend on these antikythera-internal modules.\n- All modules in a gear must be properly prefixed with the gear name, e.g. `YourGear.SomeModule` or `Mix.Tasks.YourGear.SomeTask`.\n  Modules without correct prefixes are not allowed; they can cause name clashes.\n- Dynamically creating atoms (e.g. make atoms from request bodies) is strictly prohibited as it _is_ memory leak.\n    - Atoms are stored globally within an ErlangVM and are not garbage-collected.\n      Accumulated memory resources occupied by dynamically created atoms can result in memory issue.\n    - Following code in runtime context are considered problematic:\n        - `String.to_atom/1` : use `String.to_existing_atom/1` instead\n        - `Module.concat/1,2` : use `Module.safe_concat/1,2` instead\n        - atom expression with interpolation such as `:\"abc_#{foo}\"`\n- Scheduling of Erlang processes is controlled by antikythera.\n  Gear implementations must not disturb it by manually spawning/terminating processes.\n- Accessing local file system is basically of no practical use, as they will be discarded by changes in underlying infrastructure configuration.\n  However the following use cases are allowed:\n    - Reading files in your gear's `priv` directory.\n      Files in your gear's `priv` directory are directly copied from the git repository during compilation and thus under your control.\n    - Reading/writing files within temporary directory created by `Antikythera.Tmpdir.make/2`.\n- Writing to `STDOUT`/`STDERR` must also be avoided since these streams are used by antikythera and not for a specific gear. Use `YourGear.Logger` instead.","ref":"must_nots.html","title":"What Gears Must Not Do","type":"extras"},{"doc":"# Limitations\n\nThere are certain limitations in Antikythera.\nWe list some major limitations here. But this does not cover all the limitations of Antikythera. Please also read other parts of the documentation when you implement gear.\n\n- Execution time of web/g2g request\n    - Set via `GEAR_ACTION_TIMEOUT` as a default timeout. Please refer [here](https://hexdocs.pm/antikythera/development_environment.html#environment-variables-to-tweak-behavior-of-antikythera).\n    - Also set via `:timeout` option per API. Please refer to `Antikythera.Router`.\n- Execution time of async job\n    - Set via `max_duration`. Please refer [here](https://hexdocs.pm/antikythera/Antikythera.AsyncJob.html#module-registering-jobs).\n    - Async job has other limitations too.\n- Heap size which a process can use\n    - Set via `GEAR_PROCESS_MAX_HEAP_SIZE`. Please refer [here](https://hexdocs.pm/antikythera/development_environment.html#environment-variables-to-tweak-behavior-of-antikythera).\n    - When a process uses more memory than the limit, Antikythera kills the process and outputs error log(e.g. `xxx killed`).","ref":"limitations.html","title":"Limitations","type":"extras"}]}